@@ -1,5 +1,5 @@
 /* admin.c: Primitive functions for gzochi admin API
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -45,10 +45,11 @@ SCM_DEFINE (primitive_applications, "primitive-applications", 0, 0, 0,
   
   while (applications_ptr != NULL)
     {
-      gzochid_application_context *app_context = applications_ptr->data;
+      GzochidApplicationContext *app_context = applications_ptr->data;
       SCM application_record = scm_call_1
 	(scm_make_application_context, 
-	 scm_from_locale_string (app_context->descriptor->name));
+	 scm_from_locale_string (gzochid_application_context_get_name
+				 (app_context)));
 
       ret = scm_cons (application_record, ret);      
       applications_ptr = applications_ptr->next;
@@ -62,14 +63,15 @@ SCM_DEFINE (primitive_applications, "primitive-applications", 0, 0, 0,
 SCM_DEFINE (primitive_current_application, "primitive-current-application", 
 	    0, 0, 0, (), "Returns the current application or #f if not set.")
 {
-  gzochid_application_context *context = 
+  GzochidApplicationContext *context =
     gzochid_get_current_application_context ();
 
   if (context == NULL)
     return SCM_BOOL_F;
   else return scm_call_1
 	 (scm_make_application_context,
-	  scm_from_locale_string (context->descriptor->name));
+	  scm_from_locale_string (gzochid_application_context_get_name
+				  (context)));
 }
 
 /* Wraps `gzochid_transaction_end' in a dynamic wind "unwind handler," so it can
@@ -88,10 +90,10 @@ SCM_DEFINE (primitive_with_application, "primitive-with-application",
 {
   char *name = scm_to_locale_string 
     (scm_call_1 (scm_application_context_name, context));
-  gzochid_application_context *app_context = 
+  GzochidApplicationContext *app_context = 
     gzochid_game_server_lookup_application (game_server, name);
   gzochid_auth_identity *debug_identity =
-    gzochid_auth_identity_from_name (app_context->identity_cache, "[DEBUG]");
+    gzochid_application_context_resolve_identity (app_context, "[DEBUG]");
   gzochid_transaction_timing timing = { { 0, 0 }, NULL };
 
   free (name);
