@@ -1,5 +1,5 @@
 /* session.c: Primitive functions for user-facing gzochid client session API
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ SCM_DEFINE (primitive_send_message, "primitive-send-message", 2, 0, 0,
 	    (SCM session, SCM msg), "Send a message to a client session.")
 {
   GError *err = NULL;
-  gzochid_application_context *context =
+  GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *reference = NULL;
   
@@ -58,7 +58,7 @@ SCM_DEFINE (primitive_disconnect, "primitive-disconnect", 1, 0, 0,
 	    (SCM session), "Disconnect a client session.")
 {
   GError *err = NULL;
-  gzochid_application_context *context =
+  GzochidApplicationContext *context =
     gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *reference = NULL;
   guint64 c_oid = gzochid_scheme_client_session_oid (session);
