@@ -1,5 +1,5 @@
 /* tx.c: Primitive functions for gzochi external transaction API
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 
 struct _gzochid_api_tx_context
 {
-  gzochid_application_context *context; /* The application context. */
+  GzochidApplicationContext *context; /* The application context. */
   gzochid_auth_identity *identity; /* The system identity. */
   SCM participants; /* The cons list of bound participants. */
 };
@@ -196,8 +196,8 @@ SCM_DEFINE (primitive_join_transaction, "primitive-join-transaction", 1, 0, 0,
       tx_context = malloc (sizeof (gzochid_api_tx_context));
 
       tx_context->context = gzochid_api_ensure_current_application_context ();
-      tx_context->identity = gzochid_auth_identity_from_name
-	(tx_context->context->identity_cache, "[SYSTEM]");
+      tx_context->identity = gzochid_application_context_resolve_identity
+	(tx_context->context, "[SYSTEM]");
       tx_context->participants = SCM_EOL;
 
       gzochid_transaction_join (&tx_participant, tx_context);
