@@ -1,5 +1,5 @@
 /* util.c: Shared utility procedures for Scheme->C API.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -32,10 +32,10 @@ static SCM scm_make_transaction_aborted_condition;
 static SCM scm_make_transaction_retry_condition;
 static SCM scm_make_transaction_timeout_condition;
 
-gzochid_application_context *
+GzochidApplicationContext *
 gzochid_api_ensure_current_application_context ()
 {
-  gzochid_application_context *context = 
+  GzochidApplicationContext *context = 
     gzochid_get_current_application_context();
   if (context == NULL)
     gzochid_guile_r6rs_raise 
