@@ -1,5 +1,5 @@
 /* app-task.c: Support for application-bound transactional task execution
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <assert.h>
 #include <glib.h>
+#include <glib-object.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -25,6 +26,7 @@
 #include "app-task.h"
 #include "auth_int.h"
 #include "context.h"
+#include "event.h"
 #include "game.h"
 #include "gzochid-auth.h"
 #include "io.h"
@@ -36,23 +38,46 @@
 
 struct _gzochid_event_transaction_context
 {
-  gzochid_application_context *app_context;
+  GzochidApplicationContext *app_context;
   guint64 start_time;
 };
 
 typedef struct _gzochid_event_transaction_context
 gzochid_event_transaction_context;
 
+gzochid_application_state *
+gzochid_application_state_new ()
+{
+  gzochid_application_state *state =
+    calloc (1, sizeof (gzochid_application_state));
+  
+  g_mutex_init (&state->mutex);
+  g_cond_init (&state->cond);
+  
+  return state;
+}
+
+void
+gzochid_application_state_free (gzochid_application_state *state)
+{
+  g_mutex_clear (&state->mutex);
+  g_cond_clear (&state->cond);
+  
+  free (state);
+}
+
 gzochid_application_task *
-gzochid_application_task_new (gzochid_application_context *context,
+gzochid_application_task_new (GzochidApplicationContext *context,
 			      gzochid_auth_identity *identity, 
-			      gzochid_application_worker worker, gpointer data)
+			      gzochid_application_worker worker,
+			      GDestroyNotify destroy_notify, gpointer data)
 {
   gzochid_application_task *task = malloc (sizeof (gzochid_application_task));
 
   task->context = context;
   task->identity = gzochid_auth_identity_ref (identity);
   task->worker = worker;
+  task->destroy_notify = destroy_notify;
   task->data = data;
 
   task->ref_count = 1;
@@ -143,7 +168,7 @@ transactional_task_worker (gpointer data)
 {
   void **args = data;
   
-  gzochid_application_context *context = args[0];
+  GzochidApplicationContext *context = args[0];
   gzochid_auth_identity *identity = args[1];
   gzochid_application_task *task = args[2];
   
@@ -152,7 +177,7 @@ transactional_task_worker (gpointer data)
 
 void 
 gzochid_application_transactional_task_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gpointer data)
 {
   void *args[3];
@@ -172,17 +197,25 @@ gzochid_application_transactional_task_worker
 }
 
 static gzochid_event_transaction_context *
-create_transaction_context (gzochid_application_context *app_context)
+create_transaction_context (GzochidApplicationContext *app_context)
 {
   gzochid_event_transaction_context *tx_context = 
     malloc (sizeof (gzochid_event_transaction_context));
 
-  tx_context->app_context = app_context;
+  tx_context->app_context = g_object_ref (app_context);
   tx_context->start_time = g_get_monotonic_time ();
 
   return tx_context;
 }
 
+static void
+transaction_context_free (gzochid_event_transaction_context *tx_context)
+{
+  g_object_unref (tx_context->app_context);
+
+  free (tx_context);
+}
+
 static int 
 event_prepare (gpointer data)
 {
@@ -194,14 +227,18 @@ event_commit (gpointer data)
 {
   gzochid_event_transaction_context *tx_context = data;
   guint64 duration_us = g_get_monotonic_time () - tx_context->start_time;
-  
+  gzochid_event_source *event_source = NULL;
+
+  g_object_get (tx_context->app_context, "event-source", &event_source, NULL);
+    
   gzochid_event_dispatch
-    (tx_context->app_context->event_source,
+    (event_source,
      g_object_new (GZOCHID_TYPE_TRANSACTION_EVENT,
 		   "type", TRANSACTION_COMMIT, "duration-us", duration_us,
 		   NULL));
 
-  free (tx_context);
+  g_source_unref ((GSource *) event_source);
+  transaction_context_free (tx_context);
 }
 
 static void 
@@ -209,21 +246,25 @@ event_rollback (gpointer data)
 {
   gzochid_event_transaction_context *tx_context = data;
   guint64 duration_us = g_get_monotonic_time () - tx_context->start_time;
-  
+  gzochid_event_source *event_source = NULL;
+
+  g_object_get (tx_context->app_context, "event-source", &event_source, NULL);
+
   gzochid_event_dispatch
-    (tx_context->app_context->event_source,
+    (event_source,
      g_object_new (GZOCHID_TYPE_TRANSACTION_EVENT,
 		   "type", TRANSACTION_ROLLBACK, "duration-us", duration_us,
 		   NULL));
 
-  free (tx_context);
+  g_source_unref ((GSource *) event_source);
+  transaction_context_free (tx_context);
 }
 
 static gzochid_transaction_participant 
 event_participant = { "event", event_prepare, event_commit, event_rollback };
 
 static void 
-join_transaction (gzochid_application_context *context)
+join_transaction (GzochidApplicationContext *context)
 {
   if (!gzochid_transaction_active ()
       || gzochid_transaction_context (&event_participant) == NULL)
@@ -238,21 +279,26 @@ static void
 event_func_wrapper (gpointer data)
 {
   gpointer *args = data;
-  gzochid_application_context *context = args[0];
+  GzochidApplicationContext *context = args[0];
   void (*func) (gpointer) = (void (*) (gpointer)) args[1];
   gpointer func_data = args[2];
-  
+
+  gzochid_event_source *event_source = NULL;
+
   join_transaction (context);
 
+  g_object_get (context, "event-source", &event_source, NULL);
+  
   gzochid_event_dispatch
-    (context->event_source,
-     g_object_new (GZOCHID_TYPE_EVENT, "type", TRANSACTION_START, NULL));
-     
+    (event_source, g_object_new
+     (GZOCHID_TYPE_EVENT, "type", TRANSACTION_START, NULL));
+
+  g_source_unref ((GSource *) event_source);
   func (func_data);  
 }
 
 gzochid_transaction_result 
-gzochid_application_transaction_execute (gzochid_application_context *context, 
+gzochid_application_transaction_execute (GzochidApplicationContext *context, 
 					 void (*func) (gpointer), gpointer data)
 {
   gpointer args[3];
@@ -266,7 +312,7 @@ gzochid_application_transaction_execute (gzochid_application_context *context,
 
 gzochid_transaction_result 
 gzochid_application_transaction_execute_timed 
-(gzochid_application_context *context, void (*func) (gpointer), gpointer data,
+(GzochidApplicationContext *context, void (*func) (gpointer), gpointer data,
  struct timeval timeout)
 {
   gpointer args[3];
@@ -280,7 +326,7 @@ gzochid_application_transaction_execute_timed
 
 void 
 gzochid_application_reexecuting_transactional_task_worker 
-(gzochid_application_context *app_context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *app_context, gzochid_auth_identity *identity, 
  gpointer data)
 {
   gzochid_transactional_application_task_execution *execution = data;
@@ -306,21 +352,20 @@ gzochid_application_reexecuting_transactional_task_worker
       
       gzochid_application_reexecuting_transactional_task_worker
 	(app_context, identity, catch_execution);
+
+      gzochid_transactional_application_task_execution_free (catch_execution);
     }
 
   if (execution->cleanup_task != NULL)
     execution->cleanup_task->worker
       (app_context, identity, execution->cleanup_task->data);
-
-  gzochid_transactional_application_task_execution_free (execution);
 }
 
 void 
 gzochid_application_resubmitting_transactional_task_worker 
-(gzochid_application_context *app_context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *app_context, gzochid_auth_identity *identity, 
  gpointer data)
 {
-  gboolean resubmitting_execution = FALSE;
   gzochid_application_task *application_task = NULL;
   gzochid_transactional_application_task_execution *execution = data;
 
@@ -329,13 +374,19 @@ gzochid_application_resubmitting_transactional_task_worker
   
   if (gzochid_application_should_retry (execution))
     {
-      resubmitting_execution = TRUE;
-      execution->result = GZOCHID_TRANSACTION_PENDING;
+      gzochid_transactional_application_task_execution *retry_execution = 
+	gzochid_transactional_application_task_execution_new
+	(execution->task, execution->catch_task, execution->cleanup_task);
+
+      retry_execution->attempts = execution->attempts;
+      retry_execution->max_attempts = execution->max_attempts;
+      retry_execution->result = GZOCHID_TRANSACTION_PENDING;
       
       application_task = gzochid_application_task_new
 	(app_context, identity,
-	 gzochid_application_resubmitting_transactional_task_worker, 
-	 execution);	  
+	 gzochid_application_resubmitting_transactional_task_worker,
+	 (GDestroyNotify) gzochid_transactional_application_task_execution_free,
+	 retry_execution);
     }
   else if (execution->result != GZOCHID_TRANSACTION_SUCCESS
 	   && execution->catch_task != NULL)
@@ -349,26 +400,22 @@ gzochid_application_resubmitting_transactional_task_worker
       application_task = gzochid_application_task_new
 	(app_context, identity,
 	 gzochid_application_resubmitting_transactional_task_worker,
+	 (GDestroyNotify) gzochid_transactional_application_task_execution_free,
 	 catch_execution);
     }
   else if (execution->cleanup_task != NULL)
     application_task = gzochid_application_task_ref (execution->cleanup_task);
 
-  if (!resubmitting_execution)
-
-    /* Freeing the execution has the effect of unref'ing the primary task. */
-      
-    gzochid_transactional_application_task_execution_free (execution);
-
   if (application_task != NULL)
     {
-      gzochid_task task;
+      GzochidTask *task = gzochid_task_immediate_new
+	(gzochid_application_task_thread_worker,
+	 (GDestroyNotify) gzochid_application_task_unref, application_task);
+      GzochidTaskQueue *task_queue = NULL;
 
-      task.worker = gzochid_application_task_thread_worker;
-      task.data = application_task;
-      gettimeofday (&task.target_execution_time, NULL);
-      
-      gzochid_schedule_submit_task (app_context->task_queue, &task);
+      g_object_get (app_context, "task-queue", &task_queue, NULL);
+      gzochid_schedule_submit_task (task_queue, task);
+      g_object_unref (task_queue);
     }
 }
 
@@ -376,6 +423,10 @@ void
 gzochid_application_task_free (gzochid_application_task *task)
 {
   gzochid_auth_identity_unref (task->identity);
+
+  if (task->destroy_notify)
+    task->destroy_notify (task->data);
+  
   free (task);
 }
 
@@ -384,7 +435,6 @@ gzochid_application_task_thread_worker (gpointer data, gpointer user_data)
 {
   gzochid_application_task *task = data;
   task->worker (task->context, task->identity, task->data);
-  gzochid_application_task_unref (task);
 }
 
 gboolean 
