@@ -1,5 +1,5 @@
 /* callback.c: Constructors and serialization routines for application callbacks
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "util.h"
 
 static void 
-serialize_callback (gzochid_application_context *context, gpointer data, 
+serialize_callback (GzochidApplicationContext *context, gpointer data, 
 		    GByteArray *out, GError **err)
 {
   gzochid_application_callback *callback = data;
@@ -39,7 +39,7 @@ serialize_callback (gzochid_application_context *context, gpointer data,
 }
 
 static gpointer 
-deserialize_callback (gzochid_application_context *context, GByteArray *in, 
+deserialize_callback (GzochidApplicationContext *context, GByteArray *in, 
 		      GError **err)
 {
   gzochid_application_callback *callback = 
@@ -55,7 +55,7 @@ deserialize_callback (gzochid_application_context *context, GByteArray *in,
 }
 
 static void
-finalize_callback (gzochid_application_context *context, gpointer data)
+finalize_callback (GzochidApplicationContext *context, gpointer data)
 {
   gzochid_application_callback_free (data);
 }
