@@ -1,5 +1,5 @@
 /* channel.c: Channel management routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,12 +17,14 @@
 
 #include <assert.h>
 #include <glib.h>
+#include <glib-object.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>
 
 #include "app.h"
+#include "app-session.h"
 #include "app-task.h"
 #include "auth_int.h"
 #include "channel.h"
@@ -114,7 +116,7 @@ gzochid_channel_name (gzochid_channel *channel)
 /* Serialization routines for channel objects. */
 
 static gpointer 
-deserialize_channel (gzochid_application_context *context, GByteArray *in,
+deserialize_channel (GzochidApplicationContext *context, GByteArray *in,
 		     GError **err)
 {
   gzochid_channel *channel = calloc (1, sizeof (gzochid_channel));
@@ -128,7 +130,7 @@ deserialize_channel (gzochid_application_context *context, GByteArray *in,
 }
 
 static void 
-serialize_channel (gzochid_application_context *context, gpointer obj,
+serialize_channel (GzochidApplicationContext *context, gpointer obj,
 		   GByteArray *out, GError **err)
 {
   gzochid_channel *channel = obj;
@@ -138,8 +140,8 @@ serialize_channel (gzochid_application_context *context, gpointer obj,
   gzochid_util_serialize_oid (channel->scm_oid, out);
 }
 
-static void finalize_channel
-(gzochid_application_context *context, gpointer obj)
+static void
+finalize_channel (GzochidApplicationContext *context, gpointer obj)
 {
   gzochid_channel_free (obj);
 }
@@ -192,7 +194,7 @@ gzochid_channel_pending_membership_operation;
 /* Serialization routines for logical channel operations. */
 
 static gpointer 
-deserialize_channel_operation (gzochid_application_context *context,
+deserialize_channel_operation (GzochidApplicationContext *context,
 			       GByteArray *in, GError **err)
 {
   enum gzochid_channel_operation type = gzochid_util_deserialize_int (in);
@@ -232,8 +234,8 @@ deserialize_channel_operation (gzochid_application_context *context,
 }
 
 static void 
-serialize_channel_operation (gzochid_application_context *context,
-			     gpointer data, GByteArray *out, GError **err)
+serialize_channel_operation (GzochidApplicationContext *context, gpointer data,
+			     GByteArray *out, GError **err)
 {
   gzochid_channel_pending_operation *op = data;
 
@@ -261,7 +263,7 @@ serialize_channel_operation (gzochid_application_context *context,
 }
 
 static void
-finalize_channel_operation (gzochid_application_context *context, gpointer data)
+finalize_channel_operation (GzochidApplicationContext *context, gpointer data)
 {
   gzochid_channel_pending_operation *op = data;
 
@@ -435,8 +437,8 @@ gzochid_channel_message_side_effect_new (enum gzochid_channel_operation op,
 struct _gzochid_channel_side_effect_transaction_context 
 {
   /* The gzochi game application context to which the channel belongs. */
-  
-  gzochid_application_context *app_context;
+
+  GzochidApplicationContext *app_context;
 
   /* The channel side effect to be executed on commit. There can only be one
      side effect per side effect transaction. */
@@ -451,8 +453,11 @@ static void
 cleanup_side_effect_transaction
 (gzochid_channel_side_effect_transaction_context *tx_context)
 {
+  g_object_unref (tx_context->app_context);
+  
   if (tx_context->side_effect != NULL)
     free_side_effect (tx_context->side_effect);
+
   free (tx_context);
 }
 
@@ -466,19 +471,22 @@ channel_side_effect_prepare (gpointer data)
 }
 
 void
-gzochid_channel_join_direct (gzochid_application_context *app_context,
+gzochid_channel_join_direct (GzochidApplicationContext *app_context,
 			     guint64 channel_oid, guint64 session_oid,
 			     GError **err)
 {
   GSequence *sessions = NULL;
+  gzochid_application_session_map *session_map = NULL;
 
-  g_mutex_lock (&app_context->channel_mapping_lock);
-  g_mutex_lock (&app_context->client_mapping_lock);
+  g_object_get (app_context, "session-map", &session_map, NULL);
+  
+  g_mutex_lock (&session_map->channel_mapping_lock);
+  g_mutex_lock (&session_map->client_mapping_lock);
   
   sessions = g_hash_table_lookup
-    (app_context->channel_oids_to_local_session_oids, &channel_oid);
+    (session_map->channel_oids_to_local_session_oids, &channel_oid);
 
-  if (!g_hash_table_contains (app_context->oids_to_clients, &session_oid))
+  if (!g_hash_table_contains (session_map->oids_to_clients, &session_oid))
     g_set_error
       (err, GZOCHID_CHANNEL_ERROR, GZOCHID_CHANNEL_ERROR_NOT_MAPPED,
        "No local client bound to oid %" G_GUINT64_FORMAT, session_oid);
@@ -492,7 +500,7 @@ gzochid_channel_join_direct (gzochid_application_context *app_context,
       g_sequence_append (sessions, g_memdup (&session_oid, sizeof (guint64)));
       
       g_hash_table_insert
-	(app_context->channel_oids_to_local_session_oids,
+	(session_map->channel_oids_to_local_session_oids,
 	 g_memdup (&channel_oid, sizeof (guint64)), sessions);
     }
   else
@@ -512,26 +520,30 @@ gzochid_channel_join_direct (gzochid_application_context *app_context,
 	      G_GUINT64_FORMAT, session_oid, channel_oid);
     }
 
-  g_mutex_unlock (&app_context->client_mapping_lock);
-  g_mutex_unlock (&app_context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->client_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 void
-gzochid_channel_leave_direct (gzochid_application_context *app_context,
+gzochid_channel_leave_direct (GzochidApplicationContext *app_context,
 			      guint64 channel_oid, guint64 session_oid,
 			      GError **err)
-{  
-  g_mutex_lock (&app_context->channel_mapping_lock);
-  g_mutex_lock (&app_context->client_mapping_lock);
+{
+  gzochid_application_session_map *session_map = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
   
-  if (!g_hash_table_contains (app_context->oids_to_clients, &session_oid))
+  g_mutex_lock (&session_map->channel_mapping_lock);
+  g_mutex_lock (&session_map->client_mapping_lock);
+  
+  if (!g_hash_table_contains (session_map->oids_to_clients, &session_oid))
     g_set_error
       (err, GZOCHID_CHANNEL_ERROR, GZOCHID_CHANNEL_ERROR_NOT_MAPPED,
        "No local client bound to oid %" G_GUINT64_FORMAT, session_oid);
   else
     {
       GSequence *sessions = g_hash_table_lookup
-	(app_context->channel_oids_to_local_session_oids, &channel_oid);
+	(session_map->channel_oids_to_local_session_oids, &channel_oid);
       gboolean found_session = FALSE;
   
       if (sessions != NULL)
@@ -548,7 +560,7 @@ gzochid_channel_leave_direct (gzochid_application_context *app_context,
 		  g_sequence_get_end_iter (sessions))
 		
 		g_hash_table_remove
-		  (app_context->channel_oids_to_local_session_oids,
+		  (session_map->channel_oids_to_local_session_oids,
 		   &channel_oid);
 	    }
 	}
@@ -560,20 +572,24 @@ gzochid_channel_leave_direct (gzochid_application_context *app_context,
 	   G_GUINT64_FORMAT, session_oid, channel_oid);
     }
 
-  g_mutex_unlock (&app_context->client_mapping_lock);
-  g_mutex_unlock (&app_context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->client_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 void
-gzochid_channel_close_direct (gzochid_application_context *app_context,
+gzochid_channel_close_direct (GzochidApplicationContext *app_context,
 			      guint64 channel_oid)
 {
-  g_mutex_lock (&app_context->channel_mapping_lock);
+  gzochid_application_session_map *session_map = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
+  
+  g_mutex_lock (&session_map->channel_mapping_lock);
 
   g_hash_table_remove
-    (app_context->channel_oids_to_local_session_oids, &channel_oid);
+    (session_map->channel_oids_to_local_session_oids, &channel_oid);
 
-  g_mutex_unlock (&app_context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 /*
@@ -589,27 +605,36 @@ gzochid_channel_close_direct (gzochid_application_context *app_context,
 */
 
 static void
-send_channel_message_direct (gzochid_application_context *app_context,
+send_channel_message_direct (GzochidApplicationContext *app_context,
 			     guint64 channel_oid, GArray *session_oids,
 			     const unsigned char *msg, size_t len)
 {
   int i = 0;
-  GSequence *sessions = g_hash_table_lookup
-    (app_context->channel_oids_to_local_session_oids, &channel_oid);
+  gzochid_application_session_map *session_map = NULL;
+  GSequence *sessions = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
+  
+  sessions = g_hash_table_lookup
+    (session_map->channel_oids_to_local_session_oids, &channel_oid);
   
   for (; i < session_oids->len; i++)
     { 
       guint64 session_oid = g_array_index (session_oids, guint64, i);
       gzochid_game_client *client = g_hash_table_lookup
-	(app_context->oids_to_clients, &session_oid);
+	(session_map->oids_to_clients, &session_oid);
       
       /* Grab the client connection to which the session oid corresponds. */
 	      
       if (client != NULL)
 	{
+	  gzochid_event_source *event_source = NULL;
+
+	  g_object_get (app_context, "event-source", &event_source, NULL);
 	  gzochid_event_dispatch
-	    (app_context->event_source,
+	    (event_source,
 	     g_object_new (GZOCHID_TYPE_EVENT, "type", MESSAGE_SENT, NULL));
+	  g_source_unref ((GSource *) event_source);
 	  
 	  gzochid_game_client_send (client, msg, len);
 	}
@@ -634,7 +659,7 @@ send_channel_message_direct (gzochid_application_context *app_context,
 		  g_sequence_get_end_iter (sessions))
 		
 		g_hash_table_remove
-		  (app_context->channel_oids_to_local_session_oids,
+		  (session_map->channel_oids_to_local_session_oids,
 		   &channel_oid);
 	    }
 	}
@@ -652,18 +677,21 @@ append_session_oid (gpointer data, gpointer user_data)
 }
 
 void
-gzochid_channel_message_direct (gzochid_application_context *app_context,
+gzochid_channel_message_direct (GzochidApplicationContext *app_context,
 				guint64 channel_oid, GBytes *msg_bytes)
 {
   size_t data_len = 0;
   const unsigned char *data = g_bytes_get_data (msg_bytes, &data_len);
   GSequence *sessions = NULL;
+  gzochid_application_session_map *session_map = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
   
-  g_mutex_lock (&app_context->channel_mapping_lock);
-  g_mutex_lock (&app_context->client_mapping_lock);
+  g_mutex_lock (&session_map->channel_mapping_lock);
+  g_mutex_lock (&session_map->client_mapping_lock);
 
   sessions = g_hash_table_lookup
-    (app_context->channel_oids_to_local_session_oids, &channel_oid);
+    (session_map->channel_oids_to_local_session_oids, &channel_oid);
 
   if (sessions != NULL &&
       g_sequence_get_begin_iter (sessions) !=
@@ -682,8 +710,8 @@ gzochid_channel_message_direct (gzochid_application_context *app_context,
       g_array_free (session_oids, TRUE);
     }
 
-  g_mutex_unlock (&app_context->client_mapping_lock);
-  g_mutex_unlock (&app_context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->client_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 /* Makes the channel side effect permanent with respect to the current state of
@@ -701,20 +729,27 @@ channel_side_effect_commit (gpointer data)
       /* If there is a meta client, get its channel client preemptively, since
 	 we're likely to need it below and the `g_object_get' invocation is
 	 verbose. */
-      
-      if (tx_context->app_context->metaclient != NULL)
-	g_object_get
-	  (tx_context->app_context->metaclient,
-	   "channel-client", &channelclient,
-	   NULL);
+
+      GzochidMetaClient *metaclient = gzochid_application_context_get_metaclient
+	(tx_context->app_context);
+
+      if (metaclient != NULL)
+	{
+	  g_object_get (metaclient, "channel-client", &channelclient, NULL);
+	  g_object_unref (metaclient);
+	}
       
       if (tx_context->side_effect->op == GZOCHID_CHANNEL_OP_SEND)
 	{
 	  gzochid_channel_message_side_effect *message_side_effect =
 	    (gzochid_channel_message_side_effect *) tx_context->side_effect;
+	  gzochid_application_session_map *session_map = NULL;
 
-	  g_mutex_lock (&tx_context->app_context->channel_mapping_lock);
-	  g_mutex_lock (&tx_context->app_context->client_mapping_lock);
+	  g_object_get
+	    (tx_context->app_context, "session-map", &session_map, NULL);
+	  
+	  g_mutex_lock (&session_map->channel_mapping_lock);
+	  g_mutex_lock (&session_map->client_mapping_lock);
 	  
 	  send_channel_message_direct
 	    (tx_context->app_context, tx_context->side_effect->channel_oid,
@@ -730,14 +765,15 @@ channel_side_effect_commit (gpointer data)
 		(message_side_effect->msg, message_side_effect->len);
 
 	      gzochid_channelclient_relay_message_from
-		(channelclient, tx_context->app_context->descriptor->name,
+		(channelclient,
+		 gzochid_application_context_get_name (tx_context->app_context),
 		 tx_context->side_effect->channel_oid, msg);
 
 	      g_bytes_unref (msg);
 	    }
 	  
-	  g_mutex_unlock (&tx_context->app_context->client_mapping_lock);
-	  g_mutex_unlock (&tx_context->app_context->channel_mapping_lock);
+	  g_mutex_unlock (&session_map->client_mapping_lock);
+	  g_mutex_unlock (&session_map->channel_mapping_lock);
 	}
       else if (tx_context->side_effect->op == GZOCHID_CHANNEL_OP_CLOSE)
 	{
@@ -750,7 +786,8 @@ channel_side_effect_commit (gpointer data)
 	       the channel locally. */
 	    
 	    gzochid_channelclient_relay_close_from
-	      (channelclient, tx_context->app_context->descriptor->name,
+	      (channelclient,
+	       gzochid_application_context_get_name (tx_context->app_context),
 	       tx_context->side_effect->channel_oid);
 	}
       else
@@ -777,7 +814,9 @@ channel_side_effect_commit (gpointer data)
 		       the node to which the session's connected. */
 		    
 		    gzochid_channelclient_relay_join_from
-		      (channelclient, tx_context->app_context->descriptor->name,
+		      (channelclient,
+		       gzochid_application_context_get_name
+		       (tx_context->app_context),
 		       tx_context->side_effect->channel_oid,
 		       membership_side_effect->session_oid);
 		  
@@ -801,7 +840,9 @@ channel_side_effect_commit (gpointer data)
 		       it to the node to which the session's connected. */
 		    
 		    gzochid_channelclient_relay_leave_from
-		      (channelclient, tx_context->app_context->descriptor->name,
+		      (channelclient,
+		       gzochid_application_context_get_name
+		       (tx_context->app_context),
 		       tx_context->side_effect->channel_oid,
 		       membership_side_effect->session_oid);
 
@@ -841,7 +882,7 @@ static gzochid_transaction_participant channel_side_effect_participant =
    transactional state. */
 
 static gzochid_channel_side_effect_transaction_context *
-join_side_effect_transaction (gzochid_application_context *app_context)
+join_side_effect_transaction (GzochidApplicationContext *app_context)
 {
   gzochid_channel_side_effect_transaction_context *tx_context = NULL;
 
@@ -852,7 +893,7 @@ join_side_effect_transaction (gzochid_application_context *app_context)
       tx_context =
 	malloc (sizeof (gzochid_channel_side_effect_transaction_context));
       
-      tx_context->app_context = app_context;
+      tx_context->app_context = g_object_ref (app_context);
       tx_context->side_effect = NULL;
   
       gzochid_transaction_join (&channel_side_effect_participant, tx_context);
@@ -866,15 +907,18 @@ join_side_effect_transaction (gzochid_application_context *app_context)
 /* Close the target channel as a side effect of the current transaction. */
 
 static void
-close_channel (gzochid_application_context *context,
+close_channel (GzochidApplicationContext *context,
 	       gzochid_auth_identity *identity, gpointer data)
 {
   gzochid_channel_pending_operation *op = data;
+  gzochid_application_session_map *session_map = NULL;
 
-  g_mutex_lock (&context->channel_mapping_lock);
+  g_object_get (context, "session-map", &session_map, NULL);
+
+  g_mutex_lock (&session_map->channel_mapping_lock);
 
   if (g_hash_table_contains
-      (context->channel_oids_to_local_session_oids, &op->target_channel))
+      (session_map->channel_oids_to_local_session_oids, &op->target_channel))
     {
       gzochid_channel_side_effect_transaction_context *tx_context =
 	join_side_effect_transaction (context);
@@ -883,30 +927,35 @@ close_channel (gzochid_application_context *context,
 	(GZOCHID_CHANNEL_OP_CLOSE, op->target_channel);
     }
   
-  g_mutex_unlock (&context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 /* Send a message to the target channel's members as a side effect of the 
    current transaction. */
 
 static void
-send_channel_message (gzochid_application_context *context,
+send_channel_message (GzochidApplicationContext *context,
 		      gzochid_auth_identity *identity, gpointer data)
 {
+  GzochidMetaClient *metaclient =
+    gzochid_application_context_get_metaclient (context);
   gzochid_channel_pending_send_operation *send_op = data;
   gzochid_channel_pending_operation *op =
     (gzochid_channel_pending_operation *) send_op;
+  gzochid_application_session_map *session_map = NULL;
 
-  g_mutex_lock (&context->channel_mapping_lock);
+  g_object_get (context, "session-map", &session_map, NULL);
+
+  g_mutex_lock (&session_map->channel_mapping_lock);
 
   if (g_hash_table_contains
-      (context->channel_oids_to_local_session_oids, &op->target_channel)
-      || context->metaclient != NULL)
+      (session_map->channel_oids_to_local_session_oids, &op->target_channel)
+      || metaclient != NULL)
     {
       gzochid_channel_side_effect_transaction_context *tx_context =
 	join_side_effect_transaction (context);
       GSequence *sessions = g_hash_table_lookup
-	(context->channel_oids_to_local_session_oids, &op->target_channel);
+	(session_map->channel_oids_to_local_session_oids, &op->target_channel);
       GArray *session_oids = g_array_new (FALSE, FALSE, sizeof (guint64));
 
       if (sessions != NULL)
@@ -929,31 +978,36 @@ send_channel_message (gzochid_application_context *context,
 	 send_op->message, send_op->len);
 	  
       g_array_unref (session_oids);
+
+      if (metaclient != NULL)
+	g_object_unref (metaclient);
     }
   
-  g_mutex_unlock (&context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 }
 
 /* Add the target session to the target channel as a side effect of the current
    transaction. */
 
 static void
-join_channel (gzochid_application_context *context,
+join_channel (GzochidApplicationContext *context,
 	      gzochid_auth_identity *identity, gpointer data)
 {
   gzochid_channel_pending_membership_operation *member_op = data;
   gzochid_channel_pending_operation *op = 
     (gzochid_channel_pending_operation *) member_op;
-
+  gzochid_application_session_map *session_map = NULL;
+  
   gboolean do_join = TRUE;
 
-  g_mutex_lock (&context->channel_mapping_lock);
+  g_object_get (context, "session-map", &session_map, NULL);
+  g_mutex_lock (&session_map->channel_mapping_lock);
 
   if (g_hash_table_contains
-      (context->channel_oids_to_local_session_oids, &op->target_channel))
+      (session_map->channel_oids_to_local_session_oids, &op->target_channel))
     {
       GSequence *sessions = g_hash_table_lookup
-	(context->channel_oids_to_local_session_oids, &op->target_channel);
+	(session_map->channel_oids_to_local_session_oids, &op->target_channel);
       GSequenceIter *iter = g_sequence_lookup
 	(sessions, &member_op->target_session,
 	 gzochid_util_guint64_data_compare, NULL);
@@ -961,7 +1015,7 @@ join_channel (gzochid_application_context *context,
       do_join = iter == NULL;
     }
 
-  g_mutex_unlock (&context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 
   if (do_join)
     {
@@ -978,22 +1032,24 @@ join_channel (gzochid_application_context *context,
    current transaction. */
 
 static void
-leave_channel (gzochid_application_context *context,
+leave_channel (GzochidApplicationContext *context,
 	       gzochid_auth_identity *identity, gpointer data)
 {
   gzochid_channel_pending_membership_operation *member_op = data;
   gzochid_channel_pending_operation *op = 
     (gzochid_channel_pending_operation *) member_op;
-
+  gzochid_application_session_map *session_map = NULL;
+  
   gboolean do_leave = FALSE;
 
-  g_mutex_lock (&context->channel_mapping_lock);
+  g_object_get (context, "session-map", &session_map, NULL);
+  g_mutex_lock (&session_map->channel_mapping_lock);
 
   if (g_hash_table_contains
-      (context->channel_oids_to_local_session_oids, &op->target_channel))
+      (session_map->channel_oids_to_local_session_oids, &op->target_channel))
     {
       GSequence *sessions = g_hash_table_lookup
-	(context->channel_oids_to_local_session_oids, &op->target_channel);
+	(session_map->channel_oids_to_local_session_oids, &op->target_channel);
       GSequenceIter *iter = g_sequence_lookup
 	(sessions, &member_op->target_session,
 	 gzochid_util_guint64_data_compare, NULL);
@@ -1001,7 +1057,7 @@ leave_channel (gzochid_application_context *context,
       do_leave = iter != NULL;
     }
 
-  g_mutex_unlock (&context->channel_mapping_lock);
+  g_mutex_unlock (&session_map->channel_mapping_lock);
 
   if (do_leave)
     {
@@ -1021,7 +1077,7 @@ struct _gzochid_channel_transaction_context
 {
   /* The gzochi game application context to which the channel belongs. */
   
-  gzochid_application_context *context;
+  GzochidApplicationContext *app_context;
 
   /* A managed reference to the queue of pending channel operations to be 
      scheduled. */
@@ -1038,6 +1094,25 @@ channel_prepare (gpointer data)
   return TRUE;
 }
 
+static void
+channel_cleanup (gzochid_channel_transaction_context *tx_context)
+{
+  g_object_unref (tx_context->app_context);
+  free (tx_context);
+}
+
+static void
+channel_commit (gpointer data)
+{
+  channel_cleanup (data);
+}
+
+static void
+channel_rollback (gpointer data)
+{
+  channel_cleanup (data);
+}
+
 /* The logical channel operation transaction participant doesn't actually need
    specialized commit and rollback processes; the queue of operations is 
    scheduled and persisted at the beginning of the transaction and can be 
@@ -1048,10 +1123,10 @@ channel_prepare (gpointer data)
    transational services. */
 
 static gzochid_transaction_participant channel_participant =
-  { "channel", channel_prepare, free, free };
+  { "channel", channel_prepare, channel_commit, channel_rollback };
 
 static gzochid_channel_transaction_context *
-join_transaction (gzochid_application_context *context)
+join_transaction (GzochidApplicationContext *context)
 {
   gzochid_channel_transaction_context *tx_context = NULL;
   if (!gzochid_transaction_active()
@@ -1069,7 +1144,7 @@ join_transaction (gzochid_application_context *context)
 	{
 	  tx_context = malloc (sizeof (gzochid_channel_transaction_context));
 
-	  tx_context->context = context;
+	  tx_context->app_context = g_object_ref (context);
 	  tx_context->queue_ref = queue_ref;
 
 	  gzochid_transaction_join (&channel_participant, tx_context);
@@ -1109,7 +1184,7 @@ make_channel_binding (char *name)
 }
 
 gzochid_channel *
-gzochid_channel_create (gzochid_application_context *context, char *name)
+gzochid_channel_create (GzochidApplicationContext *context, char *name)
 {
   GError *err = NULL;
   char *binding = NULL;
@@ -1164,7 +1239,7 @@ gzochid_channel_create (gzochid_application_context *context, char *name)
 }
 
 gzochid_channel *
-gzochid_channel_get (gzochid_application_context *context, char *name)
+gzochid_channel_get (GzochidApplicationContext *context, char *name)
 {
   char *binding = make_channel_binding (name);
   gzochid_channel *channel = gzochid_data_get_binding
@@ -1179,7 +1254,7 @@ gzochid_channel_get (gzochid_application_context *context, char *name)
    per-operation worker functions. */
 
 static void
-channel_operation_worker (gzochid_application_context *context,
+channel_operation_worker (GzochidApplicationContext *context,
 			  gzochid_auth_identity *identity, gpointer data)
 {
   gzochid_channel_pending_operation *op = data;
@@ -1216,14 +1291,14 @@ channel_operation_worker (gzochid_application_context *context,
    transactional tasks. */
 			  
 static void
-serialize_channel_operation_worker (gzochid_application_context *context,
+serialize_channel_operation_worker (GzochidApplicationContext *context,
 				    gzochid_application_worker worker,
 				    GByteArray *out)
 {
 }
 
 static gzochid_application_worker
-deserialize_channel_operation_worker (gzochid_application_context *context,
+deserialize_channel_operation_worker (GzochidApplicationContext *context,
 				      GByteArray *in)
 {
   return channel_operation_worker;
@@ -1245,7 +1320,7 @@ gzochid_channel_operation_task_serialization =
   };
 
 void
-gzochid_channel_join (gzochid_application_context *context,
+gzochid_channel_join (GzochidApplicationContext *context,
 		      gzochid_channel *channel, 
 		      gzochid_client_session *session)
 {
@@ -1288,8 +1363,8 @@ gzochid_channel_join (gzochid_application_context *context,
   /* Create a task to execute the join... */
 
   app_task = gzochid_application_task_new
-     (context, gzochid_auth_system_identity (), channel_operation_worker,
-      operation);
+    (context, gzochid_auth_system_identity (), channel_operation_worker, NULL,
+     operation);
   
   task_handle = gzochid_create_durable_application_task_handle
     (app_task, &gzochid_channel_operation_task_serialization,
@@ -1314,7 +1389,7 @@ gzochid_channel_join (gzochid_application_context *context,
 }
 
 void
-gzochid_channel_leave (gzochid_application_context *context,
+gzochid_channel_leave (GzochidApplicationContext *context,
 		       gzochid_channel *channel, 
 		       gzochid_client_session *session)
 {
@@ -1354,7 +1429,7 @@ gzochid_channel_leave (gzochid_application_context *context,
   /* Create a task to execute the exit from the channel... */
 
   app_task = gzochid_application_task_new
-    (context, gzochid_auth_system_identity (), channel_operation_worker,
+    (context, gzochid_auth_system_identity (), channel_operation_worker, NULL,
      operation);
   
   task_handle = gzochid_create_durable_application_task_handle
@@ -1380,7 +1455,7 @@ gzochid_channel_leave (gzochid_application_context *context,
 }
 
 void
-gzochid_channel_send (gzochid_application_context *context,
+gzochid_channel_send (GzochidApplicationContext *context,
 		      gzochid_channel *channel, unsigned char *message,
 		      short len)
 {
@@ -1415,7 +1490,7 @@ gzochid_channel_send (gzochid_application_context *context,
   /* Create a task to execute the message broadcast... */
 
   app_task = gzochid_application_task_new
-    (context, gzochid_auth_system_identity (), channel_operation_worker,
+    (context, gzochid_auth_system_identity (), channel_operation_worker, NULL,
      operation);    
   
   task_handle = gzochid_create_durable_application_task_handle
@@ -1441,7 +1516,7 @@ gzochid_channel_send (gzochid_application_context *context,
 }
 
 void
-gzochid_channel_close (gzochid_application_context *context,
+gzochid_channel_close (GzochidApplicationContext *context,
 		       gzochid_channel *channel)
 {
   gzochid_data_managed_reference *channel_reference = NULL;
@@ -1467,7 +1542,7 @@ gzochid_channel_close (gzochid_application_context *context,
   /* Create a task to execute the channel shutdown... */
 
   app_task = gzochid_application_task_new
-    (context, gzochid_auth_system_identity (), channel_operation_worker,
+    (context, gzochid_auth_system_identity (), channel_operation_worker, NULL,
      operation);
   
   task_handle = gzochid_create_durable_application_task_handle
