@@ -1,5 +1,5 @@
 /* data-protocol.h: Prototypes and declarations for data-protocol.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -75,7 +75,8 @@ typedef struct _gzochid_data_change gzochid_data_change;
 struct _gzochid_data_changeset
 {
   char *app; /* The requesting application name. */
-
+  guint64 id; /* The connection-unique changeset id. */
+  
   /* An array of `gzochid_data_change' structs. */
   
   GArray *changes;
@@ -163,17 +164,18 @@ void gzochid_data_protocol_response_write
 gzochid_data_response *gzochid_data_protocol_response_read (GBytes *);
 
 /* Create and return a new changeset with the specified gzochi game application
-   name and change array. */
+   name, connection-unique id, and change array. */
 
-gzochid_data_changeset *gzochid_data_changeset_new (const char *, GArray *);
+gzochid_data_changeset *gzochid_data_changeset_new (const char *, guint64,
+						    GArray *);
 
 /* Create and return a new changeset with the specified gzochi game application
-   name and change array. The `GDestroyNotify' function, if provided, will be
-   applied to each element in the change array when 
+   name, connection-unique id, and change array. The `GDestroyNotify' function,
+   if provided, will be applied to each element in the change array when 
    `gzochid_data_changeset_free' is called.*/
 
 gzochid_data_changeset *gzochid_data_changeset_new_with_free_func
-(const char *, GArray *, GDestroyNotify);
+(const char *, guint64, GArray *, GDestroyNotify);
 
 /* Free the specified changeset. */
 
@@ -189,13 +191,19 @@ void gzochid_data_changeset_free (gzochid_data_changeset *);
   The format of each change is:
 
   `NULL'-terminated string: Name of the target store
+  8 bytes: The big-endian encoding of the connection-unique changeset id
   2 bytes: The big-endian encoding of the length of the key; key bytes follow
   2 bytes: The big-endian encoding of the length of the value
     Two zeros indicates a deletion; else the data follows
- */
 
-void gzochid_data_protocol_changeset_write
-(gzochid_data_changeset *, GByteArray *);
+  This function will refuse to write a payload larger than the maximum message
+  size to the target byte array. The function will return the number of changes
+  that were successfully written; when this number is less than the total number
+  of changes, the changeset must be split across multiple messages.
+*/
+
+size_t gzochid_data_protocol_changeset_write (gzochid_data_changeset *,
+					      GByteArray *);
 
 /*
   Deserialize and return a changeset from the specified byte buffer, or return
