@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,12 +17,14 @@
 
 #include <assert.h>
 #include <glib.h>
+#include <glib-object.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "app.h"
+#include "app-store.h"
 #include "data.h"
 #include "game.h"
 #include "gzochid-storage.h"
@@ -36,21 +38,18 @@
 #endif /* G_LOG_DOMAIN */
 #define G_LOG_DOMAIN "gzochid.data"
 
-struct _gzochid_data_oid_block_state
-{
-  guint64 first;
-  guint64 next;
-  guint64 last;  
-};
-
-typedef struct _gzochid_data_oid_block_state gzochid_data_oid_block_state;
-
 struct _gzochid_data_transaction_context
 {
-  gzochid_application_context *context;
+  GzochidApplicationContext *app_context;
 
-  gzochid_data_oid_block_state *free_oids;
-  GList *used_oid_blocks;
+  /* The next available oid for objects created by this transaction. */
+
+  guint64 next_oid; 
+
+  /* The last oid that can be assigned by this transaction before allocating a
+     fresh block of oids. */
+
+  guint64 last_oid; 
 
   gzochid_storage_transaction *transaction;
 
@@ -84,20 +83,33 @@ gzochid_oid_holder_free (gzochid_oid_holder *holder)
   free (holder);
 }
 
+/* Convenience function for fetching the application store pointer from the
+   application context. */
+
+static gzochid_application_store *
+get_store (GzochidApplicationContext *app_context)
+{
+  gzochid_application_store *store = NULL;
+
+  g_object_get (app_context, "app-store", &store, NULL);
+  return store;
+}
+
 static gzochid_data_transaction_context *
-create_transaction_context (gzochid_application_context *app_context,
+create_transaction_context (GzochidApplicationContext *app_context,
 			    struct timeval *timeout)
 {
   gzochid_data_transaction_context *tx_context = 
     calloc (1, sizeof (gzochid_data_transaction_context));
+  gzochid_application_store *store = get_store (app_context);
 
-  tx_context->context = app_context;
+  tx_context->app_context = g_object_ref (app_context);
 
   if (timeout != NULL)
-    tx_context->transaction = app_context->storage_engine_interface
-      ->transaction_begin_timed (app_context->storage_context, *timeout);
-  else tx_context->transaction = app_context->storage_engine_interface
-	 ->transaction_begin (app_context->storage_context);
+    tx_context->transaction = store->iface->transaction_begin_timed
+      (store->storage_context, *timeout);
+  else tx_context->transaction = store->iface->transaction_begin
+	 (store->storage_context);
 
   tx_context->oids_to_references = g_hash_table_new_full 
     (g_int64_hash, g_int64_equal, g_free, NULL);
@@ -107,27 +119,11 @@ create_transaction_context (gzochid_application_context *app_context,
   return tx_context;
 }
 
-static gzochid_data_oid_block_state *
-create_oid_block_state (gzochid_data_oids_block *block)
-{
-  gzochid_data_oid_block_state *block_state =
-    malloc (sizeof (gzochid_data_oid_block_state));
-
-  block_state->first = block->block_start;
-  block_state->next = block->block_start;
-  block_state->last = block_state->first + block->block_size - 1;
-  
-  return block_state;
-}
-
 static void 
 transaction_context_free (gzochid_data_transaction_context *context)
-{  
-  if (context->free_oids != NULL)
-    free (context->free_oids);
-
-  g_list_free_full (context->used_oid_blocks, free);
-
+{
+  g_object_unref (context->app_context);
+  
   g_hash_table_destroy (context->oids_to_references);
   g_hash_table_destroy (context->ptrs_to_references);
 
@@ -141,8 +137,10 @@ flush_reference (gzochid_data_managed_reference *reference,
   GByteArray *out = NULL;
   GError *err = NULL;
 
+  gzochid_application_store *store = get_store (context->app_context);
+  gzochid_event_source *event_source = NULL;
   guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
-  
+
   switch (reference->state)
     {
     case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY:
@@ -159,7 +157,7 @@ flush_reference (gzochid_data_managed_reference *reference,
 	       reference->oid);
 
       reference->serialization->serializer
-	(context->context, reference->obj, out, &err);
+	(context->app_context, reference->obj, out, &err);
 
       if (err != NULL || gzochid_transaction_rollback_only ())
 	{
@@ -168,22 +166,23 @@ flush_reference (gzochid_data_managed_reference *reference,
 	  return FALSE;
 	}
 
-      context->context->storage_engine_interface->transaction_put
-	(context->transaction, context->context->oids,
-	 (char *) &encoded_oid, sizeof (guint64),
-	 (char *) out->data, out->len);
+      store->iface->transaction_put
+	(context->transaction, store->oids, (char *) &encoded_oid,
+	 sizeof (guint64), (char *) out->data, out->len);
 
+      g_object_get (context->app_context, "event-source", &event_source, NULL);
+      
       /* Need to cast `out->len' to `guint64' to support environments in which 
 	 `guint' is less than 64 bits. */
       
       gzochid_event_dispatch
-	(context->context->event_source,
-	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
-		       "type", BYTES_WRITTEN,
-		       "bytes", (guint64) out->len,
-		       NULL));
+	(event_source, g_object_new (GZOCHID_TYPE_DATA_EVENT,
+				     "type", BYTES_WRITTEN,
+				     "bytes", (guint64) out->len,
+				     NULL));
       
       g_byte_array_unref (out);
+      g_source_unref ((GSource *) event_source);
       
       if (context->transaction->rollback)
 	return FALSE;
@@ -205,6 +204,7 @@ data_prepare (gpointer data)
   GList *references = g_hash_table_get_values (context->oids_to_references);
   GList *reference_ptr = references;
   gboolean flush_failed = FALSE;
+  gzochid_application_store *store = get_store (context->app_context);
 
   while (reference_ptr != NULL)
     {
@@ -221,8 +221,7 @@ data_prepare (gpointer data)
   if (flush_failed)
     return FALSE;
 
-  context->context->storage_engine_interface
-    ->transaction_prepare (context->transaction);
+  store->iface->transaction_prepare (context->transaction);
 
   if (context->transaction->rollback)
     return FALSE;
@@ -230,6 +229,15 @@ data_prepare (gpointer data)
   return TRUE;
 }
 
+/* Cleanup function for managed references. */
+
+static void
+free_reference (gzochid_data_managed_reference *reference)
+{
+  g_object_unref (reference->context);
+  free (reference);
+}
+
 static void 
 finalize_references (gzochid_data_transaction_context *context)
 {
@@ -243,22 +251,22 @@ finalize_references (gzochid_data_transaction_context *context)
       if (reference->obj != NULL)
 	{
 	  reference->serialization->finalizer 
-	    (context->context, reference->obj);
+	    (context->app_context, reference->obj);
 	  reference->obj = NULL;
 	}
       reference_ptr = reference_ptr->next;
     }
   
-  g_list_free_full (references, free);
+  g_list_free_full (references, (GDestroyNotify) free_reference);
 }
 
 static void 
 data_commit (gpointer data)
 {
   gzochid_data_transaction_context *context = data;
+  gzochid_application_store *store = get_store (context->app_context);
 
-  context->context->storage_engine_interface
-    ->transaction_commit (context->transaction);
+  store->iface->transaction_commit (context->transaction);
 
   finalize_references (context);
   transaction_context_free (context);
@@ -275,8 +283,8 @@ data_rollback (gpointer data)
 
   if (context != NULL)
     {
-      context->context->storage_engine_interface
-	->transaction_rollback (context->transaction);
+      gzochid_application_store *store = get_store (context->app_context);      
+      store->iface->transaction_rollback (context->transaction);
 
       finalize_references (context);
       transaction_context_free (context);
@@ -286,46 +294,14 @@ data_rollback (gpointer data)
 static gzochid_transaction_participant data_participant = 
   { "data", data_prepare, data_commit, data_rollback };
 
-static gzochid_data_oid_block_state *
-reserve_oids (gzochid_application_context *context)
-{
-  gzochid_data_oid_block_state *block_state = NULL;
-  
-  g_mutex_lock (&context->free_oids_lock);
-  
-  if (g_list_length (context->free_oid_blocks) > 0)
-    {
-      block_state = context->free_oid_blocks->data;
-      context->free_oid_blocks = g_list_remove_link 
-	(context->free_oid_blocks, context->free_oid_blocks);
-    }
-  else
-    {
-      GError *err = NULL;
-      gzochid_data_oids_block new_block;
-
-      if (gzochid_oids_reserve_block (context->oid_strategy, &new_block, &err))
-	block_state = create_oid_block_state (&new_block);
-      else
-	{
-	  g_debug ("Failed to reserve oids: %s", err->message);
-	  gzochid_transaction_mark_for_rollback
-	    (&data_participant, err->code == GZOCHID_OIDS_ERROR_TRANSACTION);
-	  g_error_free (err);
-	}
-    }
-
-  g_mutex_unlock (&context->free_oids_lock);
-  return block_state;
-}
-
 static guint64
 get_binding (gzochid_data_transaction_context *context, char *name,
 	     GError **err)
 {
   size_t oid_bytes_len = 0;
-  char *oid_bytes = context->context->storage_engine_interface->transaction_get 
-    (context->transaction, context->context->names, name, strlen (name) + 1, 
+  gzochid_application_store *store = get_store (context->app_context);
+  char *oid_bytes = store->iface->transaction_get 
+    (context->transaction, store->names, name, strlen (name) + 1,
      &oid_bytes_len);
 
   assert (oid_bytes == NULL || oid_bytes_len == sizeof (guint64));
@@ -365,9 +341,10 @@ set_binding (gzochid_data_transaction_context *context, char *name, guint64 oid,
 	     GError **err)
 {
   guint64 encoded_oid = gzochid_util_encode_oid (oid);
-  
-  context->context->storage_engine_interface->transaction_put 
-    (context->transaction, context->context->names, name, strlen (name) + 1, 
+  gzochid_application_store *store = get_store (context->app_context);
+
+  store->iface->transaction_put 
+    (context->transaction, store->names, name, strlen (name) + 1, 
      (char *) &encoded_oid, sizeof (guint64));
 
   if (context->transaction->rollback)
@@ -381,13 +358,13 @@ set_binding (gzochid_data_transaction_context *context, char *name, guint64 oid,
 }
 
 static gzochid_data_managed_reference *
-create_empty_reference (gzochid_application_context *context, guint64 oid, 
+create_empty_reference (GzochidApplicationContext *context, guint64 oid, 
 			gzochid_io_serialization *serialization)
 {
   gzochid_data_managed_reference *reference = 
     calloc (1, sizeof (gzochid_data_managed_reference));
 
-  reference->context = context;
+  reference->context = g_object_ref (context);
   reference->oid = oid;
   reference->state = GZOCHID_MANAGED_REFERENCE_STATE_EMPTY;
   reference->serialization = serialization;
@@ -398,28 +375,23 @@ create_empty_reference (gzochid_application_context *context, guint64 oid,
 static gboolean
 next_object_id (gzochid_data_transaction_context *context, guint64 *oid,
 		GError **err)
-{
-  if (context->free_oids == NULL)
-    context->free_oids = reserve_oids (context->context);
-
-  if (context->free_oids == NULL)
-    {
-      g_set_error
-	(err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION,
-	 "Failed to compute next object id.");
-      return FALSE;
-    }
+{  
+  gboolean first_allocation = context->next_oid == 0 && context->last_oid == 0;
+  gboolean needs_more_oids = context->next_oid > context->last_oid;
   
-  *oid = context->free_oids->next;
-  context->free_oids->next += 1;
-
-  if (context->free_oids->next > context->free_oids->last)
+  if (first_allocation || needs_more_oids)
     {
-      context->used_oid_blocks = g_list_append 
-	(context->used_oid_blocks, context->free_oids);
-      context->free_oids = reserve_oids (context->context);
+      gzochid_data_oids_block block;
+      gzochid_application_store *store = NULL;
 
-      if (context->free_oids == NULL)
+      g_object_get (context->app_context, "app-store", &store, NULL);
+
+      if (gzochid_oids_reserve_block (store->oid_strategy, &block, NULL))
+	{
+	  context->next_oid = block.block_start;
+	  context->last_oid = block.block_start + block.block_size - 1;
+	}
+      else
 	{
 	  g_set_error
 	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION,
@@ -427,19 +399,22 @@ next_object_id (gzochid_data_transaction_context *context, guint64 *oid,
 	  return FALSE;
 	}
     }
+  
+  *oid = context->next_oid++;
 
   return TRUE;
 }
 
 gzochid_data_managed_reference *
-create_new_reference (gzochid_application_context *context, void *ptr, 
+create_new_reference (GzochidApplicationContext *context, void *ptr, 
 		      gzochid_io_serialization *serialization, GError **err)
 {
   guint64 oid;
   GError *local_err = NULL;
+  gzochid_application_store *store = get_store (context);
   gzochid_data_transaction_context *tx_context = 
     gzochid_transaction_context (&data_participant);
-    
+  
   if (!next_object_id (tx_context, &oid, &local_err))
     {
       g_propagate_error (err, local_err);
@@ -460,6 +435,7 @@ create_new_reference (gzochid_application_context *context, void *ptr,
 	calloc (1, sizeof (gzochid_data_managed_reference));
       guint64 encoded_oid;
 
+      reference->context = g_object_ref (tx_context->app_context);
       reference->oid = oid;
       reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NEW;
       reference->serialization = serialization;
@@ -467,9 +443,9 @@ create_new_reference (gzochid_application_context *context, void *ptr,
 
       encoded_oid = gzochid_util_encode_oid (reference->oid);
       
-      context->storage_engine_interface->transaction_get_for_update
-	(tx_context->transaction, tx_context->context->oids,
-	 (char *) &encoded_oid, sizeof (guint64), NULL);
+      store->iface->transaction_get_for_update
+	(tx_context->transaction, store->oids, (char *) &encoded_oid,
+	 sizeof (guint64), NULL);
 
       if (tx_context->transaction->rollback)	
 	{
@@ -479,7 +455,7 @@ create_new_reference (gzochid_application_context *context, void *ptr,
 	    (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_TRANSACTION, 
 	     "Transaction rolled back while creating new reference.");
 
-	  free (reference);
+	  free_reference (reference);
 	  return NULL;
 	}
       else return reference;
@@ -494,7 +470,7 @@ create_new_reference (gzochid_application_context *context, void *ptr,
 }
 
 static gzochid_data_managed_reference *
-get_reference_by_oid (gzochid_application_context *context, guint64 oid, 
+get_reference_by_oid (GzochidApplicationContext *context, guint64 oid, 
 		      gzochid_io_serialization *serialization)
 {
   gzochid_data_transaction_context *tx_context = 
@@ -514,7 +490,7 @@ get_reference_by_oid (gzochid_application_context *context, guint64 oid,
 }
 
 static gzochid_data_managed_reference *
-get_reference_by_ptr (gzochid_application_context *context, void *ptr, 
+get_reference_by_ptr (GzochidApplicationContext *context, void *ptr, 
 		      gzochid_io_serialization *serialization, GError **err)
 {
   gzochid_data_transaction_context *tx_context = 
@@ -551,6 +527,8 @@ dereference (gzochid_data_transaction_context *context,
   char *data = NULL; 
   GByteArray *in = NULL;
   guint64 encoded_oid;
+  GzochidApplicationContext *app_context = context->app_context;
+  gzochid_application_store *store = get_store (app_context);
   
   if (reference->obj != NULL
       || reference->state == GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY)
@@ -560,7 +538,7 @@ dereference (gzochid_data_transaction_context *context,
       if (for_update &&
 	  reference->state == GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED)
 	gzochid_data_mark
-	  (context->context, reference->serialization, reference->obj, err);
+	  (app_context, reference->serialization, reference->obj, err);
       
       return;
     }
@@ -571,12 +549,11 @@ dereference (gzochid_data_transaction_context *context,
   encoded_oid = gzochid_util_encode_oid (reference->oid);
   
   if (for_update)
-    data = context->context->storage_engine_interface
-      ->transaction_get_for_update
-      (context->transaction, context->context->oids, (char *) &encoded_oid,
+    data = store->iface->transaction_get_for_update
+      (context->transaction, store->oids, (char *) &encoded_oid,
        sizeof (guint64), &data_len);
-  else data = context->context->storage_engine_interface->transaction_get
-	 (context->transaction, context->context->oids, (char *) &encoded_oid, 
+  else data = store->iface->transaction_get
+	 (context->transaction, store->oids, (char *) &encoded_oid, 
 	  sizeof (guint64), &data_len);
 
   if (context->transaction->rollback)
@@ -595,21 +572,26 @@ dereference (gzochid_data_transaction_context *context,
   else 
     {
       GError *local_err = NULL;
+      gzochid_event_source *event_source = NULL;
 
+      g_object_get (app_context, "event-source", &event_source, NULL);
+      
       /* Need to cast `data_len' to `guint64' since `size_t' may be a different
 	 size on 32-bit platforms. */
       
       gzochid_event_dispatch
-	(context->context->event_source,
+	(event_source,
 	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
 		       "type", BYTES_READ, "bytes", (guint64) data_len, NULL));
+
+      g_source_unref ((GSource *) event_source);
       
       in = g_byte_array_sized_new (data_len);
       g_byte_array_append (in, (unsigned char *) data, data_len);
       free (data);
 
       reference->obj = reference->serialization->deserializer 
-	(context->context, in, &local_err);
+	(app_context, in, &local_err);
 
       if (local_err != NULL)
 	g_propagate_error (err, local_err);	  
@@ -634,7 +616,8 @@ remove_object (gzochid_data_transaction_context *context, guint64 oid,
 	       GError **err)
 {
   int ret = 0;
-
+  gzochid_application_store *store = get_store (context->app_context);
+  
   /* The transaction is presumed to have already been joined at this point. */
   
   gzochid_data_transaction_context *tx_context =
@@ -643,8 +626,8 @@ remove_object (gzochid_data_transaction_context *context, guint64 oid,
   guint64 encoded_oid = gzochid_util_encode_oid (oid);
   
   g_debug ("Removing reference '%" G_GUINT64_FORMAT "'.", oid);
-  ret = context->context->storage_engine_interface->transaction_delete 
-    (context->transaction, context->context->oids, (char *) &encoded_oid, 
+  ret = store->iface->transaction_delete 
+    (context->transaction, store->oids, (char *) &encoded_oid,
      sizeof (guint64));
 
   if (ret == GZOCHID_STORAGE_ENOTFOUND)
@@ -690,7 +673,7 @@ remove_reference (gzochid_data_transaction_context *context,
 }
 
 static gzochid_data_transaction_context *
-join_transaction (gzochid_application_context *context, GError **err)
+join_transaction (GzochidApplicationContext *context, GError **err)
 {
   gzochid_data_transaction_context *tx_context = NULL;
 
@@ -719,7 +702,7 @@ join_transaction (gzochid_application_context *context, GError **err)
 }
 
 void *
-gzochid_data_get_binding (gzochid_application_context *context, char *name, 
+gzochid_data_get_binding (GzochidApplicationContext *context, char *name, 
 			  gzochid_io_serialization *serialization, GError **err)
 {
   guint64 oid;
@@ -749,7 +732,7 @@ gzochid_data_get_binding (gzochid_application_context *context, char *name,
 }
 
 void 
-gzochid_data_set_binding_to_oid (gzochid_application_context *context, 
+gzochid_data_set_binding_to_oid (GzochidApplicationContext *context, 
 				 char *name, guint64 oid, GError **err)
 {
   GError *local_err = NULL;
@@ -762,11 +745,12 @@ gzochid_data_set_binding_to_oid (gzochid_application_context *context,
 }
 
 char *
-gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
+gzochid_data_next_binding_oid (GzochidApplicationContext *context, char *key,
 			       guint64 *oid, GError **err)
 {
   char *next_key = NULL;
   GError *local_err = NULL;
+  gzochid_application_store *store = get_store (context);
   gzochid_data_transaction_context *tx_context = 
     join_transaction (context, &local_err);
 
@@ -776,9 +760,8 @@ gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
       return NULL;
     }
 
-  next_key = context->storage_engine_interface->transaction_next_key 
-    (tx_context->transaction, tx_context->context->names, key, 
-     strlen (key) + 1, NULL);
+  next_key = store->iface->transaction_next_key 
+    (tx_context->transaction, store->names, key, strlen (key) + 1, NULL);
   if (tx_context->transaction->rollback)
     gzochid_transaction_mark_for_rollback 
       (&data_participant, tx_context->transaction->should_retry);
@@ -801,7 +784,7 @@ gzochid_data_next_binding_oid (gzochid_application_context *context, char *key,
 }
 
 void 
-gzochid_data_set_binding (gzochid_application_context *context, char *name, 
+gzochid_data_set_binding (GzochidApplicationContext *context, char *name, 
 			  gzochid_io_serialization *serialization, void *data, 
 			  GError **err)
 {
@@ -824,11 +807,12 @@ gzochid_data_set_binding (gzochid_application_context *context, char *name,
 }
 
 void 
-gzochid_data_remove_binding (gzochid_application_context *context, char *name, 
+gzochid_data_remove_binding (GzochidApplicationContext *context, char *name, 
 			     GError **err)
 {
   int ret = 0;
   GError *local_err = NULL;
+  gzochid_application_store *store = get_store (context);
   gzochid_data_transaction_context *tx_context = 
     join_transaction (context, &local_err);
 
@@ -838,9 +822,8 @@ gzochid_data_remove_binding (gzochid_application_context *context, char *name,
       return;
     }
 
-  ret = context->storage_engine_interface->transaction_delete 
-    (tx_context->transaction, tx_context->context->names, name, 
-     strlen (name) + 1);
+  ret = store->iface->transaction_delete 
+    (tx_context->transaction, store->names, name, strlen (name) + 1);
 
   if (ret == GZOCHID_STORAGE_ENOTFOUND)
     g_set_error (err, GZOCHID_DATA_ERROR, GZOCHID_DATA_ERROR_NOT_FOUND, 
@@ -856,7 +839,7 @@ gzochid_data_remove_binding (gzochid_application_context *context, char *name,
 }
 
 gboolean
-gzochid_data_binding_exists (gzochid_application_context *context, char *name, 
+gzochid_data_binding_exists (GzochidApplicationContext *context, char *name, 
 			     GError **err)
 {
   gboolean ret = TRUE;
@@ -885,8 +868,8 @@ gzochid_data_binding_exists (gzochid_application_context *context, char *name,
 }
 
 gzochid_data_managed_reference *gzochid_data_create_reference
-(gzochid_application_context *context, 
- gzochid_io_serialization *serialization, void *ptr, GError **err)
+(GzochidApplicationContext *context, gzochid_io_serialization *serialization,
+ void *ptr, GError **err)
 {
   GError *local_err = NULL;
   join_transaction (context, &local_err);
@@ -900,7 +883,7 @@ gzochid_data_managed_reference *gzochid_data_create_reference
 }
 
 gzochid_data_managed_reference *
-gzochid_data_create_reference_to_oid (gzochid_application_context *context, 
+gzochid_data_create_reference_to_oid (GzochidApplicationContext *context, 
 				      gzochid_io_serialization *serialization, 
 				      guint64 oid)
 {
@@ -960,7 +943,7 @@ gzochid_data_remove_object (gzochid_data_managed_reference *reference,
 }
 
 void 
-gzochid_data_mark (gzochid_application_context *context, 
+gzochid_data_mark (GzochidApplicationContext *context, 
 		   gzochid_io_serialization *serialization, void *ptr, 
 		   GError **err)
 {
@@ -983,14 +966,15 @@ gzochid_data_mark (gzochid_application_context *context,
     {
       char *data = NULL;
       guint64 encoded_oid = gzochid_util_encode_oid (reference->oid);
+      gzochid_application_store *store = get_store (context);
       
       reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
       g_debug ("Marking reference '%" G_GUINT64_FORMAT "' for update.",
 	       reference->oid);
 
-      data = context->storage_engine_interface->transaction_get_for_update
-	(tx_context->transaction, tx_context->context->oids,
-	 (char *) &encoded_oid, sizeof (guint64), NULL);
+      data = store->iface->transaction_get_for_update
+	(tx_context->transaction, store->oids, (char *) &encoded_oid,
+	 sizeof (guint64), NULL);
    
       if (tx_context->transaction->rollback)
 	{
