@@ -1,5 +1,5 @@
 /* dataclient.c: Dataserver client for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -80,18 +80,31 @@ struct _dataclient_callback_registration
 typedef struct _dataclient_callback_registration
 dataclient_callback_registration;
 
+/* Captures callback configuration for a request a block of oids issued through
+   the data client. */
+
+struct _dataclient_oids_callback_registration
+{
+  /* The oids callback function. */
+
+  gzochid_dataclient_oids_callback oids_callback; 
+
+  gpointer oids_callback_data; /* Closure data for the oids callback. */  
+};
+
+typedef struct _dataclient_oids_callback_registration
+dataclient_oids_callback_registration;
+
 /* Holds the state of active callbacks for value and oid requests. */
 
 struct _dataclient_callback_queue
 {  
   GMutex mutex; /* Mutex to protect the list of registrations */
 
-  /* The oids callback function. */
-
-  gzochid_dataclient_oids_callback oids_callback; 
-
-  gpointer oids_callback_data; /* Closure data for the oids callback. */  
-
+  /* List of `dataclient_oids_callback_resitration' objects. */
+  
+  GList *oids_callback_registrations;
+  
   /* List of `dataclient_callback_registration' objects. */
 
   GList *callback_registrations; 
@@ -279,12 +292,30 @@ create_callback
   return registration;
 }
 
+/* Create and return a new oid callback registration object with the specified
+   callback function and associated user data pointer. This object should freed
+   via `free' when no longer needed. */
+
+static dataclient_oids_callback_registration *
+create_oids_callback (gzochid_dataclient_oids_callback oids_callback,
+		      gpointer oids_callback_data)
+{
+  dataclient_oids_callback_registration *registration =
+    malloc (sizeof (dataclient_oids_callback_registration));
+
+  registration->oids_callback = oids_callback;
+  registration->oids_callback_data = oids_callback_data;
+  
+  return registration;
+}
+
 /* Frees the callback queue structure, including all pending callbacks. */
 
 static void
 free_callback_queue (dataclient_callback_queue *queue)
 {
   g_mutex_clear (&queue->mutex);
+  g_list_free_full (queue->oids_callback_registrations, free);
   g_list_free_full (queue->callback_registrations, free);
   free (queue);
 }
@@ -314,8 +345,7 @@ acquire_callback_queue (GzochidDataClient *client, char *app)
 
       g_mutex_init (&queue->mutex);
 
-      queue->oids_callback = NULL;
-      queue->oids_callback_data = NULL;
+      queue->oids_callback_registrations = NULL;
       queue->callback_registrations = NULL;
 
       g_hash_table_insert
@@ -404,18 +434,23 @@ gzochid_dataclient_received_oids (GzochidDataClient *client,
 
   /* If somebody's waiting for oids, let them know. */
   
-  if (queue->oids_callback != NULL)
+  if (queue->oids_callback_registrations != NULL)
     {
+      dataclient_oids_callback_registration *registration =
+	queue->oids_callback_registrations->data;
+      
+      queue->oids_callback_registrations = g_list_delete_link
+	(queue->oids_callback_registrations,
+	 queue->oids_callback_registrations);
+
       gzochid_trace ("Received oid block { %" G_GUINT64_FORMAT
 		     "/%d }; invoking callback.", response->block.block_start,
 		     response->block.block_size);
 
-      queue->oids_callback (response->block, queue->oids_callback_data);
+      registration->oids_callback
+	(response->block, registration->oids_callback_data);
 
-      /* Null out the oids callback after receipt. */
-      
-      queue->oids_callback = NULL;
-      queue->oids_callback_data = NULL;
+      free (registration);
     }
   else g_debug ("Received oid block { %" G_GUINT64_FORMAT
 		"/%d }, but no pending callbacks.", response->block.block_start,
@@ -437,13 +472,10 @@ gzochid_dataclient_reserve_oids (GzochidDataClient *client, char *app,
   write_message (client, GZOCHID_DATA_PROTOCOL_REQUEST_OIDS, message_bytes);
   g_bytes_unref (message_bytes);
 
-  /* The client is responsible for synchronizing / serializing calls to this
-     function. */
+  queue->oids_callback_registrations = g_list_append
+    (queue->oids_callback_registrations,
+     create_oids_callback (callback, user_data));
   
-  assert (queue->oids_callback == NULL);
-  queue->oids_callback = callback;
-  queue->oids_callback_data = user_data;
-
   release_callback_queue (queue);
 }
 
@@ -687,31 +719,109 @@ void
 gzochid_dataclient_submit_changeset
 (GzochidDataClient *client, char *app, GArray *changes)
 {
-  gzochid_data_changeset *changeset = gzochid_data_changeset_new (app, changes);
-  GByteArray *payload = g_byte_array_new ();
+  GList *messages = NULL;
+  static guint64 next_changeset_id = 0;
+  G_LOCK_DEFINE_STATIC (next_changeset_id);
+    
+  gzochid_data_changeset *changeset = NULL;
 
-  /* Serialize the changeset submission message. */
+  /* Guard the increment of the next changeset id - 64-bit operations aren't
+     guaranteed to be atomic. The changeset id is entirely ephemeral and only
+     needs to be unique with respect to the lifetime of a single node's
+     connection to the meta server. */
   
-  gzochid_data_protocol_changeset_write (changeset, payload);
-
-  gzochid_trace
-    ("Submitting changeset for %s with %d elements; total %d bytes.", app,
-     changes->len, payload->len);
+  G_LOCK (next_changeset_id);
+  changeset = gzochid_data_changeset_new (app, next_changeset_id++, changes);
+  G_UNLOCK (next_changeset_id);
   
-  if (payload->len <= MAX_MESSAGE_LENGTH)
+  /* Perform one iteration over the changeset, breaking it into messages to send
+     to the meta server. */
+  
+  while (TRUE)
     {
-      GBytes *payload_bytes = g_byte_array_free_to_bytes (payload);
+      GByteArray *payload = g_byte_array_new ();
+  
+      /* Serialize the changeset submission message. */
+  
+      size_t changes_written = gzochid_data_protocol_changeset_write
+	(changeset, payload);
 
-      write_message
-	(client, GZOCHID_DATA_PROTOCOL_SUBMIT_CHANGESET, payload_bytes);
-      g_bytes_unref (payload_bytes);
+      /* This might happen if we tried to submit a single object that was larger
+	 than 65k. */
+      
+      if (changes_written == 0)
+	{
+	  g_critical
+	    ("Refusing to submit %d-byte change for %s to meta server.",
+	     payload->len, app);
+
+	  g_byte_array_unref (payload);
+	  g_list_free_full (messages, (GDestroyNotify) g_bytes_unref);
+	  messages = NULL;
+	  
+	  break;
+	}
+      else
+	{
+	  messages = g_list_append
+	    (messages, g_byte_array_free_to_bytes (payload));
+
+	  /* Remove any changes that were successfully packaged into a 
+	     message. */
+	  
+	  if (changes_written < changeset->changes->len)
+	    g_array_remove_range (changeset->changes, 0, changes_written);
+	  else break;
+	}
     }
-  else
+  
+  if (messages != NULL)
     {
-      g_critical
-	("Refusing to submit %d-byte changeset for %s to meta server.",
-	 payload->len, app);
-      g_byte_array_unref (payload);
+      size_t len = 0;
+      int i = 1, total_messages = g_list_length (messages);
+      GList *message_ptr = messages;
+      GByteArray *begin_end_message = g_byte_array_new ();
+      GBytes *begin_end_message_bytes = NULL;
+
+      g_byte_array_append
+	(begin_end_message, (unsigned char *) app, strlen (app) + 1);
+
+      len = begin_end_message->len;
+      g_byte_array_set_size (begin_end_message, len + 8);
+      gzochi_common_io_write_long (changeset->id, begin_end_message->data, len);
+
+      begin_end_message_bytes = g_byte_array_free_to_bytes (begin_end_message);
+      
+      write_message (client, GZOCHID_DATA_PROTOCOL_BEGIN_CHANGESET,
+		     begin_end_message_bytes);
+      
+      /* Now do a second pass to actually send the changeset messages up to the
+	 meta server. */
+      
+      while (message_ptr != NULL)
+	{
+	  GBytes *payload_bytes = message_ptr->data;
+
+	  if (total_messages == 1)
+	    gzochid_trace
+	      ("Submitting changeset for %s with %d elements; total %"
+	       G_GSIZE_FORMAT " bytes.", app, changes->len,
+	       g_bytes_get_size (payload_bytes));
+	  else gzochid_trace
+		 ("Submitting changeset %d/%d for %s; total %" G_GSIZE_FORMAT
+		  " bytes.", i++, total_messages, app,
+		  g_bytes_get_size (payload_bytes));
+	  
+	  write_message
+	    (client, GZOCHID_DATA_PROTOCOL_SUBMIT_CHANGESET, payload_bytes);
+	  message_ptr = message_ptr->next;
+	}
+
+      g_list_free_full (messages, (GDestroyNotify) g_bytes_unref);
+      write_message (client, GZOCHID_DATA_PROTOCOL_END_CHANGESET,
+		     begin_end_message_bytes);
+
+      g_bytes_unref (begin_end_message_bytes);
     }
   
   gzochid_data_changeset_free (changeset);
