@@ -1,5 +1,5 @@
 /* dataclient.h: Prototypes and declarations for dataclient.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -29,28 +29,8 @@
 
 #define GZOCHID_TYPE_DATA_CLIENT gzochid_data_client_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_data_client_get_type (void);
-
-typedef struct _GzochidDataClient GzochidDataClient;
-
-struct _GzochidDataClientClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidDataClientClass GzochidDataClientClass;
-
-static inline GzochidDataClient *
-GZOCHID_DATA_CLIENT (gconstpointer ptr)
-{
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_data_client_get_type (), GzochidDataClient);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochidDataClient, gzochid_data_client, GZOCHID,
+		      DATA_CLIENT, GObject);
 
 /* Function pointer typedef for a "success" callback to a request for a value or
    binding. The `GBytes' argument may be null if the request was successful
