@@ -1,5 +1,5 @@
 /* dataserver-protocol.c: Implementation of dataserver protocol.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -462,6 +462,82 @@ dispatch_release_key_range (gzochi_metad_dataserver_client *client,
   return TRUE;
 }
 
+/* Processes the message payload following the 
+   `GZOZCHID_DATA_PROTOCOL_BEGIN_CHANGESET' opcode. Returns `TRUE' if the 
+   message was successfully decoded, `FALSE' otherwise. */
+
+static gboolean
+dispatch_begin_changeset (gzochi_metad_dataserver_client *client,
+			  unsigned char *data, unsigned short len)
+{
+  size_t str_len = 0;
+  const char *app = gzochid_protocol_read_str (data, len, &str_len);
+
+  if (app == NULL || str_len <= 1 || len - str_len < 8)
+    {
+      g_warning
+	("Received malformed 'BEGIN_CHANGESET' message from node %d.",
+	 client->node_id);
+      return FALSE;
+    }
+  else
+    {
+      GError *err = NULL;
+      guint64 changeset_id = gzochi_common_io_read_long (data, str_len);
+
+      gzochi_metad_dataserver_begin_changeset
+	(client->dataserver, client->node_id, app, changeset_id, &err);
+
+      if (err != NULL)
+	{
+	  g_warning
+	    ("Failed to initiate changeset submission: %s", err->message);
+	  g_error_free (err);
+
+	  return FALSE;
+	}
+      else return TRUE;
+    }
+}
+
+/* Processes the message payload following the 
+   `GZOZCHID_DATA_PROTOCOL_END_CHANGESET' opcode. Returns `TRUE' if the 
+   message was successfully decoded, `FALSE' otherwise. */
+
+static gboolean
+dispatch_end_changeset (gzochi_metad_dataserver_client *client,
+			unsigned char *data, unsigned short len)
+{
+  size_t str_len = 0;
+  const char *app = gzochid_protocol_read_str (data, len, &str_len);
+
+  if (app == NULL || str_len <= 1 || len - str_len < 8)
+    {
+      g_warning
+	("Received malformed 'END_CHANGESET' message from node %d.",
+	 client->node_id);
+      return FALSE;
+    }
+  else
+    {
+      GError *err = NULL;
+      guint64 changeset_id = gzochi_common_io_read_long (data, str_len);
+
+      gzochi_metad_dataserver_end_changeset
+	(client->dataserver, client->node_id, app, changeset_id, &err);
+
+      if (err != NULL)
+	{
+	  g_warning
+	    ("Failed to complete changeset submission: %s", err->message);
+	  g_error_free (err);
+
+	  return FALSE;
+	}
+      else return TRUE;
+    }
+}
+
 /* Attempt to dispatch a fully-buffered message from the specified client based
    on its opcode. */
 
@@ -488,6 +564,10 @@ dispatch_message (gzochi_metad_dataserver_client *client,
       dispatch_release_key (client, payload, len); break;
     case GZOCHID_DATA_PROTOCOL_RELEASE_KEY_RANGE:
       dispatch_release_key_range (client, payload, len); break;
+    case GZOCHID_DATA_PROTOCOL_BEGIN_CHANGESET:
+      dispatch_begin_changeset (client, payload, len); break;
+    case GZOCHID_DATA_PROTOCOL_END_CHANGESET:
+      dispatch_end_changeset (client, payload, len); break;
       
     default:
       g_warning ("Unexpected opcode %d received from client", opcode);
@@ -533,6 +613,8 @@ static void
 client_error (gpointer user_data)
 {
   gzochi_metad_dataserver_client *client = user_data;
+
+  gzochi_metad_dataserver_abort_all (client->dataserver, client->node_id);
   gzochi_metad_dataserver_release_all (client->dataserver, client->node_id);
 }
 
