@@ -1,5 +1,5 @@
 /* dataserver.c: Data server for gzochi-metad
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "app-store.h"
 #include "config.h"
 #include "dataserver-protocol.h"
 #include "dataserver.h"
@@ -48,40 +49,24 @@
 
 #define LOCK_ACCESS(for_write) (for_write ? "r/w" : "read")
 
-/* A store with an associated lock table. */
+/* A wrapper around `gzochid_application_store' that composes it with a pair
+   of lock tables. */
 
-struct _gzochi_metad_dataserver_lockable_store
+struct _gzochid_lockable_application_store
 {
-  gzochid_storage_store *store; /* The persistent store. */
-  gzochid_lock_table *locks; /* The lock table. */
+  gzochid_application_store *store; /* The application store. */
+
+  gzochid_lock_table *oids_locks; /* Lock table for the object store. */
+  gzochid_lock_table *names_locks; /* Lock table for the binding store. */
+
+  /* Map of node id (as `guint') to `GHashTable' mapping of changeset id (as 
+     `guint64') to `GArray' of `gzochid_data_change'. */
+
+  GHashTable *changeset_submissions; 
 };
 
-typedef struct _gzochi_metad_dataserver_lockable_store
-gzochi_metad_dataserver_lockable_store;
-
-/* The dataserver-side representation of the storage for a gzochi game 
-   application, somewhat analogous to a `gzochid_application_context' 
-   structure. */
-
-struct _gzochi_metad_dataserver_application_store
-{
-  char *name; /* The name of the application. */
-  gzochid_storage_context *storage_context; /* The storage context. */
-
-  gzochi_metad_dataserver_lockable_store *oids; /* The lockable object store. */
-
-  /* The lockable binding store. */
-  
-  gzochi_metad_dataserver_lockable_store *names;
-  gzochid_storage_store *meta; /* The metadata store. */
-
-  /* The oid allocation strategy. */
-  
-  gzochid_oid_allocation_strategy *oid_strategy; 
-};
-
-typedef struct _gzochi_metad_dataserver_application_store
-gzochi_metad_dataserver_application_store;
+typedef struct _gzochid_lockable_application_store
+gzochid_lockable_application_store;
 
 /* Boilerplate setup for the data server object. */
 
@@ -110,7 +95,7 @@ struct _GzochiMetadDataServer
 
   gzochid_storage_engine *storage_engine; /* The storage engine. */
 
-  /* Mapping application name to `gzochi_metad_dataserver_application_store'. */
+  /* Mapping application name to `gzochid_lockable_application_store'. */
 
   GHashTable *application_stores; 
 };
@@ -164,42 +149,12 @@ gzochi_metad_data_server_set_property (GObject *object, guint property_id,
     }
 }
 
-/* A `GHRFunc' implementation that cleans up 
-   `gzochi_metad_dataserver_application_store' instances managed by the data 
-   server as part of the data server shutdown process. */
-
-static gboolean
-close_application_store (gpointer key, gpointer value, gpointer user_data)
-{
-  GzochiMetadDataServer *server = user_data;
-  gzochi_metad_dataserver_application_store *store = value;
-  
-  STORAGE_INTERFACE (server)->close_store (store->oids->store);
-  gzochid_lock_table_free (store->oids->locks);
-  free (store->oids);
-  
-  STORAGE_INTERFACE (server)->close_store (store->names->store);
-  gzochid_lock_table_free (store->names->locks);
-  free (store->names);
-
-  STORAGE_INTERFACE (server)->close_store (store->meta);
-
-  STORAGE_INTERFACE (server)->close_context (store->storage_context);
-
-  gzochid_oid_allocation_strategy_free (store->oid_strategy);
-  
-  return TRUE;
-}
-
 static void
 gzochi_metad_data_server_finalize (GObject *gobject)
 {
   GzochiMetadDataServer *server = GZOCHI_METAD_DATA_SERVER (gobject);
   
   g_hash_table_destroy (server->data_configuration);
-
-  g_hash_table_foreach_remove
-    (server->application_stores, close_application_store, server);
   g_hash_table_destroy (server->application_stores);
 
   if (server->storage_engine != NULL)
@@ -256,11 +211,27 @@ gzochi_metad_data_server_class_init (GzochiMetadDataServerClass *klass)
     (object_class, N_PROPERTIES, obj_properties);
 }
 
+static void
+close_application_store (gpointer ptr)
+{
+  gzochid_lockable_application_store *lockable_store = ptr;
+
+  gzochid_oid_allocation_strategy_free (lockable_store->store->oid_strategy);
+  gzochid_application_store_close (lockable_store->store);
+  
+  gzochid_lock_table_free (lockable_store->oids_locks);
+  gzochid_lock_table_free (lockable_store->names_locks);
+
+  g_hash_table_destroy (lockable_store->changeset_submissions);
+  
+  free (lockable_store);
+}
+
 static void
 gzochi_metad_data_server_init (GzochiMetadDataServer *self)
 {
   self->application_stores = g_hash_table_new_full
-    (g_str_hash, g_str_equal, (GDestroyNotify) free, (GDestroyNotify) free);
+    (g_str_hash, g_str_equal, free, close_application_store);
 }
 
 void
@@ -297,27 +268,30 @@ FOR PRODUCTION USE.");
     }
 }
 
-/* Switch on the specified store name to return either the oids store or the
-   named binding store, or set an error if the name was not "oids" or 
+/* Switch on the specified store name to fill out the specified store and lock
+   table pointers to pointer to the oids store and lock table, the names 
+   bindings store and lock table, or set an error if the name was not "oids" or 
    "names." */
 
-static gzochi_metad_dataserver_lockable_store *
-get_lockable_store (gzochi_metad_dataserver_application_store *store,
-		    const char *name, GError **err)
+static void
+get_lockable_store (gzochid_lockable_application_store *app_store,
+		    const char *name, gzochid_storage_store **store,
+		    gzochid_lock_table **locks, GError **err)
 {
   if (strcmp (name, "oids") == 0)
-    return store->oids;
-  else if (strcmp (name, "names") == 0)
-    return store->names;
-  else
     {
-      g_set_error
-	(err, GZOCHI_METAD_DATASERVER_ERROR,
-	 GZOCHI_METAD_DATASERVER_ERROR_STORE_NAME, "Invalid store name '%s'.",
-	 name);
-
-      return NULL;
+      *store = app_store->store->oids;
+      *locks = app_store->oids_locks;
     }
+  else if (strcmp (name, "names") == 0)
+    {
+      *store = app_store->store->names;
+      *locks = app_store->names_locks;
+    }
+  else g_set_error
+	 (err, GZOCHI_METAD_DATASERVER_ERROR,
+	  GZOCHI_METAD_DATASERVER_ERROR_STORE_NAME, "Invalid store name '%s'.",
+	  name);
 }
 
 /* 
@@ -326,10 +300,10 @@ get_lockable_store (gzochi_metad_dataserver_application_store *store,
   necessary.
   
   This function returns a pointer to the new or existing application store; 
-  this poitner is owned by the data server and should not be freed. 
+  this pointer is owned by the data server and should not be freed. 
 */
 
-static gzochi_metad_dataserver_application_store *
+static gzochid_lockable_application_store *
 ensure_open_application_store (const GzochiMetadDataServer *server,
 			       const char *app)
 {
@@ -337,32 +311,22 @@ ensure_open_application_store (const GzochiMetadDataServer *server,
     return g_hash_table_lookup (server->application_stores, app);
   else
     {
-      gzochi_metad_dataserver_application_store *store =
-	malloc (sizeof (gzochi_metad_dataserver_application_store));
       gzochid_storage_engine_interface *iface = STORAGE_INTERFACE (server);
+      gzochid_lockable_application_store *lockable_store =
+	malloc (sizeof (gzochid_lockable_application_store));
 
+      lockable_store->store = gzochid_application_store_open
+	(iface, app, gzochid_storage_oid_strategy_new (iface, app));
+      lockable_store->oids_locks = gzochid_lock_table_new ("oids");
+      lockable_store->names_locks = gzochid_lock_table_new ("names");
+      lockable_store->changeset_submissions = g_hash_table_new_full
+	(g_int_hash, g_int_equal, free, (GDestroyNotify) g_hash_table_destroy);
+      
       g_message ("Initializing application storage for '%s'.", app);
       
-      store->storage_context = iface->initialize ((char *) app);
-
-      store->oids = malloc (sizeof (gzochi_metad_dataserver_lockable_store));
-      store->oids->store = iface->open
-	(store->storage_context, "oids", GZOCHID_STORAGE_CREATE);
-      store->oids->locks = gzochid_lock_table_new ("oids"); 
-
-      store->names = malloc (sizeof (gzochi_metad_dataserver_lockable_store));
-      store->names->store = iface->open
-	(store->storage_context, "names", GZOCHID_STORAGE_CREATE);
-      store->names->locks = gzochid_lock_table_new ("names"); 
-
-      store->meta = iface->open
-	(store->storage_context, "meta", GZOCHID_STORAGE_CREATE);
-
-      store->oid_strategy = gzochid_storage_oid_strategy_new
-	(iface, store->storage_context, store->meta);
-      
-      g_hash_table_insert (server->application_stores, strdup (app), store);
-      return store;
+      g_hash_table_insert
+	(server->application_stores, strdup (app), lockable_store);
+      return lockable_store;
     }
 }
 
@@ -372,13 +336,13 @@ gzochi_metad_dataserver_reserve_oids (GzochiMetadDataServer *server,
 {
   gzochid_data_oids_block oids_block;
   gzochid_data_reserve_oids_response *response = NULL;
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, app);
 
   gzochid_trace ("Node %d requested oid block for %s.", node_id, app);
   
   assert (gzochid_oids_reserve_block
-	  (app_store->oid_strategy, &oids_block, NULL));
+	  (app_store->store->oid_strategy, &oids_block, NULL));
 
   gzochid_trace ("Reserved block { %" G_GUINT64_FORMAT ", %d } for node %d/%s.",
 		 oids_block.block_start, oids_block.block_size, node_id, app);
@@ -394,16 +358,17 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
 				       gboolean for_write, GError **err)
 {
   gzochid_data_response *response = NULL;
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, app);
 
   GError *local_err = NULL;
-  gzochi_metad_dataserver_lockable_store *store = get_lockable_store
-    (app_store, store_name, &local_err);
-  
+  gzochid_storage_store *store = NULL;
+  gzochid_lock_table *locks = NULL;
   struct timeval most_recent_lock;
 
-  if (store == NULL)
+  get_lockable_store (app_store, store_name, &store, &locks, &local_err);
+  
+  if (local_err != NULL)
     {
       g_propagate_error (err, local_err);
       return NULL;
@@ -416,13 +381,13 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
 	LOCK_ACCESS (for_write), app, store_name, buf));
   
   if (gzochid_lock_check_and_set
-      (store->locks, node_id, key, for_write, &most_recent_lock))
+      (locks, node_id, key, for_write, &most_recent_lock))
     {
       size_t data_len = 0;
-      gzochid_storage_transaction *transaction = STORAGE_INTERFACE (server)
-	->transaction_begin (app_store->storage_context);
-      char *data = STORAGE_INTERFACE (server)->transaction_get
-	(transaction, store->store, (char *) g_bytes_get_data (key, NULL),
+      gzochid_storage_transaction *transaction = app_store->store->iface
+	->transaction_begin (app_store->store->storage_context);
+      char *data = app_store->store->iface->transaction_get
+	(transaction, store, (char *) g_bytes_get_data (key, NULL),
 	 g_bytes_get_size (key), &data_len);
 
       if (data != NULL)
@@ -438,7 +403,7 @@ gzochi_metad_dataserver_request_value (GzochiMetadDataServer *server,
 	}
       else response = gzochid_data_response_new (app, store_name, TRUE, NULL);
 
-      STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+      app_store->store->iface->transaction_rollback (transaction);
 
       if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
 	GZOCHID_WITH_FORMATTED_BYTES
@@ -467,13 +432,13 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
 					  GError **err)
 {
   gzochid_data_response *response = NULL;
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, app);
   GBytes *to_key = NULL;
   
   GError *local_err = NULL;
-  gzochi_metad_dataserver_lockable_store *store = get_lockable_store
-    (app_store, store_name, &local_err);
+  gzochid_storage_store *store = NULL;
+  gzochid_lock_table *locks = NULL;
 
   struct timeval most_recent_lock;
 
@@ -481,21 +446,23 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
   gzochid_storage_transaction *transaction = NULL;
   char *data = NULL;
 
-  if (store == NULL)
+  get_lockable_store (app_store, store_name, &store, &locks, &local_err);
+
+  if (local_err != NULL)
     {
       g_propagate_error (err, local_err);
       return NULL;
     }
   
-  transaction = STORAGE_INTERFACE (server)->transaction_begin
-    (app_store->storage_context);
+  transaction = app_store->store->iface->transaction_begin
+    (app_store->store->storage_context);
   
   if (key == NULL)
     {
       gzochid_trace ("Node %d requested range lock on %s/%s keyspace.", node_id,
 		     app, store_name);
-      data = STORAGE_INTERFACE (server)->transaction_first_key
-	(transaction, store->store, &data_len);
+      data = app_store->store->iface->transaction_first_key
+	(transaction, store, &data_len);
     }
   else
     {
@@ -505,8 +472,8 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
 	   ("Node %d requested range lock from key %s/%s/%s.", node_id, app,
 	    store_name, buf));
 	   
-      data = STORAGE_INTERFACE (server)->transaction_next_key
-	(transaction, store->store, (char *) g_bytes_get_data (key, NULL),
+      data = app_store->store->iface->transaction_next_key
+	(transaction, store, (char *) g_bytes_get_data (key, NULL),
 	 g_bytes_get_size (key), &data_len);
     }
   
@@ -516,10 +483,10 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
     to_key = g_bytes_new_with_free_func
       (data, data_len, (GDestroyNotify) free, data);
 
-  STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+  app_store->store->iface->transaction_rollback (transaction);
 
   if (!gzochid_lock_range_check_and_set
-      (store->locks, node_id, key, to_key, &most_recent_lock))
+      (locks, node_id, key, to_key, &most_recent_lock))
     {
       if (key == NULL)
 	g_debug ("Denied range lock on %s/%s keyspace to node %d.", app,
@@ -553,38 +520,129 @@ gzochi_metad_dataserver_request_next_key (GzochiMetadDataServer *server,
   return response;
 }
 
+void
+gzochi_metad_dataserver_begin_changeset (GzochiMetadDataServer *server,
+					 guint node_id, const char *app,
+					 guint64 changeset_id, GError **err)
+{
+  gzochid_lockable_application_store *app_store =
+    ensure_open_application_store (server, app);
+  GHashTable *node_submissions = NULL;
+  
+  if (g_hash_table_contains (app_store->changeset_submissions, &node_id))
+    node_submissions = g_hash_table_lookup
+      (app_store->changeset_submissions, &node_id);
+  else
+    {
+      node_submissions = g_hash_table_new_full
+	(g_int64_hash, g_int64_equal, free, (GDestroyNotify) g_array_unref);
+      g_hash_table_insert (app_store->changeset_submissions,
+			   g_memdup (&node_id, sizeof (guint)),
+			   node_submissions);
+    }
+
+  if (g_hash_table_contains (node_submissions, &changeset_id))
+    g_set_error
+      (err, GZOCHI_METAD_DATASERVER_ERROR,
+       GZOCHI_METAD_DATASERVER_ERROR_CHANGESET_ID,
+       "Changeset submission for changeset %" G_GUINT64_FORMAT " for %d/%s "
+       "already in progress.", changeset_id, node_id, app);
+  else g_hash_table_insert
+	 (node_submissions,
+	  g_memdup (&changeset_id, sizeof (guint64)),
+	  g_array_new (FALSE, FALSE, sizeof (gzochid_data_change)));
+}
+
 void
 gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
 					   guint node_id,
 					   gzochid_data_changeset *changeset,
 					   GError **err)
 {
-  gint i = 0;
-  gboolean needs_rollback = FALSE;
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, changeset->app);
-  gzochid_storage_transaction *transaction =
-    STORAGE_INTERFACE (server)->transaction_begin (app_store->storage_context);
+  GHashTable *node_submissions = g_hash_table_lookup
+    (app_store->changeset_submissions, &node_id);
+  GArray *pending_changes = NULL;  
+  
+  if (node_submissions == NULL
+      || !g_hash_table_contains (node_submissions, &changeset->id))
+    {
+      g_set_error (err, GZOCHI_METAD_DATASERVER_ERROR,
+		   GZOCHI_METAD_DATASERVER_ERROR_CHANGESET_ID,
+		   "Unknown changeset %" G_GUINT64_FORMAT " for %d/%s.",
+		   changeset->id, node_id, changeset->app);
+      return;
+    }
 
   gzochid_trace ("Processing %d changes from %d/%s.", changeset->changes->len,
 		 node_id, changeset->app);
+
+  pending_changes = g_hash_table_lookup (node_submissions, &changeset->id);
+  g_array_append_vals (pending_changes, changeset->changes->data,
+		       changeset->changes->len);
+}
+
+/* Frees the memory held by the internal pointers for a change record. The 
+   record itself is part of the change array and will be cleaned up when the 
+   array is destroyed. */
+
+static void
+change_clear (gzochid_data_change *change)
+{
+  free (change->store);
+  g_bytes_unref (change->key);
   
-  for (; i < changeset->changes->len; i++)
+  if (!change->delete)
+    g_bytes_unref (change->data);
+}
+
+void
+gzochi_metad_dataserver_end_changeset (GzochiMetadDataServer *server,
+				       guint node_id, const char *app,
+				       guint64 changeset_id, GError **err)
+{
+  gint i = 0;
+  gboolean needs_rollback = FALSE;
+  gzochid_lockable_application_store *app_store =
+    ensure_open_application_store (server, app);
+  GHashTable *node_submissions = g_hash_table_lookup
+    (app_store->changeset_submissions, &node_id);
+  GArray *pending_changes = NULL;
+  gzochid_storage_transaction *transaction = NULL;
+  
+  if (node_submissions == NULL
+      || !g_hash_table_contains (node_submissions, &changeset_id))
+    {
+      g_set_error (err, GZOCHI_METAD_DATASERVER_ERROR,
+		   GZOCHI_METAD_DATASERVER_ERROR_CHANGESET_ID,
+		   "Unknown changeset %" G_GUINT64_FORMAT " for %d/%s.",
+		   changeset_id, node_id, app);
+      return;
+    }
+
+  pending_changes = g_hash_table_lookup (node_submissions, &changeset_id);
+  transaction = app_store->store->iface->transaction_begin
+    (app_store->store->storage_context);
+  
+  for (; i < pending_changes->len; i++)
     {
       gzochid_data_change change = g_array_index
-	(changeset->changes, gzochid_data_change, i);
+	(pending_changes, gzochid_data_change, i);
       GError *local_err = NULL;
-      gzochi_metad_dataserver_lockable_store *store = get_lockable_store
-	(app_store, change.store, &local_err);
+      gzochid_storage_store *store = NULL;
+      gzochid_lock_table *locks = NULL;
 
-      if (store == NULL)
+      get_lockable_store (app_store, change.store, &store, &locks, &local_err);
+
+      if (local_err != NULL)
 	{
 	  g_propagate_error (err, local_err);
 	  needs_rollback = TRUE;
 	  break;
 	}
 
-      if (!gzochid_lock_check (store->locks, node_id, change.key, TRUE))
+      if (!gzochid_lock_check (locks, node_id, change.key, TRUE))
 	{
 	  g_set_error
 	    (err, GZOCHI_METAD_DATASERVER_ERROR,
@@ -597,13 +655,12 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
 	}
 
 	if (change.delete)
-	  STORAGE_INTERFACE (server)->transaction_delete
-	    (transaction, store->store,
-	     (char *) g_bytes_get_data (change.key, NULL),
+	  app_store->store->iface->transaction_delete
+	    (transaction, store, (char *) g_bytes_get_data (change.key, NULL),
 	     g_bytes_get_size (change.key));
 
-	else STORAGE_INTERFACE (server)->transaction_put
-	       (transaction, store->store,
+	else app_store->store->iface->transaction_put
+	       (transaction, store,
 		(char *) g_bytes_get_data (change.key, NULL),
 		g_bytes_get_size (change.key),
 		(char *) g_bytes_get_data (change.data, NULL),
@@ -624,31 +681,37 @@ gzochi_metad_dataserver_process_changeset (GzochiMetadDataServer *server,
     
   if (needs_rollback)
     {
-      g_debug ("Changes from %d/%s rolled back during processing.", node_id,
-	       changeset->app);
-      STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+      g_debug ("Transaction for changeset %" G_GUINT64_FORMAT " for %d/%s was "
+	       "rolled back during processing.", changeset_id, node_id, app);
+      app_store->store->iface->transaction_rollback (transaction);
     }
   else
     {
-      STORAGE_INTERFACE (server)->transaction_prepare (transaction);
-
+      app_store->store->iface->transaction_prepare (transaction);
+  
       if (transaction->rollback)
 	{
 	  g_set_error
 	    (err, GZOCHI_METAD_DATASERVER_ERROR,
 	     GZOCHI_METAD_DATASERVER_ERROR_FAILED,
-	     "Transaction failed to prepare.");
-	  g_debug ("Changes from %d/%s rolled back during prepare.", node_id,
-		   changeset->app);
-	  STORAGE_INTERFACE (server)->transaction_rollback (transaction);
+	     "Transaction for changeset %" G_GUINT64_FORMAT " for %d/%s failed "
+	     "to prepare.", changeset_id, node_id, app);
+	  g_debug ("Changeset %" G_GUINT64_FORMAT " for %d/%s rolled back "
+		   "during prepare.", changeset_id, node_id, app);
+	  app_store->store->iface->transaction_rollback (transaction);
 	}
       else
 	{
-	  gzochid_trace ("Committed changes from %d/%s.", node_id,
-			 changeset->app);
-	  STORAGE_INTERFACE (server)->transaction_commit (transaction);
+	  app_store->store->iface->transaction_commit (transaction);
+	  gzochid_trace ("Committed changes from %" G_GUINT64_FORMAT
+			 " for %d/%s.", changeset_id, node_id, app);
 	}
     }
+
+  for (i = 0; i < pending_changes->len; i++)
+    change_clear (&g_array_index (pending_changes, gzochid_data_change, i));
+  
+  g_hash_table_remove (node_submissions, &changeset_id);
 }
 
 void
@@ -656,12 +719,14 @@ gzochi_metad_dataserver_release_key (GzochiMetadDataServer *server,
 				     guint node_id, const char *app,
 				     const char *store_name, GBytes *key)
 {
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, app);
 
   GError *local_err = NULL;
-  gzochi_metad_dataserver_lockable_store *store = get_lockable_store
-    (app_store, store_name, &local_err);
+  gzochid_storage_store *store = NULL;
+  gzochid_lock_table *locks = NULL;
+
+  get_lockable_store (app_store, store_name, &store, &locks, &local_err);
 
   if (local_err != NULL)
     {
@@ -677,7 +742,7 @@ gzochi_metad_dataserver_release_key (GzochiMetadDataServer *server,
       (key, buf, 33, gzochid_trace ("Node id %d releasing key %s/%s/%s.",
 				    node_id, app, store_name, buf));
   
-  gzochid_lock_release (store->locks, node_id, key);
+  gzochid_lock_release (locks, node_id, key);
 }
 
 void
@@ -686,12 +751,14 @@ gzochi_metad_dataserver_release_range (GzochiMetadDataServer *server,
 				       const char *store_name,
 				       GBytes *first_key, GBytes *last_key)
 {
-  gzochi_metad_dataserver_application_store *app_store =
+  gzochid_lockable_application_store *app_store =
     ensure_open_application_store (server, app);
   
   GError *local_err = NULL;
-  gzochi_metad_dataserver_lockable_store *store = get_lockable_store
-    (app_store, store_name, &local_err);
+  gzochid_storage_store *store = NULL;
+  gzochid_lock_table *locks = NULL;
+  
+  get_lockable_store (app_store, store_name, &store, &locks, &local_err);
 
   if (local_err != NULL)
     {
@@ -711,7 +778,7 @@ gzochi_metad_dataserver_release_range (GzochiMetadDataServer *server,
        ("Node id %d releasing range lock from %s/%s/%s.", node_id, app,
 	store_name, buf));
   
-  gzochid_lock_release_range (store->locks, node_id, first_key, last_key);
+  gzochid_lock_release_range (locks, node_id, first_key, last_key);
 }
 
 void
@@ -725,10 +792,46 @@ gzochi_metad_dataserver_release_all (GzochiMetadDataServer *server,
 
   while (g_hash_table_iter_next (&iter, NULL, &value))
     {
-      gzochi_metad_dataserver_application_store *store = value;
+      gzochid_lockable_application_store *store = value;
 
-      gzochid_lock_release_all (store->oids->locks, node_id);
-      gzochid_lock_release_all (store->names->locks, node_id);
+      gzochid_lock_release_all (store->oids_locks, node_id);
+      gzochid_lock_release_all (store->names_locks, node_id);
+    }
+}
+
+/* A `GHFunc' implementation for clearing out each pending changeset associated
+   with a particular node's participation in a particular application. */
+
+static void
+abort_submission_ghfunc (gpointer key, gpointer value, gpointer user_data)
+{
+  gint i = 0;
+  GArray *pending_changes = value;
+
+  for (; i < pending_changes->len; i++)
+    change_clear (&g_array_index (pending_changes, gzochid_data_change, i));
+}
+
+void
+gzochi_metad_dataserver_abort_all (GzochiMetadDataServer *server, guint node_id)
+{
+  GHashTableIter iter;
+  gpointer value = NULL;
+
+  g_hash_table_iter_init (&iter, server->application_stores);
+
+  while (g_hash_table_iter_next (&iter, NULL, &value))
+    {
+      gzochid_lockable_application_store *store = value;
+      GHashTable *node_submissions = g_hash_table_lookup
+	(store->changeset_submissions, &node_id);
+
+      if (node_submissions != NULL)
+	{      
+	  g_hash_table_foreach
+	    (node_submissions, abort_submission_ghfunc, NULL);
+	  g_hash_table_remove (store->changeset_submissions, &node_id);
+	}
     }
 }
 
