@@ -1,5 +1,5 @@
 /* dataserver.h: Prototypes and declarations for dataserver.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -28,28 +28,8 @@
 
 #define GZOCHI_METAD_TYPE_DATA_SERVER gzochi_metad_data_server_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochi_metad_data_server_get_type (void);
-
-typedef struct _GzochiMetadDataServer GzochiMetadDataServer;
-
-struct _GzochiMetadDataServerClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochiMetadDataServerClass GzochiMetadDataServerClass;
-
-static inline GzochiMetadDataServer *
-GZOCHI_METAD_DATA_SERVER (gconstpointer ptr)
-{
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochi_metad_data_server_get_type (), GzochiMetadDataServer);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochiMetadDataServer, gzochi_metad_data_server,
+                      GZOCHI_METAD, DATA_SERVER, GObject);
 
 enum
   {
@@ -62,6 +42,10 @@ enum
     /* The store name was not one of "oids" or "names." */
     
     GZOCHI_METAD_DATASERVER_ERROR_STORE_NAME,
+
+    /* Received a duplicate or unknown changeset id. */
+    
+    GZOCHI_METAD_DATASERVER_ERROR_CHANGESET_ID,
     
     GZOCHI_METAD_DATASERVER_ERROR_FAILED /* Generic data server failure. */
   };
@@ -124,18 +108,39 @@ void gzochi_metad_dataserver_release_range
 (GzochiMetadDataServer *, guint, const char *, const char *, GBytes *,
  GBytes *);
 
+/* Aborts all changeset submissions in progress initiated by the specified node
+   id within the specified data server. */
+
+void gzochi_metad_dataserver_abort_all (GzochiMetadDataServer *, guint);
+
 /* Releases all locks (read / write / range) held by the specified node id 
    within the specified data server. */ 
 
-void gzochi_metad_dataserver_release_all
-(GzochiMetadDataServer *, guint);
+void gzochi_metad_dataserver_release_all (GzochiMetadDataServer *, guint);
+
+/* Initiates the submission of a new series of changesets for the specified
+   application from the specified node id. The specified changeset id is used to
+   identify the changeset transaction across multiple calls to 
+   `gzochi_metad_dataserver_process_changeset'. Sets the optional `GError' if 
+   there is already a changeset submission in progress with the specified 
+   changeset id. */
+
+void gzochi_metad_dataserver_begin_changeset (GzochiMetadDataServer *, guint,
+					      const char *, guint64, GError **);
 
 /* Process the specified changeset on behalf of the specified node id with 
-   respect to the specified data server as a single, atomic transaction. Sets
-   the optional `GError' if the transaction cannot be committed (e.g., because 
-   of missing locks). */
+   respect to the specified data server. Sets the optional `GError' if the 
+   changeset cannot be processed (e.g., because of missing locks). */
 
 void gzochi_metad_dataserver_process_changeset
 (GzochiMetadDataServer *, guint, gzochid_data_changeset *, GError **);
 
+/* Completes the submission of the series of changesets for the specified
+   application from the specified node id identified by the specified changeset
+   id. Sets the optional `GError' if there is no changeset submission in 
+   progress with the specified changeset id. */
+
+void gzochi_metad_dataserver_end_changeset (GzochiMetadDataServer *, guint,
+					    const char *, guint64, GError **);
+
 #endif /* GZOCHI_METAD_DATASERVER_H */
