@@ -132,13 +132,13 @@ gzochid_event_class_init (GzochidEventClass *klass)
 static void
 gzochid_event_init (GzochidEvent *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE
+  GzochidEventPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE
     (self, GZOCHID_TYPE_EVENT, GzochidEventPrivate);
   
   /* Initialize the event timestamp to the current time; in most cases, this 
      won't need to be overridden by clients. */     
   
-  self->priv->timestamp_us = g_get_real_time ();
+  priv->timestamp_us = g_get_real_time ();
 }
 
 /* The data event object. */
