@@ -1,5 +1,5 @@
 /* game-protocol.c: Implementation of game application protocol.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <sys/time.h>
 
 #include "app.h"
+#include "app-session.h"
 #include "app-task.h"
 #include "event.h"
 #include "game-protocol.h"
@@ -43,20 +44,20 @@
 struct _gzochid_game_protocol_closure
 {
   GzochidGameServer *game_server; /* Reference to the game server. */
-  gzochid_task_queue *task_queue; /* The game server's task queue. */
+  GzochidTaskQueue *task_queue; /* The game server's task queue. */
   struct timeval tx_timeout; /* The default task execution timeout. */
 };
 
 gzochid_game_protocol_closure *
 gzochid_game_protocol_create_closure (GzochidGameServer *game_server,
-				      gzochid_task_queue *task_queue,
+				      GzochidTaskQueue *task_queue,
 				      struct timeval tx_timeout)
 {
   gzochid_game_protocol_closure *closure =
     malloc (sizeof (gzochid_game_protocol_closure));
 
   closure->game_server = g_object_ref (game_server);
-  closure->task_queue = task_queue;
+  closure->task_queue = g_object_ref (task_queue);
   closure->tx_timeout = tx_timeout;
   
   return closure;
@@ -70,7 +71,7 @@ struct _gzochid_game_client
 
   /* The client application context; `NULL' until login has completed. */
   
-  gzochid_application_context *app_context; 
+  GzochidApplicationContext *app_context; 
   
   /* The client's identity. `NULL' until login has completed. */
 
@@ -124,21 +125,25 @@ client_can_dispatch (const GByteArray *buffer, gpointer user_data)
 */
 
 static void
-login_catch_worker (gzochid_application_context *context,
+login_catch_worker (GzochidApplicationContext *context,
 		    gzochid_auth_identity *identity, gpointer data)
 {
   guint64 *session_oid = data;
-  gzochid_game_client *client = g_hash_table_lookup
-    (context->oids_to_clients, session_oid);
+  gzochid_application_session_map *session_map = NULL;
+  gzochid_game_client *client = NULL;
+
+  g_object_get (context, "session-map", &session_map, NULL);
+
+  client = g_hash_table_lookup (session_map->oids_to_clients, session_oid);
 
   g_message
     ("Disconnecting session '%" G_GUINT64_FORMAT "'; failed login transaction.",
      *session_oid);
 
-  g_mutex_lock (&context->client_mapping_lock);
-  g_hash_table_remove (context->oids_to_clients, session_oid);
-  g_hash_table_remove (context->clients_to_oids, client);
-  g_mutex_unlock (&context->client_mapping_lock);
+  g_mutex_lock (&session_map->client_mapping_lock);
+  g_hash_table_remove (session_map->oids_to_clients, session_oid);
+  g_hash_table_remove (session_map->clients_to_oids, client);
+  g_mutex_unlock (&session_map->client_mapping_lock);
 
   /* Disconnect the client. */
   
@@ -154,7 +159,7 @@ login_catch_worker (gzochid_application_context *context,
    below); frees the heap-allocated session oid. */
 
 static void
-login_cleanup_worker (gzochid_application_context *context,
+login_cleanup_worker (GzochidApplicationContext *context,
 		      gzochid_auth_identity *identity, gpointer data)
 {
   g_free (data);
@@ -163,12 +168,14 @@ login_cleanup_worker (gzochid_application_context *context,
 /* The application task worker for the login event.  */
 
 static void
-logged_in_task (gzochid_application_context *context,
+logged_in_task (GzochidApplicationContext *context,
 		gzochid_auth_identity *identity, gpointer data)
 {
   GError *err = NULL;
   gzochid_game_client *client = data;
   gzochid_client_session *session = gzochid_client_session_new (identity);
+  gzochid_application_session_map *session_map = NULL;
+  GzochidMetaClient *metaclient = NULL;
   
   guint64 *session_oid = malloc (sizeof (guint64));
   guint64 local_session_oid = 0;
@@ -179,7 +186,7 @@ logged_in_task (gzochid_application_context *context,
   gzochid_application_task *application_task = NULL;
   gzochid_transactional_application_task_execution *execution = NULL;    
 
-  gzochid_task task;
+  GzochidTask *task = NULL;
   
   gzochid_client_session_persist (context, session, session_oid, &err);
 
@@ -201,13 +208,15 @@ logged_in_task (gzochid_application_context *context,
   
   else local_session_oid = *session_oid;
   
+  g_object_get (context, "session-map", &session_map, NULL);
+
   login_task = gzochid_application_task_new
-    (context, identity, gzochid_scheme_application_logged_in_worker,
+    (context, identity, gzochid_scheme_application_logged_in_worker, NULL,
      session_oid);
   login_catch_task = gzochid_application_task_new
-    (context, identity, login_catch_worker, session_oid);
+    (context, identity, login_catch_worker, NULL, session_oid);
   login_cleanup_task = gzochid_application_task_new
-    (context, identity, login_cleanup_worker, session_oid);
+    (context, identity, login_cleanup_worker, NULL, session_oid);
   
   execution = gzochid_transactional_application_task_timed_execution_new 
     (login_task, login_catch_task, login_cleanup_task,
@@ -220,58 +229,62 @@ logged_in_task (gzochid_application_context *context,
   gzochid_application_task_unref (login_catch_task);
   gzochid_application_task_unref (login_cleanup_task);
 
-  g_mutex_lock (&context->client_mapping_lock);
-  g_hash_table_insert (context->oids_to_clients,
+  g_mutex_lock (&session_map->client_mapping_lock);
+  g_hash_table_insert (session_map->oids_to_clients,
 		       g_memdup (session_oid, sizeof (guint64)), client);
-  g_hash_table_insert (context->clients_to_oids, client,
+  g_hash_table_insert (session_map->clients_to_oids, client,
 		       g_memdup (session_oid, sizeof (guint64)));
-  g_mutex_unlock (&context->client_mapping_lock);
+  g_mutex_unlock (&session_map->client_mapping_lock);
 
   application_task = gzochid_application_task_new
     (context, gzochid_game_client_get_identity (client), 
-     gzochid_application_reexecuting_transactional_task_worker, execution);
-     
-  task.worker = gzochid_application_task_thread_worker;
-  task.data = application_task;
-  gettimeofday (&task.target_execution_time, NULL);
+     gzochid_application_reexecuting_transactional_task_worker,
+     (GDestroyNotify) gzochid_transactional_application_task_execution_free,
+     execution);
 
-  gzochid_schedule_run_task (client->closure->task_queue, &task);
+  task = gzochid_task_immediate_new
+    (gzochid_application_task_thread_worker,
+     (GDestroyNotify) gzochid_application_task_unref, application_task);
+
+  gzochid_schedule_run_task (client->closure->task_queue, task);
 
   /* If after executing the login task the client is *still* present in the
      client-to-session oid mapping table, safe to assume they've completed the
      login process; if we're connected to a metaserver, inform it that there's
      a new session on this application server node. */
+
+  metaclient = gzochid_application_context_get_metaclient (context);
   
-  if (g_hash_table_contains (context->oids_to_clients, &local_session_oid)
-      && context->metaclient != NULL)
+  if (g_hash_table_contains (session_map->oids_to_clients, &local_session_oid)
+      && metaclient != NULL)
     {
       GzochidSessionClient *sessionclient = NULL;
 
-      g_object_get
-	(context->metaclient, "session-client", &sessionclient, NULL);
+      g_object_get (metaclient, "session-client", &sessionclient, NULL);
 
       gzochid_sessionclient_session_connected
-	(sessionclient, context->descriptor->name, local_session_oid);
+	(sessionclient, gzochid_application_context_get_name (context),
+	 local_session_oid);
       
       g_object_unref (sessionclient);
+      g_object_unref (metaclient);
     }
 }
 
 /* Schedules the transactional stage of the login process. */
 
 static void 
-logged_in (gzochid_application_context *context, gzochid_game_client *client)
+logged_in (GzochidApplicationContext *context, gzochid_game_client *client)
 {
   gzochid_application_task *application_task = gzochid_application_task_new
-    (context, gzochid_game_client_get_identity (client), logged_in_task,
+    (context, gzochid_game_client_get_identity (client), logged_in_task, NULL,
      client);
 
-  gzochid_task *task = gzochid_task_immediate_new
-    (gzochid_application_task_thread_worker, application_task);
+  GzochidTask *task = gzochid_task_immediate_new
+    (gzochid_application_task_thread_worker,
+     (GDestroyNotify) gzochid_application_task_unref, application_task);
 
   gzochid_schedule_submit_task (client->closure->task_queue, task);
-
-  gzochid_task_free (task);
 }
 
 static void 
@@ -279,7 +292,8 @@ dispatch_login_request (gzochid_game_client *client, char *endpoint,
 			unsigned char *cred, short cred_len)
 {
   GError *error = NULL;
-
+  GzochidApplicationContext *app_context = NULL;
+  
   if (client->identity != NULL)
     {
       g_warning
@@ -288,9 +302,10 @@ dispatch_login_request (gzochid_game_client *client, char *endpoint,
       return;
     }
 
-  client->app_context = gzochid_game_server_lookup_application
+  app_context = gzochid_game_server_lookup_application
     (client->closure->game_server, endpoint);
-  if (client->app_context == NULL)
+
+  if (app_context == NULL)
     {
       g_warning
 	("Client at %s attempted to authenticate to unknown endpoint %s", 
@@ -302,9 +317,9 @@ dispatch_login_request (gzochid_game_client *client, char *endpoint,
       return;
     }
 
-  assert (client->app_context->authenticator != NULL);
-  client->identity = client->app_context->authenticator 
-    (cred, cred_len, client->app_context->auth_data, &error);
+  client->app_context = g_object_ref (app_context);
+  client->identity = gzochid_application_context_authenticate
+    (client->app_context, cred, cred_len, &error);
  
   if (client->identity == NULL)
     {
@@ -332,15 +347,18 @@ dispatch_login_request (gzochid_game_client *client, char *endpoint,
 /* Schedules the transactional stage of the disconnect process. */
 
 static void 
-disconnected (gzochid_application_context *context, gzochid_game_client *client)
+disconnected (GzochidApplicationContext *context, gzochid_game_client *client)
 {
   guint64 *session_oid = NULL;
+  gzochid_application_session_map *session_map = NULL;
 
-  g_mutex_lock (&context->client_mapping_lock);
-  session_oid = g_hash_table_lookup (context->clients_to_oids, client);
+  g_object_get (context, "session-map", &session_map, NULL);
+  
+  g_mutex_lock (&session_map->client_mapping_lock);
+  session_oid = g_hash_table_lookup (session_map->clients_to_oids, client);
   if (session_oid == NULL)
     {
-      g_mutex_unlock (&context->client_mapping_lock);
+      g_mutex_unlock (&session_map->client_mapping_lock);
       return;
     }
   else 
@@ -349,15 +367,16 @@ disconnected (gzochid_application_context *context, gzochid_game_client *client)
       gzochid_application_task *callback_task = 
 	gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client),
-	 gzochid_scheme_application_disconnected_worker, session_oid_copy);
+	 gzochid_scheme_application_disconnected_worker, NULL,
+	 session_oid_copy);
       gzochid_application_task *catch_task =
 	gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client),
-	 gzochid_client_session_disconnected_worker, session_oid_copy);
+	 gzochid_client_session_disconnected_worker, NULL, session_oid_copy);
       gzochid_application_task *cleanup_task = 
 	gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client), 
-	 gzochid_scheme_application_disconnected_cleanup_worker,
+	 gzochid_scheme_application_disconnected_cleanup_worker, NULL,
 	 session_oid_copy);
       gzochid_transactional_application_task_execution *execution = 
 	gzochid_transactional_application_task_timed_execution_new 
@@ -365,9 +384,11 @@ disconnected (gzochid_application_context *context, gzochid_game_client *client)
 	 client->closure->tx_timeout);
       gzochid_application_task *application_task = gzochid_application_task_new 
 	(context, gzochid_game_client_get_identity (client),
-	 gzochid_application_resubmitting_transactional_task_worker, execution);
+	 gzochid_application_resubmitting_transactional_task_worker,
+	 (GDestroyNotify) gzochid_transactional_application_task_execution_free,
+	 execution);
 
-      gzochid_task task;
+      GzochidTask *task = NULL;
       
       /* Not necessary to hold a ref to these, as we've transferred them to the
 	 execution. */
@@ -376,38 +397,42 @@ disconnected (gzochid_application_context *context, gzochid_game_client *client)
       gzochid_application_task_unref (catch_task);
       gzochid_application_task_unref (cleanup_task);
 
-      task.worker = gzochid_application_task_thread_worker;
-      task.data = application_task;
-      gettimeofday (&task.target_execution_time, NULL);
+      task = gzochid_task_immediate_new
+	(gzochid_application_task_thread_worker,
+	 (GDestroyNotify) gzochid_application_task_unref, application_task);
 
-      if (g_hash_table_contains (context->oids_to_clients, session_oid))
+      if (g_hash_table_contains (session_map->oids_to_clients, session_oid))
 	{
+	  GzochidMetaClient *metaclient =
+	    gzochid_application_context_get_metaclient (context);
+
 	  /* If this application server node is connected to a metaserver, let
 	     the metaserver know that the session is disconnecting. This isn't
 	     the only place that a client can be unmapped, but it should be the
 	     only place that it can happen to a client that was previously
 	     announced to the metaserver. */
 	  
-	  if (context->metaclient != NULL)
+	  if (metaclient != NULL)
 	    {
 	      GzochidSessionClient *sessionclient = NULL;
 
-	      g_object_get
-		(context->metaclient, "session-client", &sessionclient, NULL);
+	      g_object_get (metaclient, "session-client", &sessionclient, NULL);
 
 	      gzochid_sessionclient_session_disconnected
-		(sessionclient, context->descriptor->name, *session_oid);
+		(sessionclient, gzochid_application_context_get_name (context),
+		 *session_oid);
 	      
 	      g_object_unref (sessionclient);
+	      g_object_unref (metaclient);
 	    }
 
-	  g_hash_table_remove (context->oids_to_clients, session_oid);
-	  g_hash_table_remove (context->clients_to_oids, client);
+	  g_hash_table_remove (session_map->oids_to_clients, session_oid);
+	  g_hash_table_remove (session_map->clients_to_oids, client);
 	}
       
-      gzochid_schedule_submit_task (client->closure->task_queue, &task);
+      gzochid_schedule_submit_task (client->closure->task_queue, task);
 
-      g_mutex_unlock (&context->client_mapping_lock);
+      g_mutex_unlock (&session_map->client_mapping_lock);
     }
 }
 
@@ -426,7 +451,7 @@ dispatch_logout_request (gzochid_game_client *client)
 /* Cleanup handler for the received message event. */
 
 static void
-cleanup_received_message_arguments (gzochid_application_context *context,
+cleanup_received_message_arguments (GzochidApplicationContext *context,
 				    gzochid_auth_identity *identity,
 				    gpointer data)
 {
@@ -442,14 +467,17 @@ cleanup_received_message_arguments (gzochid_application_context *context,
 /* Schedules the transactional stage of the "received message" process. */
 
 static void 
-received_message (gzochid_application_context *context,
+received_message (GzochidApplicationContext *context,
 		  gzochid_game_client *client, unsigned char *msg, short len)
 {
   guint64 *session_oid = NULL;
+  gzochid_application_session_map *session_map = NULL;
 
-  g_mutex_lock (&context->client_mapping_lock);
-  session_oid = g_hash_table_lookup (context->clients_to_oids, client);
-  g_mutex_unlock (&context->client_mapping_lock);
+  g_object_get (context, "session-map", &session_map, NULL);
+
+  g_mutex_lock (&session_map->client_mapping_lock);
+  session_oid = g_hash_table_lookup (session_map->clients_to_oids, client);
+  g_mutex_unlock (&session_map->client_mapping_lock);
 
   if (session_oid == NULL)
     return;
@@ -461,18 +489,18 @@ received_message (gzochid_application_context *context,
       gzochid_application_task *cleanup_task = NULL;
       gzochid_application_task *application_task = NULL;
       gzochid_transactional_application_task_execution *execution = NULL;
-      gzochid_task task;
+      GzochidTask *task = NULL;
 
       data[0] = session_oid;
       data[1] = g_bytes_new (msg, len);
       
       transactional_task = gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client),
-	 gzochid_scheme_application_received_message_worker, data);
+	 gzochid_scheme_application_received_message_worker, NULL, data);
 
       cleanup_task = gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client),
-	 cleanup_received_message_arguments, data);
+	 cleanup_received_message_arguments, NULL, data);
       
       execution = gzochid_transactional_application_task_timed_execution_new
 	(transactional_task, NULL, cleanup_task,
@@ -486,13 +514,15 @@ received_message (gzochid_application_context *context,
       
       application_task = gzochid_application_task_new
 	(context, gzochid_game_client_get_identity (client),
-	 gzochid_application_resubmitting_transactional_task_worker, execution);
-      
-      task.worker = gzochid_application_task_thread_worker;
-      task.data = application_task;
-      gettimeofday (&task.target_execution_time, NULL);
+	 gzochid_application_resubmitting_transactional_task_worker,
+	 (GDestroyNotify) gzochid_transactional_application_task_execution_free,
+	 execution);
 
-      gzochid_schedule_submit_task (client->closure->task_queue, &task);
+      task = gzochid_task_immediate_new
+	(gzochid_application_task_thread_worker,
+	 (GDestroyNotify) gzochid_application_task_unref, application_task);
+
+      gzochid_schedule_submit_task (client->closure->task_queue, task);
     }
 }
 
@@ -506,10 +536,15 @@ dispatch_session_message (gzochid_game_client *client, unsigned char *msg,
 	 gzochid_client_socket_get_connection_description (client->sock));
   else
     {
-      gzochid_event_dispatch
-	(client->app_context->event_source,
-	 g_object_new (GZOCHID_TYPE_EVENT, "type", MESSAGE_RECEIVED, NULL));
+      gzochid_event_source *event_source = NULL;
+
+      g_object_get (client->app_context, "event-source", &event_source, NULL);
       
+      gzochid_event_dispatch
+	(event_source,
+	 g_object_new (GZOCHID_TYPE_EVENT, "type", MESSAGE_RECEIVED, NULL));
+
+      g_source_unref ((GSource *) event_source);      
       received_message (client->app_context, client, msg, len);
     }
 }
@@ -587,8 +622,12 @@ client_free (gpointer data)
 {
   gzochid_game_client *client = data;
 
+  if (client->app_context != NULL)
+    g_object_unref (client->app_context);
+  
   if (client->identity != NULL)
     gzochid_auth_identity_unref (client->identity);
+
   free (client);
 }
 
