@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,10 +23,12 @@
 #include <libguile.h>
 #include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
 #include "app.h"
+#include "app-store.h"
 #include "auth_int.h"
 #include "channel.h"
 #include "config.h"
@@ -35,14 +37,11 @@
 #include "game.h"
 #include "game-protocol.h"
 #include "gzochid.h"
-#include "gzochid-storage.h"
 #include "metaclient.h"
 #include "scheme.h"
 #include "scheme-task.h"
 #include "socket.h"
-#include "storage.h"
-#include "storage-dataclient.h"
-#include "storage-mem.h"
+#include "storageengine.h"
 #include "task.h"
 #include "threads.h"
 
@@ -64,13 +63,15 @@
 struct _GzochidGameServer
 {
   GObject parent_instance; /* The parent struct, for casting. */
-  
-  GThreadPool *pool; /* Thread pool for task queue. */
 
+  /* The default dependency resolution context. */
+
+  GzochidResolutionContext *resolution_context; 
+  
   /* Non-durable queue of tasks pending execution on behalf of running 
      applications. */
 
-  gzochid_task_queue *task_queue; 
+  GzochidTaskQueue *task_queue; 
   
   int port; /* Port on which the game server listens for connections. */
   char *apps_dir; /* Directory to scan for application deployments. */
@@ -81,13 +82,13 @@ struct _GzochidGameServer
   
   struct timeval tx_timeout; /* The default timeout for transactional tasks. */
 
-  /* Map of application name to `gzochid_application_context'. */
+  /* Map of application name to `GzochidApplicationContext'. */
 
   GHashTable *applications; 
 
-  /* The storage engine loaded by the game manager. */
+  /* The storage engine container. */
 
-  gzochid_storage_engine *storage_engine; 
+  GzochidStorageEngineContainer *storage_engine_container; 
 
   gzochid_server_socket *server_socket; /* The game protocol server socket. */
 
@@ -112,8 +113,11 @@ G_DEFINE_TYPE (GzochidGameServer, gzochid_game_server, G_TYPE_OBJECT);
 enum gzochid_game_server_properties
   {
     PROP_CONFIGURATION = 1,
+    PROP_RESOLUTION_CONTEXT,
+    PROP_TASK_QUEUE,
     PROP_SOCKET_SERVER,
     PROP_EVENT_LOOP,
+    PROP_STORAGE_ENGINE_CONTAINER,
     PROP_META_CLIENT_CONTAINER,
     PROP_AUTH_PLUGIN_REGISTRY,
     N_PROPERTIES
@@ -128,14 +132,9 @@ game_server_constructed (GObject *obj)
 
   GHashTable *config = gzochid_configuration_extract_group
     (self->configuration, "game");
-  int max_threads = gzochid_config_to_int 
-    (g_hash_table_lookup (config, "thread_pool.max_threads"), 4);
   long tx_timeout_ms = gzochid_config_to_long 
     (g_hash_table_lookup (config, "tx.timeout"), DEFAULT_TX_TIMEOUT_MS);
        
-  self->pool = gzochid_thread_pool_new (self, max_threads, TRUE, NULL);
-  self->task_queue = gzochid_schedule_task_queue_new (self->pool);
-
   self->port = gzochid_config_to_int
     (g_hash_table_lookup (config, "server.port"), 8001);
 
@@ -158,11 +157,17 @@ game_server_dispose (GObject *obj)
 {
   GzochidGameServer *self = GZOCHID_GAME_SERVER (obj);
 
+  /* Release all application references. */
+  
+  g_hash_table_remove_all (self->applications);
+  
   g_object_unref (self->auth_plugin_registry);
   g_object_unref (self->configuration);
   g_object_unref (self->event_loop);
   g_object_unref (self->socket_server);
-
+  g_object_unref (self->storage_engine_container);
+  g_object_unref (self->task_queue);
+  
   G_OBJECT_CLASS (gzochid_game_server_parent_class)->dispose (obj);
 }
 
@@ -187,6 +192,14 @@ game_server_set_property (GObject *obj, guint property_id, const GValue *value,
     case PROP_CONFIGURATION:
       self->configuration = g_object_ref (g_value_get_object (value));
       break;
+
+    case PROP_RESOLUTION_CONTEXT:
+      self->resolution_context = g_object_ref (g_value_get_object (value));
+      break;
+      
+    case PROP_TASK_QUEUE:
+      self->task_queue = g_object_ref (g_value_get_object (value));
+      break;
       
     case PROP_SOCKET_SERVER:
       self->socket_server = g_object_ref (g_value_get_object (value));
@@ -196,6 +209,11 @@ game_server_set_property (GObject *obj, guint property_id, const GValue *value,
       self->event_loop = g_object_ref (g_value_get_object (value));
       break;
 
+    case PROP_STORAGE_ENGINE_CONTAINER:
+      self->storage_engine_container = g_object_ref
+	(g_value_get_object (value));
+      break;
+      
     case PROP_META_CLIENT_CONTAINER:
       self->metaclient_container = g_object_ref (g_value_get_object (value));
       break;
@@ -220,6 +238,15 @@ gzochid_game_server_class_init (GzochidGameServerClass *klass)
     ("configuration", "config", "The global configuration object",
      GZOCHID_TYPE_CONFIGURATION, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
+  obj_properties[PROP_RESOLUTION_CONTEXT] = g_param_spec_object
+    ("resolution-context", "resolution-context",
+     "The global resolution context", GZOCHID_TYPE_RESOLUTION_CONTEXT,
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  
+  obj_properties[PROP_TASK_QUEUE] = g_param_spec_object
+    ("task-queue", "task-queue", "The global task queue",
+     GZOCHID_TYPE_TASK_QUEUE, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
   obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
     ("socket-server", "socket-server", "The client-facing socket server",
      GZOCHID_TYPE_SOCKET_SERVER, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
@@ -228,6 +255,11 @@ gzochid_game_server_class_init (GzochidGameServerClass *klass)
     ("event-loop", "event-loop", "The global event loop",
      GZOCHID_TYPE_EVENT_LOOP, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
+  obj_properties[PROP_STORAGE_ENGINE_CONTAINER] = g_param_spec_object
+    ("storag-engine-container", "storage-engine-container",
+     "The storage engine container", GZOCHID_TYPE_STORAGE_ENGINE_CONTAINER,
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+  
   obj_properties[PROP_META_CLIENT_CONTAINER] = g_param_spec_object
     ("metaclient-container", "metaclient-container", "The metaclient container",
      GZOCHID_TYPE_META_CLIENT_CONTAINER,
@@ -245,75 +277,10 @@ gzochid_game_server_class_init (GzochidGameServerClass *klass)
 void
 gzochid_game_server_init (GzochidGameServer *self)
 {
-  self->applications = g_hash_table_new (g_str_hash, g_str_equal);
+  self->applications = g_hash_table_new_full
+    (g_str_hash, g_str_equal, free, g_object_unref);
 }
 
-/* Bootstrap the storage engine to be used by all hosted applications. If the
-   gzochid application server is running in distributed mode, the "dataclient"
-   storage engine is used; otherwise the engine named in the `gzochid.conf' file
-   is loaded. If the requested storage engine cannot be loaded, the server will
-   fall back to a non-durable "in-memory" storage engine. */
-
-static void
-initialize_storage (GzochidGameServer *server)
-{
-  GHashTable *config = gzochid_configuration_extract_group
-    (server->configuration, "game");
-
-  GzochidMetaClient *metaclient = NULL;
-  
-  g_object_get (server->metaclient_container, "metaclient", &metaclient, NULL);
-
-  if (metaclient != NULL)
-    {
-      char *conf_storage_engine = g_hash_table_lookup
-	(config, "storage.engine");
-
-      if (conf_storage_engine != NULL)
-	g_message
-	  ("Meta server client configuration detected; ignoring configured "
-	   "storage engine '%s'.", conf_storage_engine);
-      else g_message
-	     ("Meta server client configuration detected; using data client "
-	      "storage engine.");
-
-      server->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
-      server->storage_engine->interface =
-	&gzochid_storage_engine_interface_dataclient;
-
-      g_object_unref (metaclient);
-    }
-  else if (g_hash_table_contains (config, "storage.engine"))
-    {
-      char *dir = NULL;
-      char *env = getenv ("GZOCHID_STORAGE_ENGINE_DIR");
-      
-      if (env != NULL)
-	dir = env;
-      else 
-	{
-	  char *conf_dir = g_hash_table_lookup (config, "storage.engine.dir");
-	  dir = conf_dir == NULL ? GZOCHID_STORAGE_ENGINE_DIR : conf_dir;
-	}
-
-      server->storage_engine = gzochid_storage_load_engine 
-	(dir, g_hash_table_lookup (config, "storage.engine"));
-    }
-  else g_message 
-	 ("No durable storage engine configured; memory engine will be used.");
-
-  if (server->storage_engine == NULL)
-    {
-      g_message
-	("Using in-memory storage for application data. THIS CONFIGURATION IS "
-	 "NOT SAFE FOR PRODUCTION USE.");
-
-      server->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
-      server->storage_engine->interface = &gzochid_storage_engine_interface_mem;
-    }
-
-  g_hash_table_destroy (config);
-}
 
 /* Adds a few core `gzochid_application_task_serialization' objects to the 
    global task serialization registry. */
@@ -329,33 +296,44 @@ initialize_application_task_serializations ()
   gzochid_task_register_serialization
     (&gzochid_channel_operation_task_serialization);
   gzochid_task_register_serialization
-    (&gzochid_task_chain_bootstrap_task_serialization);
+    (&gzochid_task_chain_coordinator_task_serialization);
 }
 
 /* Initialize the application corresponding to the specified 
    `GzochidApplicationDescriptor'. */
 
 static void 
-initialize_application (GzochidGameServer *server, const char *dir,
+initialize_application (GzochidGameServer *server,
 			GzochidApplicationDescriptor *descriptor)
 {
-  gzochid_application_context *application_context =
-    gzochid_application_context_new ();
-  
-  application_context->deployment_root = strdup (dir);
-  application_context->load_paths = g_list_prepend 
-    (g_list_copy (descriptor->load_paths), strdup (dir));
-  
-  gzochid_application_context_init
-    (application_context, descriptor, server->metaclient_container,
-     server->auth_plugin_registry, server->storage_engine->interface,
-     server->work_dir, server->task_queue, server->tx_timeout);
+  GzochidApplicationContext *app_context = NULL;
 
+  /* The application context expects to be injected with a host of global
+     dependencies, along with an application descriptor. Clone the default
+     resolution context to create a sandbox context just for this 
+     application... */
+  
+  GzochidResolutionContext *app_resolution_context =
+    gzochid_resolver_clone (server->resolution_context);
+
+  /* ...inject the descriptor into that context... */
+  
+  gzochid_resolver_provide
+    (app_resolution_context, G_OBJECT (descriptor), NULL);  
+
+  /* ...and wire up the application! */
+  
+  app_context = gzochid_resolver_require_full
+    (app_resolution_context, GZOCHID_TYPE_APPLICATION_CONTEXT, NULL);
+  
   g_hash_table_insert
-    (server->applications, descriptor->name, application_context);
+    (server->applications, strdup (descriptor->name),
+     g_object_ref (app_context));
 
-  gzochid_event_source_attach
-    (server->event_loop, application_context->event_source);
+  /* We can throw away the per-application context once we've wired the 
+     application. */
+  
+  g_object_unref (app_resolution_context);
 }
 
 /* Attempt to bootstrap an application from the specified directory by searching
@@ -365,7 +343,6 @@ initialize_application (GzochidGameServer *server, const char *dir,
 static void 
 scan_app_dir (GzochidGameServer *server, const char *dir)
 {
-  FILE *descriptor_file = NULL;
   char *descriptor_filename = g_strconcat (dir, "/", GAME_DESCRIPTOR_XML, NULL);
 
   GzochidApplicationDescriptor *descriptor = NULL;
@@ -378,9 +355,8 @@ scan_app_dir (GzochidGameServer *server, const char *dir)
       return;
     }
 
-  descriptor_file = fopen (descriptor_filename, "r");
-  descriptor = gzochid_config_parse_application_descriptor (descriptor_file);
-  fclose (descriptor_file);
+  descriptor = gzochid_config_parse_application_descriptor_from_file
+    (descriptor_filename);
 
   if (descriptor == NULL)
     g_warning
@@ -390,7 +366,7 @@ scan_app_dir (GzochidGameServer *server, const char *dir)
     g_warning
       ("Application in %s with name '%s' already exists; skipping.", 
        descriptor_filename, descriptor->name);
-  else initialize_application (server, dir, descriptor);
+  else initialize_application (server, descriptor);
 
   g_free (descriptor_filename);
 }
@@ -495,7 +471,6 @@ gzochid_game_server_start (GzochidGameServer *server, GError **err)
 
   gzochid_schedule_task_queue_start (server->task_queue);
 
-  initialize_storage (server);
   initialize_apps (server);
   
   server->server_socket = gzochid_server_socket_new
@@ -507,7 +482,7 @@ gzochid_game_server_start (GzochidGameServer *server, GError **err)
     (server->socket_server, server->server_socket, server->port);
 }
 
-gzochid_application_context *
+GzochidApplicationContext *
 gzochid_game_server_lookup_application (GzochidGameServer *server,
 					const char *name)
 {
