@@ -1,5 +1,5 @@
 /* gzochid-migrate.c: Utility for migrating game data schema
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -29,12 +29,14 @@
 #include <sys/time.h>
 
 #include "app.h"
+#include "app-store.h"
 #include "config.h"
 #include "data.h"
 #include "descriptor.h"
 #include "guile.h"
 #include "gzochid-storage.h"
 #include "reloc.h"
+#include "resolver.h"
 #include "scheme.h"
 #include "scheme-task.h"
 #include "toollib.h"
@@ -73,7 +75,7 @@ gzochid_root_context_get_type ()
 
 struct migration 
 {
-  gzochid_application_context *context;
+  GzochidApplicationContext *context;
   struct migration_descriptor *descriptor;
   gboolean dry_run;
   gboolean explicit_rollback;
@@ -413,10 +415,12 @@ append_load_paths (GList *load_paths)
 }
 
 static struct migration *
-create_migration (gzochid_application_context *context, 
+create_migration (GzochidApplicationContext *context, 
 		  struct migration_descriptor *md, gboolean dry_run, 
 		  gboolean quiet)
-{ 
+{
+  GzochidApplicationDescriptor *descriptor = NULL;
+  gzochid_application_store *app_store = NULL;
   struct migration *m = malloc (sizeof (struct migration));
   char *scratch_dir_template = strdup ("gzochid-migrate-XXXXXXX");
   char *scratch_dir = g_dir_make_tmp (scratch_dir_template, NULL);
@@ -431,7 +435,12 @@ create_migration (gzochid_application_context *context,
   assert (md->callback_name != NULL && strlen (md->callback_name) > 0);
   assert (md->callback_module != NULL && strlen (md->callback_module) > 0);
 
-  append_load_paths (context->load_paths);
+  g_object_get (context,
+		"descriptor", &descriptor,
+		"app-store", &app_store,
+		NULL);
+  
+  append_load_paths (descriptor->load_paths);
   append_load_paths (md->load_paths);
 
   m->context = context;
@@ -440,9 +449,12 @@ create_migration (gzochid_application_context *context,
   m->explicit_rollback = FALSE;
   m->quiet = quiet;
   m->pending_oids = g_queue_new ();
+
+  m->num_visited = 0;
+  m->num_transformed = 0;
+  m->num_removed = 0;
   
-  m->visited_oids = create_oid_scratch_storage 
-    (context->storage_engine_interface, scratch_dir);
+  m->visited_oids = create_oid_scratch_storage (app_store->iface, scratch_dir);
   m->visited_oids_path = scratch_dir;
 
   m->callback = resolve_or_die (md->callback_module, md->callback_name);
@@ -469,26 +481,22 @@ create_migration (gzochid_application_context *context,
   scm_gc_protect_object (m->output_registry);
   scm_gc_protect_object (m->callback);
 
+  g_object_unref (descriptor);
+  
   return m;
 }
 
-static void 
-cleanup_application_context (gzochid_application_context *context)
-{
-  context->storage_engine_interface->close_store (context->meta);
-  context->storage_engine_interface->close_store (context->names);
-  context->storage_engine_interface->close_store (context->oids);
-
-  gzochid_application_context_free (context);
-}
-
 static void
 cleanup_migration (struct migration *m)
 {
+  gzochid_application_store *app_store = NULL;
+
+  g_object_get (m->context, "app-store", &app_store, NULL);
+  
   cleanup_oids_scratch_storage 
-    (m->context->storage_engine_interface, m->visited_oids,
-     m->visited_oids_path);
-  cleanup_application_context (m->context);
+    (app_store->iface, m->visited_oids, m->visited_oids_path);
+
+  g_object_unref (m->context);
 
   free (m->visited_oids_path);
 
@@ -503,47 +511,47 @@ cleanup_migration (struct migration *m)
   free (m);
 }
 
-static gzochid_storage_store *
-open_store (gzochid_application_context *context, char *path, char *db)
-{
-  gchar *filename = g_strconcat (path, "/", db, NULL);
-  gzochid_storage_store *store = gzochid_tool_open_store 
-    (context->storage_engine_interface, context->storage_context, filename);
-
-  g_free (filename);
-  return store;
-}
-
-static gzochid_application_context *
+static GzochidApplicationContext *
 create_application_context (char *path, char *app)
 {
-  GHashTable *config = NULL; 
-  FILE *descriptor_file = NULL;
-
+  GError *err = NULL;
+  GKeyFile *key_file = g_key_file_new ();
+  GzochidConfiguration *configuration = NULL;
+  GHashTable *game_config = NULL;
+  GzochidResolutionContext *resolution_context =
+    g_object_new (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
+  GzochidApplicationDescriptor *descriptor = NULL;
+  
   const char *env = getenv ("GZOCHID_CONF_LOCATION");
-  const char *gzochid_conf = NULL;
+  const char *conf_path = NULL;
 
-  gzochid_storage_engine *storage_engine = NULL;
-  gzochid_application_context *context = gzochid_application_context_new ();
-  char *work_dir = NULL, *data_dir = NULL, *apps_dir = NULL,
-    *descriptor_path = NULL, *engine_name = NULL;
+  GzochidApplicationContext *context = NULL;
+  char *apps_dir = NULL, *descriptor_path = NULL;
 
+  /* Construct the `GzochidConfiguration' and `GzochidApplicationDescriptor'
+     objects based on the specified path and app name... */
+  
   if (path != NULL)
-    gzochid_conf = path;
+    conf_path = path;
   else if (env != NULL)
-    gzochid_conf = env;
-  else gzochid_conf = QUOTE (GZOCHID_CONF_LOCATION);
+    conf_path = env;
+  else conf_path = QUOTE (GZOCHID_CONF_LOCATION);
 
-  config = gzochid_tool_load_game_config (gzochid_conf);
+  g_key_file_load_from_file (key_file, conf_path, G_KEY_FILE_NONE, &err);
 
-  if (g_hash_table_contains (config, "server.fs.data"))
-    work_dir = strdup (g_hash_table_lookup (config, "server.fs.data"));
-  else work_dir = strdup (SERVER_FS_DATA_DEFAULT);
+  if (err != NULL)
+    {
+      g_critical ("Failed to open %s: %s", conf_path, err->message);
+      exit (EXIT_FAILURE);
+    }
 
-  data_dir = g_strconcat (work_dir, "/", app, NULL);  
+  configuration = g_object_new
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, "path", conf_path, NULL);
 
-  if (g_hash_table_contains (config, "server.fs.apps"))    
-    apps_dir = strdup (g_hash_table_lookup (config, "server.fs.apps"));
+  game_config = gzochid_configuration_extract_group (configuration, "game");
+  
+  if (g_hash_table_contains (game_config, "server.fs.apps"))    
+    apps_dir = strdup (g_hash_table_lookup (game_config, "server.fs.apps"));
   else apps_dir = strdup (SERVER_FS_APPS_DEFAULT);
 
   descriptor_path = g_build_filename (apps_dir, app, GAME_DESCRIPTOR_XML, NULL);
@@ -553,66 +561,44 @@ create_application_context (char *path, char *app)
       (apps_dir, app, GAME_DESCRIPTOR_XML, NULL);
   else
     {
-      gchar *basedir = g_path_get_dirname (gzochid_conf);
+      gchar *basedir = g_path_get_dirname (conf_path);
       descriptor_path = g_build_filename
 	(basedir, apps_dir, app, GAME_DESCRIPTOR_XML, NULL);
       g_free (basedir);
     }
   
-  descriptor_file = fopen (descriptor_path, "r");
-  if (descriptor_file == NULL)
-    {
-      g_critical
-	("Failed to open application descriptor in %s", descriptor_path);
-      exit (EXIT_FAILURE);
-    }
-  
-  context->descriptor =
-    gzochid_config_parse_application_descriptor (descriptor_file);
+  descriptor = gzochid_config_parse_application_descriptor_from_file
+    (descriptor_path);
 
-  fclose (descriptor_file);
-
-  if (context->descriptor == NULL)
+  if (descriptor == NULL)
     {
       g_critical
 	("Failed to parse application descriptor in %s", descriptor_path);
       exit (EXIT_FAILURE);
     }
 
-  context->deployment_root = g_path_get_dirname (descriptor_path);
-  context->load_paths = g_list_prepend
-    (g_list_copy (context->descriptor->load_paths),
-     strdup (context->deployment_root));
+  /* 
+     ...and then inject the config and the descriptor into a resolution context
+     so we can bootstrap the application context and all related bits of the
+     object ecosystem.
 
-  if (g_hash_table_contains (config, "storage.engine"))
-    engine_name = strdup (g_hash_table_lookup (config, "storage.engine"));
-  else
-    {
-      g_critical ("storage.engine is required.");
-      exit (EXIT_FAILURE);
-    }
+     This is like a sandboxed version of what the game server does when
+     bootstrapping an application.
+  */
+  
+  gzochid_resolver_provide (resolution_context, G_OBJECT (configuration), NULL);
+  gzochid_resolver_provide (resolution_context, G_OBJECT (descriptor), NULL);
 
-  storage_engine = gzochid_tool_probe_storage_engine (config, engine_name);
+  context = gzochid_resolver_require_full
+    (resolution_context, GZOCHID_TYPE_APPLICATION_CONTEXT, NULL);
 
-  context->storage_engine_interface = storage_engine->interface;
-  context->storage_context = context->storage_engine_interface
-    ->initialize (data_dir);
-
-  if (context->storage_context == NULL)
-    {
-      fprintf (stderr, "Failed to initialize store in %s", data_dir);
-      exit (EXIT_FAILURE);
-    }
-
-  context->meta = open_store (context, data_dir, "meta");
-  context->names = open_store (context, data_dir, "names");
-  context->oids = open_store (context, data_dir, "oids");
-
-  free (work_dir);
-  free (data_dir);
+  g_object_unref (descriptor);
+  g_hash_table_destroy (game_config);
+  g_object_unref (configuration);
+  
   free (apps_dir);
   free (descriptor_path);
-  free (engine_name);
+  g_key_file_unref (key_file);
 
   return context;
 }
@@ -752,20 +738,25 @@ migrate_object_tx (gpointer data)
 static void 
 migrate_object (struct migration *m, guint64 oid)
 {
+  gzochid_application_store *app_store = NULL;
   gpointer args[2];
 
+  gzochid_storage_transaction *tx = NULL;
+  guint64 encoded_oid = gzochid_util_encode_oid (oid);
+  char *val = NULL;
+
   args[0] = m;
   args[1] = &oid;
 
-  gzochid_storage_transaction *tx = m->context->storage_engine_interface
-    ->transaction_begin (m->context->storage_context);
-  guint64 encoded_oid = gzochid_util_encode_oid (oid);
-  char *val = m->context->storage_engine_interface->transaction_get
+  g_object_get (m->context, "app-store", &app_store, NULL);
+
+  tx = app_store->iface->transaction_begin (app_store->storage_context);
+  val = app_store->iface->transaction_get
     (tx, m->visited_oids, (char *) &encoded_oid, sizeof (guint64), NULL);
-      
+  
   if (val == NULL)
     {
-      m->context->storage_engine_interface->transaction_put
+      app_store->iface->transaction_put
 	(tx, m->visited_oids, (char *) &encoded_oid, sizeof (guint64), "", 1);
       if (gzochid_transaction_execute (migrate_object_tx, args) 
 	  != GZOCHID_TRANSACTION_SUCCESS && !m->explicit_rollback)
@@ -776,8 +767,8 @@ migrate_object (struct migration *m, guint64 oid)
     }
   else free (val);
 
-  m->context->storage_engine_interface->transaction_prepare (tx);
-  m->context->storage_engine_interface->transaction_commit (tx);
+  app_store->iface->transaction_prepare (tx);
+  app_store->iface->transaction_commit (tx);
 }
 
 struct datum
@@ -790,17 +781,20 @@ static struct datum
 next_key (struct migration *m, struct datum last_key)
 {
   struct datum key = { NULL, 0 };
-  gzochid_storage_transaction *tx = m->context->storage_engine_interface
-    ->transaction_begin (m->context->storage_context);
+  gzochid_application_store *app_store = NULL;
+  gzochid_storage_transaction *tx = NULL;
+
+  g_object_get (m->context, "app-store", &app_store, NULL);
+  tx = app_store->iface->transaction_begin (app_store->storage_context);
   
   if (last_key.data == NULL)
-    key.data = m->context->storage_engine_interface
-      ->transaction_next_key (tx, m->context->names, "o.", 2, &key.data_len);
-  else key.data = m->context->storage_engine_interface
-	 ->transaction_next_key (tx, m->context->names, last_key.data,
-				 last_key.data_len, &key.data_len);
+    key.data = app_store->iface->transaction_next_key
+      (tx, app_store->names, "o.", 2, &key.data_len);
+  else key.data = app_store->iface->transaction_next_key
+	 (tx, app_store->names, last_key.data, last_key.data_len,
+	  &key.data_len);
 
-  m->context->storage_engine_interface->transaction_rollback (tx);
+  app_store->iface->transaction_rollback (tx);
   
   if (key.data != NULL && strncmp ("o.", key.data, 2) != 0)
     {
@@ -814,8 +808,11 @@ next_key (struct migration *m, struct datum last_key)
 static void
 run_migration (struct migration *m)
 {
+  gzochid_application_store *app_store = NULL;
   struct datum key = { NULL, 0 }, last_key = { NULL, 0 };
 
+  g_object_get (m->context, "app-store", &app_store, NULL);
+  
   while (TRUE)
     {
       guint64 *oid = NULL;
@@ -833,16 +830,15 @@ run_migration (struct migration *m)
 	    break;
 	  else
 	    {
-	      gzochid_storage_transaction *tx =
-		m->context->storage_engine_interface->transaction_begin
-		(m->context->storage_context);
+	      gzochid_storage_transaction *tx = app_store->iface
+		->transaction_begin (app_store->storage_context);
 
 	      size_t oid_len = 0;
 	      char *oid_bytes = NULL;
 	      guint64 encoded_oid = 0;
 	      
-	      oid_bytes = m->context->storage_engine_interface->transaction_get 
-		(tx, m->context->names, key.data, key.data_len, &oid_len);
+	      oid_bytes = app_store->iface->transaction_get 
+		(tx, app_store->names, key.data, key.data_len, &oid_len);
 
 	      assert (oid_len == sizeof (guint64));
 
@@ -851,7 +847,7 @@ run_migration (struct migration *m)
 	      oid = malloc (sizeof (guint64));
 	      *oid = gzochid_util_decode_oid (encoded_oid);
 	      
-	      m->context->storage_engine_interface->transaction_rollback (tx);
+	      app_store->iface->transaction_rollback (tx);
 	    }
 	}
       
