@@ -37,17 +37,8 @@ struct _gzochid_game_context;
 
 /* Boilerplate setup for the gzochid root context. */
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_root_context_get_type (void);
-
-struct _GzochidRootContextClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidRootContextClass GzochidRootContextClass;
+G_DECLARE_FINAL_TYPE (GzochidRootContext, gzochid_root_context, GZOCHID,
+                      ROOT_CONTEXT, GObject);
 
 /* The root context object. */
 
@@ -79,14 +70,4 @@ struct _GzochidRootContext
   gzochid_context *admin_context; /* The admin service context. */
 };
 
-typedef struct _GzochidRootContext GzochidRootContext;
-
-static inline GzochidRootContext *
-GZOCHID_ROOT_CONTEXT (gconstpointer ptr) {
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_root_context_get_type (), GzochidRootContext);
-}
-
-/* End boilerplate. */
-
 #endif /* GZOCHID_H */
