@@ -1,5 +1,5 @@
 /* httpd-app.c: Handler implementations for the gzochid admin web console
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <string.h>
 
 #include "app.h"
+#include "app-store.h"
 #include "config.h"
 #include "event.h"
 #include "event-app.h"
@@ -31,6 +32,7 @@
 #include "httpd.h"
 #include "httpd-app.h"
 #include "resolver.h"
+#include "stats.h"
 #include "util.h"
 
 #define HEADER "  <head><title>gzochid v" VERSION "</title></head>"
@@ -359,12 +361,13 @@ list_apps (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
       
       while (apps_ptr != NULL)
 	{
-	  gzochid_application_context *app = apps_ptr->data;
+	  GzochidApplicationContext *app = apps_ptr->data;
+	  const char *app_name = gzochid_application_context_get_name (app);
 	  
   	  g_string_append_printf 
 	    (response_str, 
 	     "      <li><a href=\"/app/%s/\">%s</a></li><br />\n", 
-	     app->descriptor->name, app->descriptor->name);
+	     app_name, app_name);
 
 	  apps_ptr = apps_ptr->next;
 	}
@@ -386,7 +389,7 @@ bind_app (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 {
   gchar *app = g_match_info_fetch (match_info, 1);
   GzochidGameServer *game_server = user_data;
-  gzochid_application_context *app_context =
+  GzochidApplicationContext *app_context =
     gzochid_game_server_lookup_application (game_server, app);
 
   g_free (app);
@@ -402,15 +405,19 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	  gpointer request_context, gpointer user_data)
 {
   GString *response_str = g_string_new (NULL);
-  gzochid_application_context *app_context = request_context;
+  GzochidApplicationContext *app_context = request_context;
+  GzochidApplicationStats *app_stats = NULL;
+  const char *app_name = gzochid_application_context_get_name (app_context);
   gzochid_server_state *state = user_data;
 
+  g_object_get (app_context, "app-stats", &app_stats, NULL);
+  
   append_header (response_str);
 
+  g_string_append_printf (response_str, "    <h1>%s</h1>\n", app_name);
   g_string_append_printf 
-    (response_str, "    <h1>%s</h1>\n", app_context->descriptor->name);
-  g_string_append_printf 
-    (response_str, "    <p>%s</p>\n", app_context->descriptor->description);
+    (response_str, "    <p>%s</p>\n",
+     gzochid_application_context_get_description (app_context));
 
   /* Shouldn't show users links to dump contents of store if we're in 
      distributed mode. */
@@ -420,10 +427,10 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
       g_string_append (response_str, "    <h2>Application data</h2>\n");
       g_string_append_printf 
 	(response_str, "    <a href=\"/app/%s/names/\">names</a><br />\n", 
-	 app_context->descriptor->name);
+	 app_name);
       g_string_append_printf 
 	(response_str, "    <a href=\"/app/%s/oids/\">oids</a><br />\n", 
-	 app_context->descriptor->name);
+	 app_name);
     }
   
   g_string_append (response_str, "    <h2>Application statistics</h2>\n");
@@ -432,58 +439,58 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Bytes read</td>\n");
   g_string_append_printf (response_str, "        <td>%lu</td>\n", 
-			  app_context->stats->bytes_read);
+			  app_stats->bytes_read);
   g_string_append (response_str, "      </tr>\n");
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Bytes written</td>\n");
   g_string_append_printf (response_str, "        <td>%lu</td>\n", 
-			  app_context->stats->bytes_written);
+			  app_stats->bytes_written);
   g_string_append (response_str, "      </tr>\n");
 
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Messages received</td>\n");
   g_string_append_printf (response_str, "        <td>%u</td>\n", 
-			  app_context->stats->num_messages_received);
+			  app_stats->num_messages_received);
   g_string_append (response_str, "      </tr>\n");
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Messages sent</td>\n");
   g_string_append_printf (response_str, "        <td>%u</td>\n", 
-			  app_context->stats->num_messages_sent);
+			  app_stats->num_messages_sent);
   g_string_append (response_str, "      </tr>\n");
 
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Transactions started</td>\n");
   g_string_append_printf (response_str, "        <td>%u</td>\n", 
-			  app_context->stats->num_transactions_started);
+			  app_stats->num_transactions_started);
   g_string_append (response_str, "      </tr>\n");
   g_string_append (response_str, "      <tr>\n");
   g_string_append (response_str, "        <td>Transactions committed</td>\n");
   g_string_append_printf (response_str, "        <td>%u</td>\n", 
-			  app_context->stats->num_transactions_committed);
+			  app_stats->num_transactions_committed);
   g_string_append (response_str, "      </tr>\n");
   g_string_append (response_str, "      <tr>\n");
   g_string_append 
     (response_str, "        <td>Transactions rolled back</td>\n");
   g_string_append_printf (response_str, "        <td>%u</td>\n", 
-			  app_context->stats->num_transactions_rolled_back);
+			  app_stats->num_transactions_rolled_back);
   g_string_append (response_str, "      </tr>\n");
 
-  if (app_context->stats->num_transactions_committed > 0)
+  if (app_stats->num_transactions_committed > 0)
     {
       g_string_append (response_str, "      <tr>\n");
       g_string_append
 	(response_str, "        <td>Maximum transaction duration</td>\n");
       g_string_append_printf 
-	(response_str, "        <td>%ld</td>\n", 
-	 app_context->stats->max_transaction_duration);
+	(response_str, "        <td>%ld</td>\n",
+	 app_stats->max_transaction_duration);
       g_string_append (response_str, "      </tr>\n");
 
       g_string_append (response_str, "      <tr>\n");
       g_string_append
 	(response_str, "        <td>Minimum transaction duration</td>\n");
       g_string_append_printf 
-	(response_str, "        <td>%ld</td>\n", 
-	 app_context->stats->min_transaction_duration);
+	(response_str, "        <td>%ld</td>\n",
+	 app_stats->min_transaction_duration);
       g_string_append (response_str, "      </tr>\n");
 
       g_string_append (response_str, "      <tr>\n");
@@ -491,10 +498,29 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	(response_str, "        <td>Average transaction duration</td>\n");
       g_string_append_printf 
 	(response_str, "        <td>%.2f</td>\n", 
-	 app_context->stats->average_transaction_duration);
+	 app_stats->average_transaction_duration);
       g_string_append (response_str, "      </tr>\n");
     }
 
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Tasks assigned</td>\n");
+  g_string_append_printf
+    (response_str, "        <td>%ld</td>\n",
+     app_stats->tasks_submitted + app_stats->tasks_assigned);
+  g_string_append (response_str, "      </tr>\n");
+
+  g_string_append (response_str, "      <tr>\n");
+  g_string_append (response_str, "        <td>Tasks completed</td>\n");
+  g_string_append_printf (response_str, "        <td>%ld</td>\n",
+			  app_stats->tasks_completed);
+  g_string_append (response_str, "      </tr>\n");
+  g_string_append (response_str, "      <tr>\n");
+
+  g_string_append (response_str, "        <td>Tasks unassigned</td>\n");
+  g_string_append_printf (response_str, "        <td>%ld</td>\n",
+			  app_stats->tasks_unassigned);
+  g_string_append (response_str, "      </tr>\n");
+
   g_string_append (response_str, "    </table>\n");
   append_footer (response_str);
 
@@ -502,28 +528,33 @@ app_info (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
     (sink, 200, response_str->str, response_str->len);
 
   g_string_free (response_str, TRUE);
+  g_boxed_free (GZOCHID_TYPE_APPLICATION_STATS, app_stats);
 }
 
 static void
 list_oids (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	   gpointer request_context, gpointer user_data)
 {
-  gzochid_application_context *app_context = request_context;
+  GzochidApplicationContext *app_context = request_context;
   GString *response_str = g_string_new (NULL);
   
   size_t klen = 0;
-  gzochid_storage_transaction *tx = app_context->storage_engine_interface
-    ->transaction_begin (app_context->storage_context);
-  char *k = app_context->storage_engine_interface
-    ->transaction_first_key (tx, app_context->oids, &klen);
+  gzochid_application_store *store = NULL;
+  gzochid_storage_transaction *tx = NULL;
+  char *k = NULL;
 
-  app_context->storage_engine_interface->transaction_rollback (tx);
+  g_object_get (app_context, "app-store", &store, NULL);
+  
+  tx = store->iface->transaction_begin (store->storage_context);
+  k = store->iface->transaction_first_key (tx, store->oids, &klen);
+  store->iface->transaction_rollback (tx);
 
   assert (klen == sizeof (guint64));  
 
   append_header (response_str);
   g_string_append_printf 
-    (response_str, "    <h1>%s</h1><br />\n", app_context->descriptor->name);
+    (response_str, "    <h1>%s</h1><br />\n",
+     gzochid_application_context_get_name (app_context));
   g_string_append (response_str, "    <ul>\n");
   
   while (k != NULL)
@@ -543,11 +574,10 @@ list_oids (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 	 gzochid_util_decode_oid (encoded_oid));
       g_string_append (response_str, "</a></li>\n");
 
-      tx = app_context->storage_engine_interface
-	->transaction_begin (app_context->storage_context);      
-      next_k = app_context->storage_engine_interface->transaction_next_key
-	(tx, app_context->oids, k, klen, &klen);
-      app_context->storage_engine_interface->transaction_rollback (tx);
+      tx = store->iface->transaction_begin (store->storage_context);      
+      next_k = store->iface->transaction_next_key
+	(tx, store->oids, k, klen, &klen);
+      store->iface->transaction_rollback (tx);
       
       free (k);
       k = next_k;
@@ -568,14 +598,18 @@ render_oid (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
 {
   size_t data_length = 0;
   char *oid_str = g_match_info_fetch (match_info, 1);
-  gzochid_application_context *app_context = request_context;
-  gzochid_storage_transaction *tx = app_context->storage_engine_interface
-    ->transaction_begin (app_context->storage_context);
-  guint64 oid = gzochid_util_encode_oid (g_ascii_strtoull (oid_str, NULL, 16)); 
-  char *data = app_context->storage_engine_interface->transaction_get
-    (tx, app_context->oids, (char *) &oid, sizeof (guint64), &data_length);
-  
-  app_context->storage_engine_interface->transaction_rollback (tx);
+  GzochidApplicationContext *app_context = request_context;
+  gzochid_application_store *store = NULL;
+  gzochid_storage_transaction *tx = NULL;
+  guint64 oid = gzochid_util_encode_oid (g_ascii_strtoull (oid_str, NULL, 10)); 
+  char *data = NULL;
+
+  g_object_get (app_context, "app-store", &store, NULL);
+
+  tx = store->iface->transaction_begin (store->storage_context);
+  data = store->iface->transaction_get
+    (tx, store->oids, (char *) &oid, sizeof (guint64), &data_length);
+  store->iface->transaction_rollback (tx);
   
   free (oid_str);
 
@@ -607,17 +641,21 @@ list_names (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   GString *response_str = g_string_new (NULL);
   
   size_t klen = 0;
-  gzochid_application_context *app_context = request_context;
-  gzochid_storage_transaction *tx = app_context->storage_engine_interface
-    ->transaction_begin (app_context->storage_context);
-  char *k = app_context->storage_engine_interface
-    ->transaction_first_key (tx, app_context->names, &klen);
+  GzochidApplicationContext *app_context = request_context;
+  gzochid_application_store *store = NULL;
+  gzochid_storage_transaction *tx = NULL;
+  char *k = NULL;
 
-  app_context->storage_engine_interface->transaction_rollback (tx);
+  g_object_get (app_context, "app-store", &store, NULL);
+
+  tx = store->iface->transaction_begin (store->storage_context);
+  k = store->iface->transaction_first_key (tx, store->names, &klen);
+  store->iface->transaction_rollback (tx);
 
   append_header (response_str);
   g_string_append_printf 
-    (response_str, "    <h1>%s</h1><br />\n", app_context->descriptor->name);
+    (response_str, "    <h1>%s</h1><br />\n",
+     gzochid_application_context_get_name (app_context));
   g_string_append (response_str, "    <ul>\n");
 
   while (k != NULL)
@@ -628,11 +666,10 @@ list_names (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
       g_string_append_len (response_str, k, klen);
       g_string_append (response_str, "</li>\n");
 
-      tx = app_context->storage_engine_interface
-	->transaction_begin (app_context->storage_context);      
-      next_k = app_context->storage_engine_interface->transaction_next_key
-	(tx, app_context->names, k, klen, &klen);
-      app_context->storage_engine_interface->transaction_rollback (tx);
+      tx = store->iface->transaction_begin (store->storage_context);      
+      next_k = store->iface->transaction_next_key
+	(tx, store->names, k, klen, &klen);
+      store->iface->transaction_rollback (tx);
       
       free (k);
       k = next_k;
@@ -641,8 +678,7 @@ list_names (const GMatchInfo *match_info, gzochid_http_response_sink *sink,
   g_string_append (response_str, "    </ul>\n");
   append_footer (response_str);
 
-  gzochid_http_write_response
-    (sink, 200, response_str->str, response_str->len);
+  gzochid_http_write_response (sink, 200, response_str->str, response_str->len);
 
   g_string_free (response_str, TRUE);
 }
