@@ -1,5 +1,5 @@
 /* httpd-meta.c: Handler implementations for the gzochi-metad admin web console
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "httpd.h"
 #include "httpd-meta.h"
 #include "sessionserver.h"
+#include "taskserver.h"
 
 #define HEADER "  <head><title>gzochi-metad v" VERSION "</title></head>"
 
@@ -52,7 +53,31 @@ struct _gzochi_metad_client_info
 
   /* The number of sessions connected to this client across all applications. */
 
-  guint num_connected_sessions; 
+  guint num_connected_sessions;
+
+  /*
+    The number of tasks active on this client across all applications.
+    
+    Computed as the number of incoming tasks (see below) minus the number of
+    tasks completed locally or unassigned.
+  */
+
+  guint num_active_tasks;
+  
+  /* The number of tasks assigned to or submitted by this client across all 
+     applications. */
+  
+  guint num_incoming_tasks;
+  
+  /* The number of assigned tasks that were completed on this client, across all
+     applications. */
+  
+  guint num_completed_tasks;
+
+  /* The number of previously-assigned tasks that were unassigned from this 
+     client, across all applications. */
+  
+  guint num_unassigned_tasks;
 };
 
 typedef struct _gzochi_metad_client_info gzochi_metad_client_info;
@@ -80,6 +105,11 @@ gzochi_metad_client_info_new (guint node_id, const char *connection_description,
     g_time_val_to_iso8601 (&connection_timestamp);
 
   info->num_connected_sessions = 0;
+
+  info->num_active_tasks = 0;
+  info->num_incoming_tasks = 0;
+  info->num_completed_tasks = 0;
+  info->num_unassigned_tasks = 0;
   
   return info;
 }
@@ -182,6 +212,11 @@ render_server (gpointer key, gpointer value, gpointer user_data)
 
   g_string_append_printf
     (response_str, "  <td>%d</td>\n", info->num_connected_sessions);
+
+  g_string_append_printf
+    (response_str, "  <td>%d (%d/%d/%d)</td>\n", info->num_active_tasks,
+     info->num_incoming_tasks, info->num_completed_tasks,
+     info->num_unassigned_tasks);
   
   g_string_append (response_str, "</tr>\n");
 }
@@ -202,6 +237,9 @@ list_servers (GString *response_str, const gzochi_metad_server_state *state)
   g_string_append (response_str, "    <th>server address</th>\n");
   g_string_append (response_str, "    <th>connection timestamp</th>\n");
   g_string_append (response_str, "    <th># of connected sessions</th>\n");
+  g_string_append
+    (response_str,
+     "    <th>active tasks (incoming/completed/unassigned)</th>\n");
   g_string_append (response_str, "  </tr>\n");
   
   g_hash_table_foreach (state->connected_servers, render_server, response_str);
@@ -341,6 +379,54 @@ handle_session_event (GzochidEvent *event, gpointer user_data)
   g_mutex_unlock (&state->mutex);
 }
 
+/* `gzochid_event_handler' implementation to update the admin console's
+   representation of the meta server state in response to task events. */
+
+static void
+handle_task_event (GzochidEvent *event, gpointer user_data)
+{
+  gzochi_metad_server_state *state = user_data;  
+  GzochiMetadTaskEvent *task_event = GZOCHI_METAD_TASK_EVENT (event);
+  gzochi_metad_client_info *info = NULL;
+  gzochi_metad_task_event_type type;
+  guint node_id = 0;
+
+  g_object_get (task_event, "type", &type, "node-id", &node_id, NULL);
+
+  info = g_hash_table_lookup (state->connected_servers, &node_id);
+
+  if (info == NULL)
+    {
+      g_warning ("Ignoring session event for unknown node %d.", node_id);
+      return;
+    }
+  
+  g_mutex_lock (&state->mutex);
+  
+  switch (type)
+    {
+    case TASK_ASSIGNED:
+    case TASK_SUBMITTED:
+      info->num_active_tasks++;
+      info->num_incoming_tasks++;
+      break;
+      
+    case TASK_COMPLETED:
+      info->num_active_tasks--;
+      info->num_completed_tasks++;
+      break;
+
+    case TASK_UNASSIGNED:
+      info->num_active_tasks--;
+      info->num_unassigned_tasks++;
+
+    default:
+      break;
+    };
+
+  g_mutex_unlock (&state->mutex);
+}
+
 void
 gzochid_httpd_meta_register_handlers (GzochidHttpServer *httpd_context,
 				      gzochid_event_source *root_event_source,
@@ -350,14 +436,21 @@ gzochid_httpd_meta_register_handlers (GzochidHttpServer *httpd_context,
   gzochid_event_source *event_source = NULL;
   GzochiMetadSessionServer *sessionserver = gzochid_resolver_require_full
     (res_context, GZOCHI_METAD_TYPE_SESSION_SERVER, NULL);
+  GzochiMetadTaskServer *taskserver = gzochid_resolver_require_full
+    (res_context, GZOCHI_METAD_TYPE_TASK_SERVER, NULL);
 
   gzochid_event_attach (root_event_source, handle_client_event, state);  
 
   g_object_get (sessionserver, "event-source", &event_source, NULL);  
   gzochid_event_attach (event_source, handle_session_event, state);  
   g_source_unref ((GSource *) event_source);
-  
+
+  g_object_get (taskserver, "event-source", &event_source, NULL);  
+  gzochid_event_attach (event_source, handle_task_event, state);  
+  g_source_unref ((GSource *) event_source);
+
   gzochid_httpd_add_terminal (httpd_context, "/", hello_world, state);
 
   g_object_unref (sessionserver);
+  g_object_unref (taskserver);
 }
