@@ -1,5 +1,5 @@
 /* httpd-meta.h: Prototypes and declarations for httpd-meta.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,17 +18,25 @@
 #ifndef GZOCHID_HTTPD_META_H
 #define GZOCHID_HTTPD_META_H
 
+#include "event.h"
 #include "httpd.h"
+#include "resolver.h"
 
-/* Configure the specified HTTP server context with a RESTful hierarchy of 
-   handlers for providing feedback about the state of a running gzochi meta
-   server.
+/*
+  Configure the specified HTTP server context with a RESTful hierarchy of 
+  handlers for providing feedback about the state of a running gzochi meta
+  server.
 
-   The resource tree looks like:
+  The resource tree looks like:
 
-   / - Server root
+  / - Server root
+
+  The specified `GzochidResolutionContext' is used to resolve other parts of
+  the meta server's infrastructure. 
 */
 
-void gzochid_httpd_meta_register_handlers (GzochidHttpServer *);
+void gzochid_httpd_meta_register_handlers (GzochidHttpServer *,
+					   gzochid_event_source *,
+					   GzochidResolutionContext *);
 
 #endif /* GZOCHID_HTTPD_META_H */
