@@ -1,5 +1,5 @@
 /* httpd.h: Prototypes and declarations for httpd.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,27 +26,8 @@
 
 #define GZOCHID_TYPE_HTTP_SERVER gzochid_http_server_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_http_server_get_type (void);
-
-typedef struct _GzochidHttpServer GzochidHttpServer;
-
-struct _GzochidHttpServerClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidHttpServerClass GzochidHttpServerClass;
-
-static inline GzochidHttpServer *
-GZOCHID_HTTP_SERVER (gconstpointer ptr) {
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_http_server_get_type (), GzochidHttpServer);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochidHttpServer, gzochid_http_server, GZOCHID,
+		      HTTP_SERVER, GObject);
 
 enum GzochidHttpServerError
   { 
