@@ -1,5 +1,5 @@
 /* lock-mem.c: In-memory lock table implementation for gzochi-metad
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -703,7 +703,7 @@ gzochid_lock_range_check_and_set (gzochid_lock_table *lock_table, guint node_id,
 	       if the key is locked for write, there'll only be a single 
 	       locker. */
 	    
-	    if (lock->for_write)
+	    if (lock->for_write && lock->node_id != node_id) 
 	      return lock_failure (lock->timestamp, ret_timestamp);
 	    
 	    point_iter = g_sequence_iter_next (point_iter);
@@ -791,6 +791,10 @@ gzochid_lock_release (gzochid_lock_table *lock_table, guint node_id,
 
       if (lock_ptr != NULL)
 	remove_lock (lock_table, lock_ptr->data);
+      else GZOCHID_WITH_FORMATTED_BYTES
+	     (key, buf, 33, g_warning
+	      ("Attempted to release non-existent lock on %s for node %d.", buf,
+	       node_id));
     }
 }
 
@@ -889,6 +893,20 @@ gzochid_lock_release_range (gzochid_lock_table *lock_table, guint node_id,
 
   if (match_context.range_lock != NULL)  
     remove_range_lock (lock_table, match_context.range_lock);
+  else
+    {
+      char from_buf[33], to_buf[33];
+
+      if (from != NULL)
+	gzochid_util_format_bytes (from, from_buf, 33);
+      else memcpy (from_buf, "NULL", 5);
+      if (to != NULL)
+	gzochid_util_format_bytes (to, to_buf, 33);
+      else memcpy (to_buf, "NULL", 5);
+
+      g_warning ("Attempted to release non-existent range lock from %s to %s "
+		 "for node %d.", from_buf, to_buf, node_id);
+    }
 }
 
 /* Adapts `remove_lock' as a `GFunc' for use by `g_list_foreach' in 
