@@ -1,5 +1,5 @@
 /* oids-storage.h: Protoypes and declarations for oids-storage.h
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,17 +23,13 @@
 
 /* 
    Create and return an oid allocation strategy that uses the specified storage
-   engine, environment, and store to manage the state of allocated ids.
+   engine and environment path to manage the state of allocated ids.
 
    The returned strategy should be freed with 
-   `gzochid_oid_allocation_strategy_free' when no longer needed. The storage
-   resources - in particular the storage context and store - should remain open
-   for the lifetime of the strategy; if the environment needs to be closed for
-   some reason, the strategy should be recreated.
+   `gzochid_oid_allocation_strategy_free' when no longer needed.
 */
 
 gzochid_oid_allocation_strategy *gzochid_storage_oid_strategy_new
-(gzochid_storage_engine_interface *, gzochid_storage_context *,
- gzochid_storage_store *);
+(gzochid_storage_engine_interface *, const char *);
 
 #endif /* GZOCHID_OIDS_STORAGE_H */
