@@ -1,5 +1,5 @@
 /* reloc.c: Support for assigning fixed native pointer locations to SCM ojects
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  */
 
 #include <glib.h>
+#include <glib-object.h>
 #include <libguile.h>
 #include <stdlib.h>
 
@@ -24,9 +25,29 @@
 #include "scheme.h"
 #include "tx.h"
 
+/* A heap-allocated wrapper around the opaque `scm_t_bits' type. */
+
+struct _gzochid_scm_location_info
+{
+  scm_t_bits bits; /* The serialized representation of the target SCM object. */
+};
+
+/* Holds transactional state for the SCM location management transaction 
+   participate. */
+
+struct _gzochid_scm_location_transaction_context
+{
+  GzochidApplicationContext *app_context; /* The application context. */
+
+  /* A mapping of `scm_t_bits' to `gzochid_scm_location_info *'. */
+
+  GHashTable *bits_cache; 
+};
+
 static void 
 transaction_context_free (gzochid_scm_location_transaction_context *context)
 {
+  g_object_unref (context->app_context);
   g_hash_table_destroy (context->bits_cache);
   free (context);
 }
@@ -83,12 +104,12 @@ free_bits (gpointer ptr)
 }
 
 static gzochid_scm_location_transaction_context *
-create_transaction_context (gzochid_application_context *app_context)
+create_transaction_context (GzochidApplicationContext *app_context)
 {
   gzochid_scm_location_transaction_context *tx_context =
     malloc (sizeof (gzochid_scm_location_transaction_context));
 
-  tx_context->context = app_context;
+  tx_context->app_context = g_object_ref (app_context);
   tx_context->bits_cache = g_hash_table_new_full 
     (scm_bits_hash, scm_bits_equal, free_bits, free);  
   
@@ -96,7 +117,7 @@ create_transaction_context (gzochid_application_context *app_context)
 }
 
 static void 
-join_transaction (gzochid_application_context *context)
+join_transaction (GzochidApplicationContext *context)
 {
   if (!gzochid_transaction_active ()
       || gzochid_transaction_context (&scm_location_participant) == NULL)
@@ -106,7 +127,7 @@ join_transaction (gzochid_application_context *context)
 
 static void 
 location_aware_scheme_serializer 
-(gzochid_application_context *context, void *ptr, GByteArray *out, GError **err)
+(GzochidApplicationContext *context, void *ptr, GByteArray *out, GError **err)
 {
   GError *local_err = NULL;
 
@@ -130,7 +151,7 @@ location_aware_scheme_serializer
 
 static void *
 location_aware_scheme_deserializer
-(gzochid_application_context *context, GByteArray *in, GError **err)
+(GzochidApplicationContext *context, GByteArray *in, GError **err)
 {
   GError *local_err = NULL;
   SCM obj = gzochid_scheme_data_serialization.deserializer 
@@ -153,8 +174,7 @@ location_aware_scheme_deserializer
 }
 
 static void 
-location_aware_scheme_finalizer (gzochid_application_context *context, 
-				 void *ptr)
+location_aware_scheme_finalizer (GzochidApplicationContext *context, void *ptr)
 {
 }
 
@@ -166,7 +186,7 @@ gzochid_io_serialization gzochid_scm_location_aware_serialization =
   };
 
 gzochid_scm_location_info *
-gzochid_scm_location_get (gzochid_application_context *context, SCM obj)
+gzochid_scm_location_get (GzochidApplicationContext *context, SCM obj)
 {
   gzochid_scm_location_transaction_context *tx_context = NULL;
   scm_t_bits bits = SCM_UNPACK (obj);
@@ -199,7 +219,7 @@ gzochid_scm_location_get (gzochid_application_context *context, SCM obj)
  
 SCM 
 gzochid_scm_location_resolve 
-(gzochid_application_context *context, gzochid_scm_location_info *location)
+(GzochidApplicationContext *context, gzochid_scm_location_info *location)
 {
   return SCM_PACK (location->bits);
 }
