@@ -1,5 +1,5 @@
 /* reloc.h: Prototypes and declarations for reloc.c
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,20 +26,15 @@
 
 extern gzochid_io_serialization gzochid_scm_location_aware_serialization;
 
-typedef struct _gzochid_scm_location_info
-{
-  scm_t_bits bits;
-} gzochid_scm_location_info;
+typedef struct _gzochid_scm_location_info gzochid_scm_location_info;
 
 typedef struct _gzochid_scm_location_transaction_context
-{
-  gzochid_application_context *context;
-  GHashTable *bits_cache;
-} gzochid_scm_location_transaction_context;
+gzochid_scm_location_transaction_context;
 
-gzochid_scm_location_info *gzochid_scm_location_get 
-(gzochid_application_context *, SCM);
-SCM gzochid_scm_location_resolve 
-(gzochid_application_context *, gzochid_scm_location_info *);
+gzochid_scm_location_info *gzochid_scm_location_get
+(GzochidApplicationContext *, SCM);
+
+SCM gzochid_scm_location_resolve (GzochidApplicationContext *,
+				  gzochid_scm_location_info *);
 
 #endif /* GZOCHID_RELOC_H */
