@@ -1,5 +1,5 @@
 /* resolver.h: Prototypes and declarations for resolver.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -41,27 +41,8 @@
 
 #define GZOCHID_TYPE_RESOLUTION_CONTEXT gzochid_resolution_context_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_resolution_context_get_type (void);
-
-typedef struct _GzochidResolutionContext GzochidResolutionContext;
-
-struct _GzochidResolutionContextClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidResolutionContextClass GzochidResolutionContextClass;
-
-static inline GzochidResolutionContext *
-GZOCHID_RESOLUTION_CONTEXT (gconstpointer ptr) {
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_resolution_context_get_type (), GzochidResolutionContext);
-}
-
-/* End boilerplate. */
+G_DECLARE_FINAL_TYPE (GzochidResolutionContext, gzochid_resolution_context,
+                      GZOCHID, RESOLUTION_CONTEXT, GObject);
 
 enum GzochidResolutionError
   {
