@@ -1,5 +1,5 @@
 /* scheme-task.c: Scheme callback handlers and interface to gzochi Scheme API
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "app-task.h"
 #include "callback.h"
 #include "data.h"
+#include "descriptor.h"
 #include "durable-task.h"
 #include "guile.h"
 #include "gzochid-auth.h"
@@ -61,7 +62,7 @@ static gzochid_transaction_participant scheme_participant =
   { "scheme", scheme_prepare, scheme_commit, scheme_rollback };
 
 void 
-gzochid_scheme_application_worker (gzochid_application_context *context, 
+gzochid_scheme_application_worker (GzochidApplicationContext *context, 
 				   gzochid_auth_identity *identity, 
 				   gpointer ptr)
 {
@@ -88,7 +89,7 @@ gzochid_scheme_application_worker (gzochid_application_context *context,
 }
 
 void 
-gzochid_scheme_application_task_worker (gzochid_application_context *context, 
+gzochid_scheme_application_task_worker (GzochidApplicationContext *context, 
 					gzochid_auth_identity *identity, 
 					gpointer task)
 {
@@ -117,14 +118,18 @@ gzochid_scheme_application_task_worker (gzochid_application_context *context,
 
 void 
 gzochid_scheme_application_initialized_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gpointer data)
 {
   GHashTable *properties = data;
   gzochid_data_managed_reference *callback_reference = NULL;
-  SCM callback = gzochid_scheme_create_callback 
-    (context->descriptor->initialized, NULL);
+  GzochidApplicationDescriptor *descriptor = NULL;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  SCM callback;
+
+  g_object_get (context, "descriptor", &descriptor, NULL);
+  callback = gzochid_scheme_create_callback (descriptor->initialized, NULL);
+  g_object_unref (descriptor);
 
   callback_reference = gzochid_data_create_reference 
     (context, &gzochid_scheme_data_serialization, callback, NULL);
@@ -163,7 +168,7 @@ gzochid_scheme_application_initialized_worker
 }
 
 static gzochid_application_callback *
-scm_to_callback (gzochid_application_context *context, SCM scm_callback)
+scm_to_callback (GzochidApplicationContext *context, SCM scm_callback)
 {
   gzochid_scm_location_info *scm_callback_reloc = gzochid_scm_location_get
     (context, scm_callback);
@@ -195,7 +200,7 @@ unpack_handler (gpointer data)
   SCM received_message = gzochid_scheme_handler_received_message (handler);
   SCM disconnected = gzochid_scheme_handler_disconnected (handler);
 
-  gzochid_application_context *context = 
+  GzochidApplicationContext *context = 
     gzochid_get_current_application_context ();
   gzochid_client_session_handler *session_handler = 
     malloc (sizeof (gzochid_client_session_handler));
@@ -217,12 +222,13 @@ is_client_session_listener (SCM obj)
 }
 
 void 
-gzochid_scheme_application_logged_in_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
+gzochid_scheme_application_logged_in_worker (GzochidApplicationContext *context,
+					     gzochid_auth_identity *identity, 
+					     gpointer data)
 {
   GError *err = NULL;
   guint64 *session_oid = data;
+  GzochidApplicationDescriptor *descriptor = NULL;
   gzochid_client_session *session = NULL;
   gzochid_data_managed_reference *session_reference = NULL;
 
@@ -249,7 +255,9 @@ gzochid_scheme_application_logged_in_worker
   scm_session = gzochid_scheme_create_client_session 
     (session, session_reference->oid);
 
-  cb = gzochid_scheme_create_callback (context->descriptor->logged_in, NULL);
+  g_object_get (context, "descriptor", &descriptor, NULL); 
+  cb = gzochid_scheme_create_callback (descriptor->logged_in, NULL);
+  g_object_unref (descriptor);
   
   gzochid_transaction_join (&scheme_participant, NULL);
 
@@ -346,7 +354,7 @@ gzochid_scheme_application_logged_in_worker
 
 void 
 gzochid_scheme_application_received_message_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gpointer ptr)
 {
   GError *err = NULL;
@@ -411,8 +419,8 @@ gzochid_scheme_application_received_message_worker
 }
 
 void 
-gzochid_scheme_application_disconnected_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+gzochid_scheme_application_disconnected_worker
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gpointer ptr)
 {
   GError *err = NULL;
@@ -530,20 +538,23 @@ gzochid_scheme_application_disconnected_worker
 
 void 
 gzochid_scheme_application_disconnected_cleanup_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
+(GzochidApplicationContext *context, gzochid_auth_identity *identity, 
  gpointer ptr)
 {
   g_free (ptr);
 }
 
 void
-gzochid_scheme_application_ready (gzochid_application_context *context, 
+gzochid_scheme_application_ready (GzochidApplicationContext *context, 
 				  gzochid_auth_identity *identity, GError **err)
 {
-  SCM callback = gzochid_scheme_create_callback 
-    (context->descriptor->ready, NULL);
+  GzochidApplicationDescriptor *descriptor = NULL;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  SCM callback;
 
+  g_object_get (context, "descriptor", &descriptor, NULL);
+  callback = gzochid_scheme_create_callback (descriptor->ready, NULL);
+  
   /* This handler is called with a "current application" and identity, but isn't
      called within a transaction and doesn't attempt to join one. */
      
@@ -551,13 +562,15 @@ gzochid_scheme_application_ready (gzochid_application_context *context,
     (context, identity, "gzochi:execute-ready", gzochi_private_app,
      scm_list_2 (callback,
 		 gzochid_scheme_ghashtable_to_hashtable 
-		 (context->descriptor->properties, 
+		 (descriptor->properties, 
 		  gzochid_scheme_string_hash,
 		  gzochid_scheme_string_equiv,
 		  (SCM (*) (gpointer)) scm_from_locale_string,
 		  (SCM (*) (gpointer)) scm_from_locale_string)),
      exception_var);
 
+  g_object_unref (descriptor);
+
   /* Bridge any exception raised to the `GError' argument. */
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
@@ -566,14 +579,13 @@ gzochid_scheme_application_ready (gzochid_application_context *context,
 }
 
 static void 
-scheme_worker_serializer (gzochid_application_context *context, 
+scheme_worker_serializer (GzochidApplicationContext *context, 
 			  gzochid_application_worker worker, GByteArray *out)
 {
 }
 
 static gzochid_application_worker 
-scheme_worker_deserializer (gzochid_application_context *context,
-			    GByteArray *in)
+scheme_worker_deserializer (GzochidApplicationContext *context, GByteArray *in)
 {
   return gzochid_scheme_application_task_worker;
 }
@@ -589,7 +601,7 @@ gzochid_application_task_serialization gzochid_scheme_task_serialization =
   };
 
 gzochid_application_task *
-gzochid_scheme_task_new (gzochid_application_context *context, 
+gzochid_scheme_task_new (GzochidApplicationContext *context, 
 			 gzochid_auth_identity *identity, char *procedure, 
 			 GList *module_name, SCM data)
 {
@@ -609,7 +621,7 @@ gzochid_scheme_task_new (gzochid_application_context *context,
 
   scm_gc_protect_object (scm_data);
   task = gzochid_application_task_new
-    (context, identity, gzochid_scheme_application_task_worker, scm_data);
+    (context, identity, gzochid_scheme_application_task_worker, NULL, scm_data);
   
   return task;
 }
