@@ -1,5 +1,5 @@
 /* scheme.c: Supplementary Scheme routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@ scheme_invoke_inner (gpointer data)
 }
 
 SCM
-gzochid_scheme_invoke (gzochid_application_context *context, 
+gzochid_scheme_invoke (GzochidApplicationContext *context, 
 		       gzochid_auth_identity *identity, SCM proc, SCM args, 
 		       SCM exception_var)
 {
@@ -127,7 +127,7 @@ resolve_procedure (char *procedure, GList *module)
 }
 
 SCM 
-gzochid_scheme_invoke_callback (gzochid_application_context *context,
+gzochid_scheme_invoke_callback (GzochidApplicationContext *context,
 				gzochid_auth_identity *identity, 
 				char *procedure, GList *module, SCM args, 
 				SCM exception_var)
@@ -138,7 +138,7 @@ gzochid_scheme_invoke_callback (gzochid_application_context *context,
 }
 
 static void 
-scheme_managed_record_serializer (gzochid_application_context *context, SCM arg,
+scheme_managed_record_serializer (GzochidApplicationContext *context, SCM arg,
 				  GByteArray *out, GError **err)
 {
   unsigned char vec_len_str[4];
@@ -187,7 +187,7 @@ scheme_managed_record_serializer (gzochid_application_context *context, SCM arg,
 }
 
 static void *
-scheme_managed_record_deserializer (gzochid_application_context *context, 
+scheme_managed_record_deserializer (GzochidApplicationContext *context, 
 				    GByteArray *in, GError **err)
 {
   int vec_len = gzochi_common_io_read_int (in->data, 0);
@@ -249,7 +249,7 @@ static void *
 scheme_serializer_inner (void *data)
 {
   void **ptr = data;
-  gzochid_application_context *context = ptr[0];
+  GzochidApplicationContext *context = ptr[0];
   SCM task = ptr[1];
   GByteArray *out = ptr[2];
   GError **err = ptr[3];
@@ -260,7 +260,7 @@ scheme_serializer_inner (void *data)
 }
 
 static void 
-scheme_serializer (gzochid_application_context *context, void *ptr, 
+scheme_serializer (GzochidApplicationContext *context, void *ptr,
 		   GByteArray *out, GError **err)
 {
   void *args[4];
@@ -277,7 +277,7 @@ static void *
 scheme_deserializer_inner (void *data)
 {
   void **ptr = data;
-  gzochid_application_context *context = ptr[0];
+  GzochidApplicationContext *context = ptr[0];
   GByteArray *in = ptr[1];
   GError **err = ptr[2];
 
@@ -285,7 +285,7 @@ scheme_deserializer_inner (void *data)
 }
 
 static void *
-scheme_deserializer (gzochid_application_context *context, GByteArray *in, 
+scheme_deserializer (GzochidApplicationContext *context, GByteArray *in, 
 		     GError **err)
 {
   void *args[3];
@@ -305,7 +305,7 @@ scheme_finalizer_inner (void *data)
 }
 
 static void 
-scheme_finalizer (gzochid_application_context *context, gpointer data)
+scheme_finalizer (GzochidApplicationContext *context, gpointer data)
 {
   scm_with_guile (scheme_finalizer_inner, data);
 }
