@@ -1,5 +1,5 @@
 ;; gzochi/app.scm: Public exports for general gzochi application support 
-;; Copyright (C) 2014 Julian Graham
+;; Copyright (C) 2017 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -28,8 +28,10 @@
 
   (import (only (guile) make-fluid)
 	  (gzochi private app)
+	  (gzochi private reflect)
 	  (rnrs base)
 	  (rnrs syntax-case)
+	  (only (srfi :1) find)
 	  (system syntax))
 
   (define-syntax g:@
@@ -40,13 +42,18 @@
 	((_ (module ...) procedure data)
 	 #'(gzochi:make-callback 
 	    (quote procedure) (quote (module ...)) data))
-	((_ procedure) 
-	 #`(gzochi:make-callback 
-	    (quote procedure) (syntax-module #'procedure)))
+	((_ procedure)
+	 (let ((module (datum->syntax
+			stx (gzochi:check-syntax-module
+			     (syntax->datum #'procedure)
+			     (syntax-module #'procedure)))))
+	   #`(gzochi:make-callback (quote procedure) (quote #,module))))
 	((_ procedure data)
-	 #`(gzochi:make-callback
-	    (quote procedure) (syntax-module #'procedure)
-	    data)))))
+	 (let ((module (datum->syntax
+			stx (gzochi:check-syntax-module
+			     (syntax->datum #'procedure)
+			     (syntax-module #'procedure)))))
+	   #`(gzochi:make-callback (quote procedure) (quote #,module) data))))))
 
   (define %gzochi:application-root (make-fluid))
 )
