@@ -54,7 +54,9 @@
     (let ((procedure (gzochi:resolve-procedure 
 		      (gzochi:callback-procedure callback)
 		      (gzochi:callback-module callback))))
-      (procedure (gzochi:callback-data callback))))
+      (if (thunk? procedure)
+	  (procedure)
+	  (procedure (gzochi:callback-data callback)))))
 
   (define* (gzochi:schedule-task callback #:optional delay period)
     (or (gzochi:callback? callback)
