@@ -1,5 +1,5 @@
 /* session.h: Prototypes and declarations for session.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <glib.h>
 
 #include "app.h"
+#include "callback.h"
 #include "data.h"
 #include "gzochid-auth.h"
 #include "io.h"
@@ -55,18 +56,18 @@ extern gzochid_io_serialization gzochid_client_session_serialization;
    it exists. */
 
 void gzochid_client_session_disconnected_worker
-(gzochid_application_context *, gzochid_auth_identity *, gpointer);
+(GzochidApplicationContext *, gzochid_auth_identity *, gpointer);
 
 gzochid_client_session *gzochid_client_session_new (gzochid_auth_identity *);
 void gzochid_client_session_free (gzochid_client_session *);
 void gzochid_client_session_disconnect 
-(gzochid_application_context *, gzochid_client_session *);
+(GzochidApplicationContext *, gzochid_client_session *);
 void gzochid_client_session_send_login_success 
-(gzochid_application_context *, gzochid_client_session *);
+(GzochidApplicationContext *, gzochid_client_session *);
 void gzochid_client_session_send_login_failure 
-(gzochid_application_context *, gzochid_client_session *);
+(GzochidApplicationContext *, gzochid_client_session *);
 void gzochid_client_session_send_message 
-(gzochid_application_context *, gzochid_client_session *, unsigned char *, 
+(GzochidApplicationContext *, gzochid_client_session *, unsigned char *, 
  short);
 
 gzochid_auth_identity *gzochid_client_session_identity
@@ -103,7 +104,7 @@ void gzochid_client_session_set_handler
 (gzochid_client_session *, gzochid_client_session_handler *);
 
 void gzochid_client_session_persist 
-(gzochid_application_context *, gzochid_client_session *, guint64 *, GError **);
-void gzochid_sweep_client_sessions (gzochid_application_context *, GError **);
+(GzochidApplicationContext *, gzochid_client_session *, guint64 *, GError **);
+void gzochid_sweep_client_sessions (GzochidApplicationContext *, GError **);
 
 #endif /* GZOCHID_SESSION_H */
