@@ -1,5 +1,5 @@
 /* socket.h: Prototypes and declarations for socket.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -28,10 +28,8 @@
 
 #define GZOCHID_TYPE_SOCKET_SERVER gzochid_socket_server_get_type ()
 
-/* The following boilerplate can be consolidated once GLib 2.44 makes it into
-   Debian stable and `G_DECLARE_FINAL_TYPE' can be used. */
-
-GType gzochid_socket_server_get_type (void);
+G_DECLARE_FINAL_TYPE (GzochidSocketServer, gzochid_socket_server, GZOCHID,
+                      SOCKET_SERVER, GObject);
 
 /* The socket server struct itself; this part of the server is public. */
 
@@ -43,23 +41,6 @@ struct _GzochidSocketServer
   GMainLoop *main_loop; /* The main loop for socket events. */
 };
 
-typedef struct _GzochidSocketServer GzochidSocketServer;
-
-struct _GzochidSocketServerClass
-{
-  GObjectClass parent_class;
-};
-
-typedef struct _GzochidSocketServerClass GzochidSocketServerClass;
-
-static inline GzochidSocketServer *
-GZOCHID_SOCKET_SERVER (gconstpointer ptr) {
-  return G_TYPE_CHECK_INSTANCE_CAST
-    (ptr, gzochid_socket_server_get_type (), GzochidSocketServer);
-}
-
-/* End boilerplate. */
-
 /* The following functions can be used to control the lifecycle of a 
    `GzochidSocketServer' object. */
 
