@@ -1,5 +1,5 @@
 /* storage-mem.c: Database storage routines for gzochid (in-memory)
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -3342,7 +3342,8 @@ find_key_gte (btree_transaction *btx, btree *bt, char *key, size_t key_len,
 	{
 	  int i = 0;
 	  size_t offset = 0;
-
+	  btree_node *next_node = NULL;
+	  
 	  while (offset < page->page_size)
 	    {
 	      unsigned short len = gzochi_common_io_read_short
@@ -3368,7 +3369,60 @@ find_key_gte (btree_transaction *btx, btree *bt, char *key, size_t key_len,
 		  i++;
 		}
 	    }
-	  return NULL;
+
+	  /* There's an edge case where the key we're looking for falls directly
+	     _after_ the last key on the page returned by `search'. */
+
+	  next_node = tx_next_sibling (node, btx, NULL);
+
+	  /* Walk up the tree until we find an ancestor with a "next" sibling
+	     node... */
+	  
+	  while (next_node == NULL && node != NULL)
+	    {
+	      node = tx_parent (node, btx, NULL);
+
+	      if (node != NULL)
+		next_node = tx_next_sibling (node, btx, NULL);
+	    }
+
+	  if (next_node != NULL)
+	    {
+	      btree_page *page = NULL;
+	      unsigned short len;
+	      char *ret = NULL;
+
+	      /* ...then walk back down the tree taking the left-most path until
+		 we get back to the leaf level. If we identified a structural
+		 node to the right of the original one, that means there's 
+		 guaranteed to be a page of data back down at the leaf level. */
+	      
+	      while (TRUE)
+		{
+		  page = effective_page (next_node, FALSE);
+
+		  if (page == NULL)
+		    next_node = tx_first_child (next_node, btx, NULL);
+		  else break;
+		}
+
+	      /* Take the first key in that leaf's page. */
+	      
+	      len = gzochi_common_io_read_short (page->data, 0);
+	      ret = malloc (sizeof (char) * len);
+	      
+	      memcpy (ret, page->data + 2, len);
+	      
+	      if (found_key_len != NULL)
+		*found_key_len = len;
+	      
+	      return ret;
+	    }
+
+	  /* If we couldn't find a single next sibling, that means we're truly
+	     at the end of the keyspace. */
+	  
+	  else return NULL;
 	}
       else return NULL;
     }
