@@ -62,7 +62,7 @@ transaction_new (void)
   transaction->id = transaction_counter++;
   g_mutex_unlock (&transaction_mutex);
 
-  transaction->name = g_strdup_printf ("%lx", transaction->id);
+  transaction->name = g_strdup_printf ("%" G_GUINT64_FORMAT, transaction->id);
   transaction->state = GZOCHID_TRANSACTION_STATE_ACTIVE;
 
   return transaction;
@@ -171,7 +171,7 @@ prepare (gzochid_transaction *transaction)
 	(gzochid_transaction_participant_registration *) participant_ptr->data;
       if (!participant->participant->prepare (participant->data))
 	{
-	  g_info 
+	  g_message 
 	    ("Participant '%s' in transaction '%s' failed to prepare.",
 	     participant->participant->name, transaction->name);
 
