@@ -1,5 +1,5 @@
 /* util.h: Prototypes and declarations for util.c
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -75,6 +75,10 @@ struct timeval gzochid_util_deserialize_timeval (GByteArray *);
 
 gint gzochid_util_string_data_compare (gconstpointer, gconstpointer, gpointer);
 
+/* A `GCompareFunc' implementation that operates on pointers to `guint64's. */
+
+gint gzochid_util_guint64_compare (gconstpointer, gconstpointer);
+
 /* A `GCompareDataFunc' implementation that operates on pointers to 
    `guint64's. */
 
