@@ -1,5 +1,5 @@
 /* test-password_file.c: Test routines for the password file auth plugin.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #include <stddef.h>
 #include <stdlib.h>
 
+#include "app.h"
+#include "auth_int.h"
 #include "gzochid-auth.h"
 
 #define Q(x) #x
@@ -30,6 +32,13 @@
 
 gint gzochid_auth_init_plugin (gzochid_auth_plugin *);
 
+gzochid_auth_identity *
+gzochid_application_context_resolve_identity
+(GzochidApplicationContext *app_context, char *name)
+{
+  return gzochid_auth_identity_new (name);
+}
+
 static void 
 test_initialize ()
 {
