@@ -1,5 +1,5 @@
 /* test-channelclient.c: Tests for channelclient.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "app.h"
+#include "app-session.h"
 #include "channel.h"
 #include "channelclient.h"
 #include "config.h"
@@ -34,6 +36,14 @@ static gboolean channel_left = FALSE;
 static gboolean channel_closed = FALSE;
 static GBytes *channel_msg = NULL;
 
+void
+gzochid_mock_application_install_properties (GObjectClass *object_class)
+{
+  g_object_class_install_property
+    (object_class, 1, g_param_spec_pointer
+     ("session-map", "session-map", "session-map", G_PARAM_READWRITE));
+}
+
 static void
 clear_channel_state (void)
 {
@@ -79,17 +89,21 @@ gzochid_reconnectable_socket_write (gzochid_reconnectable_socket *sock,
 }
 
 void
-gzochid_channel_join_direct (gzochid_application_context *app_context,
+gzochid_channel_join_direct (GzochidApplicationContext *app_context,
 			     guint64 channel_oid, guint64 session_oid,
 			     GError **err)
 {
-  if (!g_hash_table_contains (app_context->oids_to_clients, &session_oid))
+  gzochid_application_session_map *session_map = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
+  
+  if (!g_hash_table_contains (session_map->oids_to_clients, &session_oid))
     g_set_error (err, GZOCHID_CHANNEL_ERROR, GZOCHID_CHANNEL_ERROR_NOT_MAPPED,
 		 "Unmapped session.");
   else
     {
       GSequence *sessions = g_hash_table_lookup
-	(app_context->channel_oids_to_local_session_oids, &channel_oid);
+	(session_map->channel_oids_to_local_session_oids, &channel_oid);
 
       if (sessions != NULL &&
 	  g_sequence_lookup (sessions, &session_oid,
@@ -103,17 +117,21 @@ gzochid_channel_join_direct (gzochid_application_context *app_context,
 }
 
 void
-gzochid_channel_leave_direct (gzochid_application_context *app_context,
+gzochid_channel_leave_direct (GzochidApplicationContext *app_context,
 			      guint64 channel_oid, guint64 session_oid,
 			      GError **err)
 {
-  if (!g_hash_table_contains (app_context->oids_to_clients, &session_oid))
+  gzochid_application_session_map *session_map = NULL;
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
+
+  if (!g_hash_table_contains (session_map->oids_to_clients, &session_oid))
     g_set_error (err, GZOCHID_CHANNEL_ERROR, GZOCHID_CHANNEL_ERROR_NOT_MAPPED,
 		 "Unmapped session.");
   else
     {
       GSequence *sessions = g_hash_table_lookup
-	(app_context->channel_oids_to_local_session_oids, &channel_oid);
+	(session_map->channel_oids_to_local_session_oids, &channel_oid);
 
       if (sessions == NULL ||
 	  g_sequence_lookup (sessions, &session_oid,
@@ -127,14 +145,14 @@ gzochid_channel_leave_direct (gzochid_application_context *app_context,
 }
 
 void
-gzochid_channel_close_direct (gzochid_application_context *app_context,
+gzochid_channel_close_direct (GzochidApplicationContext *app_context,
 			      guint64 channel_oid)
 {
   channel_closed = TRUE;
 }
 
 void
-gzochid_channel_message_direct (gzochid_application_context *app_context,
+gzochid_channel_message_direct (GzochidApplicationContext *app_context,
 				guint64 channel_oid, GBytes *msg)
 {
   channel_msg = g_bytes_ref (msg);
@@ -171,39 +189,26 @@ gzochid_game_server_class_init (GzochidGameServerClass *klass)
   object_class->finalize = game_server_finalize;
 }
 
-static gzochid_application_context *
-application_context_new ()
-{
-  gzochid_application_context *app_context =
-    calloc (1, sizeof (gzochid_application_context));
-
-  app_context->oids_to_clients = g_hash_table_new_full
-    (g_int64_hash, g_int64_equal, g_free, NULL);
-  app_context->channel_oids_to_local_session_oids = g_hash_table_new_full
-    (g_int64_hash, g_int64_equal, g_free, (GDestroyNotify) g_sequence_free);
-  
-  return app_context;
-}
-
 static void
-application_context_free (gpointer data)
+free_app (gpointer data)
 {
-  gzochid_application_context *app_context = data;
+  GzochidApplicationContext *context = GZOCHID_APPLICATION_CONTEXT (data);
+  gzochid_application_session_map *session_map = NULL;
 
-  g_hash_table_destroy (app_context->oids_to_clients);
-  g_hash_table_destroy (app_context->channel_oids_to_local_session_oids);
-
-  free (app_context);
+  g_object_get (context, "session-map", &session_map, NULL);
+  gzochid_application_session_map_free (session_map);
+  
+  g_object_unref (context);
 }
 
 static void
 gzochid_game_server_init (GzochidGameServer *self)
 {
   self->applications = g_hash_table_new_full
-    (g_str_hash, g_str_equal, free, (GDestroyNotify) application_context_free);
+    (g_str_hash, g_str_equal, free, free_app);
 }
 
-gzochid_application_context *
+GzochidApplicationContext *
 gzochid_game_server_lookup_application (GzochidGameServer *game_server,
 					const char *app)
 {
@@ -224,7 +229,7 @@ channelclient_fixture_setup (channelclient_fixture *fixture,
 			     gconstpointer user_data)
 {
   fixture->game_server = gzochid_resolver_require
-    (GZOCHID_TYPE_GAME_SERVER, NULL);  
+    (GZOCHID_TYPE_GAME_SERVER, NULL);
   fixture->socket = gzochid_reconnectable_socket_new ();
 
   fixture->channelclient = g_object_new
@@ -248,33 +253,41 @@ create_app (channelclient_fixture *fixture, const char *app)
 {
   g_hash_table_insert
     (fixture->game_server->applications, strdup (app),
-     application_context_new ());
+     g_object_new (GZOCHID_TYPE_APPLICATION_CONTEXT,
+		   "session-map", gzochid_application_session_map_new (),
+		   NULL));
 }
 
 static void
 register_client (channelclient_fixture *fixture, const char *app,
 		 guint64 session_oid)
 {
-  gzochid_application_context *app_context = g_hash_table_lookup
+  GzochidApplicationContext *app_context = g_hash_table_lookup
     (fixture->game_server->applications, app);
+  gzochid_application_session_map *session_map = NULL;
   guint64 *session_oid_ptr = g_memdup (&session_oid, sizeof (guint64));
+
+  g_object_get (app_context, "session-map", &session_map, NULL);
   
   g_hash_table_insert
-    (app_context->oids_to_clients, session_oid_ptr, session_oid_ptr);
+    (session_map->oids_to_clients, session_oid_ptr, session_oid_ptr);
 }
 
 static void
 add_to_channel (channelclient_fixture *fixture, const char *app,
 		guint64 channel_oid, guint64 session_oid)
 {
-  gzochid_application_context *app_context = g_hash_table_lookup
+  GzochidApplicationContext *app_context = g_hash_table_lookup
     (fixture->game_server->applications, app);
+  gzochid_application_session_map *session_map = NULL;
   GSequence *sessions = g_sequence_new (g_free);
 
+  g_object_get (app_context, "session-map", &session_map, NULL);
+
   g_sequence_append (sessions, g_memdup (&session_oid, sizeof (guint64)));
   
   g_hash_table_insert
-    (app_context->channel_oids_to_local_session_oids,
+    (session_map->channel_oids_to_local_session_oids,
      g_memdup (&channel_oid, sizeof (guint64)), sessions);
 }
 
