@@ -1,5 +1,5 @@
 /* test-data-protocol.c: Test routines for data-protocol.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -53,9 +53,20 @@ test_reserve_oids_response ()
   g_bytes_unref (bytes);
 }
 
+static void
+change_clear (gzochid_data_change *change)
+{
+  free (change->store);
+  g_bytes_unref (change->key);
+  
+  if (!change->delete)
+    g_bytes_unref (change->data);
+}
+
 static void
 test_changeset ()
 {
+  gint i = 0;
   GBytes *bytes = NULL, *data = NULL;
   GByteArray *arr = g_byte_array_new ();
 
@@ -98,14 +109,15 @@ test_changeset ()
   binding_change2a->key = g_bytes_new_static ("bar", 4);
   binding_change2a->data = g_bytes_new_static ("3", 2);
   
-  changeset1 = gzochid_data_changeset_new ("test", changes);
+  changeset1 = gzochid_data_changeset_new ("test", 123, changes);
   
   gzochid_data_protocol_changeset_write (changeset1, arr);
 
   bytes = g_byte_array_free_to_bytes (arr);
   changeset2 = gzochid_data_protocol_changeset_read (bytes);
-
+  
   g_assert (changeset2 != NULL);
+  g_assert_cmpint (changeset2->id, ==, 123);
   
   object_change1b = &g_array_index (changes, gzochid_data_change, 0);
   object_change2b = &g_array_index (changes, gzochid_data_change, 1);
@@ -126,6 +138,9 @@ test_changeset ()
   g_assert (g_bytes_equal (binding_change2a->data, binding_change2b->data));
   
   gzochid_data_changeset_free (changeset1);
+
+  for (; i < changeset2->changes->len; i++)
+    change_clear (&g_array_index (changeset2->changes, gzochid_data_change, i));
   gzochid_data_changeset_free (changeset2);
 
   free (object_change1a->store);
