@@ -1,5 +1,5 @@
 /* test-dataclient.c: Tests for dataclient.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -460,10 +460,16 @@ test_submit_changeset_simple (dataclient_fixture *fixture,
   g_array_append_val (changes, binding_change1);
   g_array_append_val (changes, binding_change2);
 
-  changeset = gzochid_data_changeset_new ("test", changes);
-  g_byte_array_append (expected_outbound_message_array, "\x00\x3f\x30", 3);
+  changeset = gzochid_data_changeset_new ("test", 0, changes);
+  g_byte_array_append
+    (expected_outbound_message_array,
+     "\x00\x0d\x32test\x00\x00\x00\x00\x00\x00\x00\x00\x00", 16);
+  g_byte_array_append (expected_outbound_message_array, "\x00\x47\x30", 3);
   gzochid_data_protocol_changeset_write
     (changeset, expected_outbound_message_array);
+  g_byte_array_append
+    (expected_outbound_message_array,
+     "\x00\x0d\x34test\x00\x00\x00\x00\x00\x00\x00\x00\x00", 16);
   gzochid_data_changeset_free (changeset);
   
   gzochid_dataclient_submit_changeset (fixture->dataclient, "test", changes);
