@@ -1,5 +1,5 @@
 /* test-dataserver.c: Tests for dataserver.c in gzochi-metad.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "resolver.h"
 #include "storage-mem.h"
 
+static int test_context_refs = 0;
 static gzochid_storage_context *test_context = NULL;
 
 static gzochid_storage_store *oids = NULL;
@@ -46,6 +47,8 @@ test_storage_initialize (char *dir)
 {
   if (test_context == NULL)
     test_context = mem_initialize (dir);
+
+  test_context_refs++;
   return test_context;
 }
 
@@ -53,27 +56,31 @@ static void
 test_storage_close_context (gzochid_storage_context *context)
 {
   assert (test_context == context);
-  mem_close_context (context);
-  test_context = NULL;  
+
+  if (--test_context_refs == 0)
+    {
+      mem_close_context (context);
+      test_context = NULL;
+    }
 }
 
 static gzochid_storage_store *
 test_storage_open (gzochid_storage_context *context, char *name,
 		   unsigned int flags)
 {
-  if (strcmp (name, "oids") == 0)
+  if (strcmp (name, "test/oids") == 0)
     {
       if (oids == NULL)
 	oids = mem_open (context, name, flags);
       return oids;
     }
-  else if (strcmp (name, "names") == 0)
+  else if (strcmp (name, "test/names") == 0)
     {
       if (names == NULL)
 	names = mem_open (context, name, flags);
       return names;
     }
-  else if (strcmp (name, "meta") == 0)
+  else if (strcmp (name, "test/meta") == 0)
     {
       if (meta == NULL)
 	meta = mem_open (context, name, flags);
@@ -170,7 +177,7 @@ test_request_value (dataserver_fixture *fixture, gconstpointer user_data)
   GBytes *expected = g_bytes_new_static ("foo", 4);
 
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "oids", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/oids", GZOCHID_STORAGE_CREATE);
   
   put (oids, "1", 2, "foo", 4);
   
@@ -233,7 +240,7 @@ test_request_next_key (dataserver_fixture *fixture, gconstpointer user_data)
   gzochid_data_response *response2 = NULL;
 
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
   put (names, "a", 2, "1", 2);
   put (names, "b", 2, "2", 2);
@@ -266,7 +273,7 @@ test_request_next_key_null (dataserver_fixture *fixture,
   gzochid_data_response *response = NULL;
 
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
   put (names, "a", 2, "1", 2);
 
@@ -289,7 +296,7 @@ test_request_next_key_not_found (dataserver_fixture *fixture,
   gzochid_data_response *response = NULL;
 
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
   put (names, "a", 2, "1", 2);
 
@@ -313,7 +320,7 @@ test_request_next_key_failure (dataserver_fixture *fixture,
   gzochid_data_response *response2 = NULL;
 
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
   
   put (names, "a", 2, "1", 2);
   put (names, "b", 2, "2", 2);
@@ -498,11 +505,11 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   g_array_insert_val (changes, 4, binding_change2);
   g_array_insert_val (changes, 5, binding_change3);
 
-  changeset = gzochid_data_changeset_new ("test", changes);
+  changeset = gzochid_data_changeset_new ("test", 123, changes);
   
   test_storage_initialize (NULL);
-  test_storage_open (test_context, "oids", GZOCHID_STORAGE_CREATE);
-  test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/oids", GZOCHID_STORAGE_CREATE);
+  test_storage_open (test_context, "test/names", GZOCHID_STORAGE_CREATE);
 
   obtain_lock (fixture->server, "test", "oids", object_change1.key);
   obtain_lock (fixture->server, "test", "oids", object_change2.key);
@@ -517,12 +524,20 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
 
   put (names, "binding-1", 10, "1", 2);
   put (names, "binding-2", 10, "2", 2);
+
+  gzochi_metad_dataserver_begin_changeset
+    (fixture->server, 1, "test", 123, &err);
+
+  g_assert_no_error (err);
   
   gzochi_metad_dataserver_process_changeset
     (fixture->server, 1, changeset, &err);
 
   g_assert_no_error (err);
 
+  gzochi_metad_dataserver_end_changeset (fixture->server, 1, "test", 123, &err);
+  g_assert_no_error (err);
+
   transaction = iface->transaction_begin (test_context);
   data = iface->transaction_get (transaction, oids, "1", 2, NULL);
 
@@ -555,28 +570,6 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   free (data);
   
   iface->transaction_rollback (transaction);
-
-  free (object_change1.store);
-  g_bytes_unref (object_change1.key);
-  g_bytes_unref (object_change1.data);
-  
-  free (object_change2.store);
-  g_bytes_unref (object_change2.key);
-
-  free (object_change3.store);
-  g_bytes_unref (object_change3.key);
-  g_bytes_unref (object_change3.data);
-
-  free (binding_change1.store);
-  g_bytes_unref (binding_change1.key);
-  g_bytes_unref (binding_change1.data);
-
-  free (binding_change2.store);
-  g_bytes_unref (binding_change2.key);
-
-  free (binding_change3.store);
-  g_bytes_unref (binding_change3.key);
-  g_bytes_unref (binding_change3.data);
   
   gzochid_data_changeset_free (changeset);
   g_array_unref (changes);
