@@ -1,5 +1,5 @@
 /* test-metaserver-protocol.c: Test routines for test-metaserver-protocol.c.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 #include "sessionserver.h"
 #include "sessionserver-protocol.h"
 #include "socket.h"
+#include "taskserver.h"
+#include "taskserver-protocol.h"
 
 #define GZOCHI_METAD_TYPE_ROOT_CONTEXT gzochi_metad_root_context_get_type ()
 
@@ -57,7 +59,8 @@ struct _GzochiMetadRootContext
   GObject *channelserver;
   GObject *dataserver;
   GObject *sessionserver;
-
+  GObject *taskserver;
+  
   gzochid_event_source *event_source;
   char *admin_server_base_url;
 };
@@ -79,6 +82,7 @@ enum gzochi_metad_data_server_properties
     PROP_CHANNEL_SERVER = 1,
     PROP_DATA_SERVER,
     PROP_SESSION_SERVER,
+    PROP_TASK_SERVER,
     PROP_EVENT_SOURCE,
     PROP_ADMIN_SERVER_BASE_URL,
     N_PROPERTIES
@@ -94,6 +98,7 @@ root_context_dispose (GObject *object)
   g_object_unref (root_context->channelserver);
   g_object_unref (root_context->dataserver);
   g_object_unref (root_context->sessionserver);
+  g_object_unref (root_context->taskserver);
   
   G_OBJECT_CLASS (gzochi_metad_root_context_parent_class)->dispose (object);  
 }
@@ -130,7 +135,11 @@ root_context_get_property (GObject *object, guint property_id, GValue *value,
     case PROP_SESSION_SERVER:
       g_value_set_object (value, root_context->sessionserver);
       break;
-      
+
+    case PROP_TASK_SERVER:
+      g_value_set_object (value, root_context->taskserver);
+      break;
+
     case PROP_EVENT_SOURCE:
       g_value_set_boxed (value, root_context->event_source);
       break;
@@ -185,6 +194,10 @@ gzochi_metad_root_context_class_init (GzochiMetadRootContextClass *klass)
     ("session-server", "session-server", "Test sessionserver object",
      G_TYPE_OBJECT, G_PARAM_READABLE);
 
+  obj_properties[PROP_TASK_SERVER] = g_param_spec_object
+    ("task-server", "task-server", "Test taskserver object", G_TYPE_OBJECT,
+     G_PARAM_READABLE);
+  
   obj_properties[PROP_EVENT_SOURCE] = g_param_spec_boxed
     ("event-source", "event-source", "Test event source", G_TYPE_SOURCE,
      G_PARAM_READABLE);
@@ -203,6 +216,7 @@ gzochi_metad_root_context_init (GzochiMetadRootContext *self)
   self->channelserver = g_object_new (G_TYPE_OBJECT, NULL);
   self->dataserver = g_object_new (G_TYPE_OBJECT, NULL);
   self->sessionserver = g_object_new (G_TYPE_OBJECT, NULL);
+  self->taskserver = g_object_new (G_TYPE_OBJECT, NULL);
   self->event_source = gzochid_event_source_new ();
 }
 
@@ -297,6 +311,33 @@ gzochid_client_protocol gzochi_metad_sessionserver_client_protocol =
     test_free
   };
 
+static GList *taskserver_messages = NULL;
+static gboolean taskserver_connected_flag = FALSE;
+static gboolean taskserver_error_flag = FALSE;
+
+static unsigned int
+taskserver_dispatch (const GByteArray *buffer, gpointer user_data)
+{
+  taskserver_messages = g_list_append
+    (taskserver_messages, g_bytes_new (buffer->data, buffer->len));
+  
+  return 1;
+}
+
+static void
+taskserver_error (gpointer user_data)
+{
+  taskserver_error_flag = TRUE;
+}
+
+gzochid_client_protocol gzochi_metad_taskserver_client_protocol =
+  {
+    test_can_dispatch,
+    taskserver_dispatch,
+    taskserver_error,
+    test_free
+  };
+
 struct _metaserver_protocol_fixture
 {
   GzochiMetadRootContext *root_context;
@@ -349,6 +390,22 @@ gzochi_metad_sessionserver_server_connected
   sessionserver_connected_flag = TRUE;
 }
 
+gzochi_metad_taskserver_client *
+gzochi_metad_taskserver_client_new (GzochiMetadTaskServer *taskserver,
+				    gzochid_client_socket *socket,
+				    unsigned int node_id)
+{
+  return NULL;
+}
+
+void
+gzochi_metad_taskserver_server_connected
+(GzochiMetadTaskServer *taskserver, int node_id, gzochid_client_socket *socket,
+ GError **err)
+{
+  taskserver_connected_flag = TRUE;
+}
+
 static gzochid_client_socket *
 server_accept_wrapper (GIOChannel *channel, const char *desc, gpointer data)
 {
@@ -418,6 +475,12 @@ metaserver_protocol_fixture_tear_down (metaserver_protocol_fixture *fixture,
 
   sessionserver_connected_flag = FALSE;
   sessionserver_error_flag = FALSE;
+
+  g_list_free_full (taskserver_messages, (GDestroyNotify) g_bytes_unref);
+  taskserver_messages = NULL;
+
+  taskserver_connected_flag = FALSE;
+  taskserver_error_flag = FALSE;
 }
 
 static void
@@ -550,6 +613,7 @@ test_client_dispatch_one_login (metaserver_protocol_fixture *fixture,
   g_assert (memcmp (buf, "\x00\x18\x11\x02http://localhost:8081/", 27) == 0);
   g_assert (channelserver_connected_flag);
   g_assert (sessionserver_connected_flag);
+  g_assert (taskserver_connected_flag);
 }
 
 static void
@@ -584,6 +648,7 @@ test_client_dispatch_one_login_base_url (metaserver_protocol_fixture *fixture,
   g_assert(memcmp (buf, "\x00\x18\x11\x02http://localhost:8081/", 27) == 0);
   g_assert (channelserver_connected_flag);
   g_assert (sessionserver_connected_flag);
+  g_assert (taskserver_connected_flag);
 }
 
 static void
@@ -650,6 +715,26 @@ test_client_dispatch_one_session (metaserver_protocol_fixture *fixture,
   g_bytes_unref (expected_bytes);
 }
 
+static void
+test_client_dispatch_one_task (metaserver_protocol_fixture *fixture,
+			       gconstpointer user_data)
+{
+  gzochi_metad_metaserver_client *client =
+    _gzochid_client_socket_get_protocol_data (fixture->client_socket);
+
+  GBytes *expected_bytes = g_bytes_new_static ("\x00\x04\x80""foo\x00", 7);
+  GByteArray *input_bytes = g_byte_array_new ();
+
+  g_byte_array_append (input_bytes, "\x00\x04\x80""foo\x00", 7);
+  gzochi_metad_metaserver_client_protocol.dispatch (input_bytes, client);  
+  g_byte_array_unref (input_bytes);
+  
+  g_assert (g_list_find_custom
+	    (taskserver_messages, expected_bytes, g_bytes_compare) != NULL);
+
+  g_bytes_unref (expected_bytes);
+}
+
 static void
 handle_client_disconnected (GzochidEvent *event, gpointer user_data)
 {
@@ -705,6 +790,7 @@ test_client_error (metaserver_protocol_fixture *fixture,
   g_assert (channelserver_error_flag);
   g_assert (dataserver_error_flag);
   g_assert (sessionserver_error_flag);
+  g_assert (taskserver_error_flag);
   
   g_mutex_clear (&callback_data.mutex);
   g_cond_clear (&callback_data.cond);
@@ -754,6 +840,9 @@ main (int argc, char *argv[])
     ("/client/dispatch/one/session", metaserver_protocol_fixture, NULL,
      metaserver_protocol_fixture_set_up, test_client_dispatch_one_session,
      metaserver_protocol_fixture_tear_down);
+  g_test_add ("/client/dispatch/one/task", metaserver_protocol_fixture, NULL,
+	      metaserver_protocol_fixture_set_up, test_client_dispatch_one_task,
+	      metaserver_protocol_fixture_tear_down);
   
   g_test_add ("/client/error", metaserver_protocol_fixture, NULL,
 	      metaserver_protocol_fixture_set_up, test_client_error,
