@@ -1,5 +1,5 @@
 /* test-oids-storage.c: Test routines for oids.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "mock-storage-engine.h"
 #include "oids.h"
 #include "oids-storage.h"
 #include "storage-mem.h"
@@ -39,20 +40,18 @@ typedef struct _test_oids_storage test_oids_storage;
 static void
 setup_storage (test_oids_storage *fixture, gconstpointer user_data)
 {
-  fixture->interface = &gzochid_storage_engine_interface_mem;
-  fixture->context = fixture->interface->initialize ("test");
-  fixture->meta = fixture->interface->open (fixture->context, "meta", 0);
-
+  fixture->interface = &gzochid_mock_storage_engine_interface;
   fixture->strategy = gzochid_storage_oid_strategy_new
-    (fixture->interface, fixture->context, fixture->meta);
+    (fixture->interface, "test");
+
+  fixture->context = gzochid_mock_storage_engine_get_context ("test");
+  fixture->meta = gzochid_mock_storage_engine_get_store
+    (fixture->context, "test/meta");
 }
 
 static void
 teardown_storage (test_oids_storage *fixture, gconstpointer user_data)
 {
-  fixture->interface->close_store (fixture->meta);
-  fixture->interface->close_context (fixture->context);
-
   gzochid_oid_allocation_strategy_free (fixture->strategy);
 }
 
@@ -122,6 +121,7 @@ int
 main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
+  gzochid_mock_storage_engine_initialize ();
 
   g_test_add
     ("/oids-storage/reserve/first", test_oids_storage, NULL, setup_storage,
