@@ -1,5 +1,5 @@
 /* test-scheme-task.c: Test routines for scheme-task.c in gzochid.
- * Copyright (C) 2017 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,7 +22,10 @@
 #include <stdlib.h>
 
 #include "app.h"
+#include "app-factory.h"
+#include "app-store.h"
 #include "app-task.h"
+#include "callback.h"
 #include "config.h"
 #include "descriptor.h"
 #include "game.h"
@@ -43,9 +46,8 @@ struct test_scheme_task_fixture
   GzochidResolutionContext *resolution_context;
   GzochidGameServer *game_server;
   
-  gzochid_application_context *app_context;
+  GzochidApplicationContext *app_context;
   gzochid_auth_identity *identity;
-  gzochid_storage_engine_interface *storage_interface;
 };
 
 static gboolean
@@ -62,69 +64,30 @@ static void
 test_scheme_task_fixture_setup (struct test_scheme_task_fixture *fixture,
 				gconstpointer user_data)
 {
-  GKeyFile *key_file = g_key_file_new ();
-  GzochidConfiguration *configuration = g_object_new
-    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
-
-  fixture->app_context = gzochid_application_context_new ();
-  fixture->app_context->deployment_root = "/tmp";
-  fixture->app_context->descriptor = g_object_new
-    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
-  fixture->app_context->identity_cache = gzochid_auth_identity_cache_new ();
-
-  fixture->identity = gzochid_auth_identity_new ("[TEST]");
-
-  fixture->resolution_context = g_object_new
+  GzochidResolutionContext *context = g_object_new
     (GZOCHID_TYPE_RESOLUTION_CONTEXT, NULL);
-
-  gzochid_resolver_provide
-    (fixture->resolution_context, G_OBJECT (configuration), NULL);
-
+  GKeyFile *key_file = g_key_file_new ();
+  GzochidConfiguration *config = g_object_new
+    (GZOCHID_TYPE_CONFIGURATION, "key_file", key_file, NULL);
+  
+  gzochid_resolver_provide (context, G_OBJECT (config), NULL);
+  
+  fixture->app_context = gzochid_create_test_application_with_context (context);
+  fixture->identity = gzochid_auth_identity_new ("[TEST]");
   fixture->game_server = gzochid_resolver_require_full
-    (fixture->resolution_context, GZOCHID_TYPE_GAME_SERVER, NULL);
-  
-  fixture->storage_interface = &gzochid_storage_engine_interface_mem;
-  
-  fixture->app_context->storage_engine_interface = fixture->storage_interface;
-  fixture->app_context->storage_context =
-    fixture->storage_interface->initialize ("/tmp");
-  fixture->app_context->meta = fixture->storage_interface->open
-    (fixture->app_context->storage_context, "/tmp/meta",
-     GZOCHID_STORAGE_CREATE);
-  fixture->app_context->oids = fixture->storage_interface->open
-    (fixture->app_context->storage_context, "/tmp/oids",
-     GZOCHID_STORAGE_CREATE);
-  fixture->app_context->names = fixture->storage_interface->open
-    (fixture->app_context->storage_context, "/tmp/names",
-     GZOCHID_STORAGE_CREATE);
-
-  fixture->app_context->oid_strategy = gzochid_storage_oid_strategy_new
-    (fixture->storage_interface, fixture->app_context->storage_context,
-     fixture->app_context->meta);
+    (context, GZOCHID_TYPE_GAME_SERVER, NULL);  
 
+  g_object_unref (config);
   g_key_file_unref (key_file);
-  g_object_unref (configuration);
+  g_object_unref (context);
 }
 
 static void
 test_scheme_task_fixture_teardown (struct test_scheme_task_fixture *fixture,
 				   gconstpointer user_data)
 {
-  g_object_unref (fixture->app_context->descriptor);
-  gzochid_auth_identity_cache_destroy (fixture->app_context->identity_cache);
-
-  fixture->storage_interface->close_store (fixture->app_context->meta);
-  fixture->storage_interface->close_store (fixture->app_context->oids);
-  fixture->storage_interface->close_store (fixture->app_context->names);
-  fixture->storage_interface->close_context
-    (fixture->app_context->storage_context);
-
-  gzochid_oid_allocation_strategy_free (fixture->app_context->oid_strategy);
-
+  g_object_unref (fixture->app_context);
   g_object_unref (fixture->game_server);
-  g_object_unref (fixture->resolution_context);
-
-  gzochid_application_context_free (fixture->app_context);
   gzochid_auth_identity_unref (fixture->identity);
 }
 
@@ -132,21 +95,24 @@ static void
 persist_client_session (struct test_scheme_task_fixture *fixture,
 			gzochid_client_session *session)
 {
+  gzochid_application_store *app_store = NULL;
   GByteArray *out = g_byte_array_new ();
+  gzochid_storage_transaction *tx = NULL;
 
-  gzochid_storage_transaction *tx = fixture->storage_interface
-    ->transaction_begin (fixture->app_context->storage_context);
+  g_object_get (fixture->app_context, "app-store", &app_store, NULL);
+
+  tx = app_store->iface->transaction_begin (app_store->storage_context);
   
   gzochid_client_session_serialization.serializer 
     (fixture->app_context, session, out, NULL);
   
-  fixture->storage_interface->transaction_put
-    (tx, fixture->app_context->oids, "1", 2, out->data, out->len);
-  fixture->storage_interface->transaction_put
-    (tx, fixture->app_context->names, "s.session.1", 12, "1", 2);
+  app_store->iface->transaction_put
+    (tx, app_store->oids, "1", 2, out->data, out->len);
+  app_store->iface->transaction_put
+    (tx, app_store->names, "s.session.1", 12, "1", 2);
 
-  fixture->storage_interface->transaction_prepare (tx);
-  fixture->storage_interface->transaction_commit (tx);
+  app_store->iface->transaction_prepare (tx);
+  app_store->iface->transaction_commit (tx);
 
   g_byte_array_unref (out);
 }
@@ -167,6 +133,7 @@ test_disconnected_worker_no_handler (struct test_scheme_task_fixture *fixture,
 				     gconstpointer user_data)
 {
   guint64 one = gzochid_util_encode_oid (1);
+  gzochid_application_store *app_store = NULL;
   gzochid_storage_transaction *tx = NULL;
   gzochid_client_session *session =
     gzochid_client_session_new (fixture->identity);
@@ -176,15 +143,15 @@ test_disconnected_worker_no_handler (struct test_scheme_task_fixture *fixture,
   gzochid_transaction_execute
     (test_disconnected_worker_no_handler_inner, fixture);
 
-  tx = fixture->storage_interface->transaction_begin
-    (fixture->app_context->storage_context);
+  g_object_get (fixture->app_context, "app-store", &app_store, NULL);
+  
+  tx = app_store->iface->transaction_begin (app_store->storage_context);
 
   g_assert
-    (fixture->storage_interface->transaction_get
-     (tx, fixture->app_context->oids, (char *) &one, sizeof (guint64), NULL) ==
-     NULL);
+    (app_store->iface->transaction_get
+     (tx, app_store->oids, (char *) &one, sizeof (guint64), NULL) == NULL);
 
-  fixture->storage_interface->transaction_rollback (tx);
+  app_store->iface->transaction_rollback (tx);
 
   gzochid_client_session_free (session);
 }
@@ -212,16 +179,23 @@ static void
 test_logged_in_worker_throws_exception_inner (gpointer data)
 {
   struct test_scheme_task_fixture *fixture = data;
+  GzochidApplicationDescriptor *descriptor = NULL;
   
   SCM module = scm_c_resolve_module ("test");
   SCM logged_in = scm_c_make_gsubr 
     ("logged-in-exception", 1, 0, 0, raise_condition);
   GList *test_module = g_list_append (NULL, "test");
-
+  
   scm_c_module_define (module, "logged-in-exception", logged_in);
 
-  fixture->app_context->descriptor->logged_in = make_callback 
-    ("logged-in-exception", test_module);
+  /* Replace the default logged-in callback. */
+  
+  g_object_get (fixture->app_context, "descriptor", &descriptor, NULL);
+  if (descriptor->logged_in != NULL)
+    gzochid_application_callback_free (descriptor->logged_in);
+  
+  descriptor->logged_in = make_callback ("logged-in-exception", test_module);
+  g_object_unref (descriptor);
   
   gzochid_scheme_application_logged_in_worker
     (fixture->app_context, fixture->identity, "1");
@@ -248,7 +222,8 @@ static void
 test_logged_in_worker_returns_unspecified_inner (gpointer data)
 {
   struct test_scheme_task_fixture *fixture = data;
-    
+  GzochidApplicationDescriptor *descriptor = NULL;
+  
   SCM module = scm_c_resolve_module ("test");
   SCM logged_in = scm_c_make_gsubr 
     ("logged-in-unspecified", 1, 0, 0, return_unspecified);
@@ -256,8 +231,14 @@ test_logged_in_worker_returns_unspecified_inner (gpointer data)
 
   scm_c_module_define (module, "logged-in-unspecified", logged_in);
 
-  fixture->app_context->descriptor->logged_in = make_callback 
-    ("logged-in-unspecified", test_module);
+  /* Replace the default logged-in callback. */
+
+  g_object_get (fixture->app_context, "descriptor", &descriptor, NULL);
+  if (descriptor->logged_in != NULL)
+    gzochid_application_callback_free (descriptor->logged_in);
+  
+  descriptor->logged_in = make_callback ("logged-in-unspecified", test_module);
+  g_object_unref (descriptor);
 
   gzochid_scheme_application_logged_in_worker
     (fixture->app_context, fixture->identity, "1");
@@ -282,10 +263,9 @@ test_logged_in_worker_returns_unspecified
 static void 
 test_ready_throws_exception ()
 {
-  gzochid_application_context *context = gzochid_application_context_new ();
-  GzochidApplicationDescriptor *descriptor = g_object_new
-    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
+  GzochidApplicationContext *context = gzochid_create_test_application ();
   gzochid_auth_identity *identity = gzochid_auth_identity_new ("[TEST]");
+  GzochidApplicationDescriptor *descriptor = NULL;
   
   SCM module = scm_c_resolve_module ("test");
   SCM ready = scm_c_make_gsubr ("ready", 1, 0, 0, raise_condition);
@@ -295,11 +275,10 @@ test_ready_throws_exception ()
 
   scm_c_module_define (module, "ready", ready);
 
+  g_object_get (context, "descriptor", &descriptor, NULL);
   descriptor->ready = make_callback ("ready", test_module);
-
-  context->deployment_root = "/";
-  context->descriptor = descriptor;
-
+  g_object_unref (descriptor);
+  
   gzochid_scheme_application_ready (context, identity, &tmp_err);
 
   g_assert_error (tmp_err, GZOCHID_SCHEME_ERROR, GZOCHID_SCHEME_ERROR_FAILED);
@@ -307,8 +286,7 @@ test_ready_throws_exception ()
   g_error_free (tmp_err);
   g_list_free (test_module);
   gzochid_auth_identity_unref (identity);
-  g_object_unref (descriptor);
-  gzochid_application_context_free (context);
+  g_object_unref (context);
 }
 
 void gzochid_api_channel_init () { }
