@@ -449,28 +449,6 @@ maybe_bytes_unref (gpointer data)
     g_bytes_unref (data);
 }
 
-static void
-clear_changeset (gpointer data)
-{
-  GArray *changes = data;
-  int i = 0;
-
-  for (; i < changes->len; i++)
-    {
-      gzochid_data_change *change = &g_array_index
-	(changes, gzochid_data_change, i);
-
-      free (change->store);
-
-      g_bytes_unref (change->key);
-
-      if (change->data != NULL)
-	g_bytes_unref (change->data);
-    }
-  
-  g_array_unref (changes);
-}
-
 static void
 dataclient_storage_fixture_teardown (dataclient_storage_fixture *fixture,
 				     gconstpointer user_data)
@@ -481,7 +459,8 @@ dataclient_storage_fixture_teardown (dataclient_storage_fixture *fixture,
   g_list_free_full
     (fixture->dataclient->deferred_response_closures, execute_deferred_closure);
   g_list_free_full (fixture->dataclient->requested_keys, g_free);
-  g_list_free_full (fixture->dataclient->changesets, clear_changeset);
+  g_list_free_full
+    (fixture->dataclient->changesets, (GDestroyNotify) g_array_unref);
   g_list_free_full
     (fixture->dataclient->release_closures, execute_release_closure);
   g_list_free_full (fixture->dataclient->released_keys, g_free);
@@ -1404,6 +1383,39 @@ test_range_lock_release_eviction (dataclient_storage_fixture *fixture,
   g_bytes_unref (key_bytes);
 }
 
+static void
+test_compaction_simple (dataclient_storage_fixture *fixture,
+			gconstpointer user_data)
+{
+  dataclient_storage_response *response = create_success_response (NULL);
+  gzochid_storage_transaction *tx = NULL;
+
+  GArray *changeset = NULL;
+  gzochid_data_change *change = NULL;
+  
+  fixture->dataclient->responses = g_list_append
+    (fixture->dataclient->responses, response);
+  
+  tx = fixture->iface->transaction_begin (fixture->storage_context);
+  
+  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar1", 5);
+  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar2", 5);
+  fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "bar3", 5);
+    
+  fixture->iface->transaction_prepare (tx);
+  fixture->iface->transaction_commit (tx);
+
+  g_assert_cmpint (g_list_length (fixture->dataclient->changesets), ==, 1);
+
+  changeset = fixture->dataclient->changesets->data;
+  g_assert_cmpint (changeset->len, ==, 1);
+
+  change = &g_array_index (changeset, gzochid_data_change, 0);
+  g_assert_cmpstr ("bar3", ==, g_bytes_get_data (change->data, NULL));
+  
+  free_response (response);  
+}
+
 static void
 test_cache_store_consistency (dataclient_storage_fixture *fixture,
 			      gconstpointer user_data)
@@ -1627,6 +1639,11 @@ main (int argc, char *argv[])
      dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
      test_range_lock_release_eviction, dataclient_storage_fixture_teardown);
 
+  g_test_add
+    ("/storage-dataclient/compaction/simple",
+     dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
+     test_compaction_simple, dataclient_storage_fixture_teardown);
+  
   g_test_add
     ("/storage-dataclient/consistency/cache-store",
      dataclient_storage_fixture, NULL, dataclient_storage_fixture_setup,
