@@ -1,5 +1,5 @@
 /* test-task.c: Test routines for task.c in gzochid.
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -32,11 +32,11 @@ test_immediate_new ()
 {
   struct timeval t;
   gboolean data = FALSE;
-  gzochid_task *task = NULL;
+  GzochidTask *task = NULL;
 
   gettimeofday (&t, NULL);
 
-  task = gzochid_task_immediate_new (test_thread_worker, &data);
+  task = gzochid_task_immediate_new (test_thread_worker, NULL, &data);
   
   g_assert (task->worker == test_thread_worker);
   g_assert (task->data == &data);
@@ -44,7 +44,7 @@ test_immediate_new ()
   /* Workaround for platforms with broken `timercmp'. */
   
   g_assert (! timercmp (&task->target_execution_time, &t, <));
-  gzochid_task_free (task);
+  g_object_unref (task);
 }
 
 static void
@@ -52,11 +52,11 @@ test_new ()
 {
   struct timeval t;
   gboolean data = FALSE;
-  gzochid_task *task = NULL;
+  GzochidTask *task = NULL;
 
   gettimeofday (&t, NULL);
 
-  task = gzochid_task_new (test_thread_worker, &data, t);
+  task = gzochid_task_new (test_thread_worker, NULL, &data, t);
 
   g_assert (task->worker == test_thread_worker);
   g_assert (task->data == &data);
@@ -64,7 +64,7 @@ test_new ()
   /* Workaround for platforms with broken `timercmp'. */
 
   g_assert (! timercmp (&task->target_execution_time, &t, !=));
-  gzochid_task_free (task);
+  g_object_unref (task);
 }
 
 int
