@@ -1,5 +1,5 @@
 /* test-util.c: Test routines for util.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2018 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -372,6 +372,13 @@ test_util_deserialize_sequence ()
   g_byte_array_unref (in);
 }
 
+static void
+free_key_value_ghfunc (gpointer key, gpointer value, gpointer user_data)
+{
+  free (key);
+  free (value);
+}
+
 static void
 test_util_deserialize_hash_table ()
 {
@@ -392,6 +399,8 @@ test_util_deserialize_hash_table ()
   g_assert (g_hash_table_contains (ht, "baz"));
   g_assert_cmpstr (g_hash_table_lookup (ht, "baz"), ==, "qux");
 
+  g_hash_table_foreach (ht, free_key_value_ghfunc, NULL);
+  
   g_hash_table_destroy (ht);
   g_byte_array_unref (in);
 }
@@ -411,6 +420,17 @@ test_util_deserialize_timeval ()
   g_byte_array_unref (in);
 }
 
+static void
+test_util_guint64_compare ()
+{
+  guint64 zero = 0;
+  guint64 one = 1;
+
+  g_assert_cmpint (gzochid_util_guint64_compare (&zero, &one), <, 0);
+  g_assert_cmpint (gzochid_util_guint64_compare (&one, &one), ==, 0);
+  g_assert_cmpint (gzochid_util_guint64_compare (&one, &zero), >, 0);
+}
+
 static void
 test_util_guint64_data_compare ()
 {
@@ -618,6 +638,7 @@ main (int argc, char *argv[])
     ("/util/deserialize/hash_table", test_util_deserialize_hash_table);
   g_test_add_func ("/util/deserialize/timeval", test_util_deserialize_timeval);
 
+  g_test_add_func ("/util/guint64_compare/simple", test_util_guint64_compare);
   g_test_add_func
     ("/util/guint64_data_compare/simple", test_util_guint64_data_compare);
   g_test_add_func
